@@ -31,6 +31,7 @@
 #include "fileIO.h"
 #include "fileLock.h"
 #include "unicodeTypes.h"
+#include "memaligned.h"
 
 #if defined __linux__
 /*
@@ -100,34 +101,34 @@ typedef struct FileData {
 } FileData;
 
 #if defined(_WIN32)
-EXTERN int FileMapErrorToErrno(char *functionName,
-                               DWORD status);
+int FileMapErrorToErrno(char *functionName,
+                        DWORD status);
 
-EXTERN Bool FileRetryThisError(DWORD error,
-                               uint32 numCodes,
-                               DWORD *codes);
+Bool FileRetryThisError(DWORD error,
+                        uint32 numCodes,
+                        DWORD *codes);
 
-EXTERN int FileAttributesRetry(ConstUnicode pathName,
-                               uint32 msecMaxWaitTime,
-                               FileData *fileData);
+int FileAttributesRetry(ConstUnicode pathName,
+                        uint32 msecMaxWaitTime,
+                        FileData *fileData);
 
-EXTERN int FileRenameRetry(ConstUnicode fromPath,
-                           ConstUnicode toPath,
-                           uint32 msecMaxWaitTime);
+int FileRenameRetry(ConstUnicode fromPath,
+                    ConstUnicode toPath,
+                    uint32 msecMaxWaitTime);
 
-EXTERN int FileDeletionRetry(ConstUnicode pathName,
-                             Bool handleLink,
+int FileDeletionRetry(ConstUnicode pathName,
+                      Bool handleLink,
+                      uint32 msecMaxWaitTime);
+
+int FileCreateDirectoryRetry(ConstUnicode pathName,
                              uint32 msecMaxWaitTime);
 
-EXTERN int FileCreateDirectoryRetry(ConstUnicode pathName,
-                                    uint32 msecMaxWaitTime);
+int FileRemoveDirectoryRetry(ConstUnicode pathName,
+                             uint32 msecMaxWaitTime);
 
-EXTERN int FileRemoveDirectoryRetry(ConstUnicode pathName,
-                                    uint32 msecMaxWaitTime);
-
-EXTERN int FileListDirectoryRetry(ConstUnicode pathName,
-                                  uint32 msecMaxWaitTime,
-                                  Unicode **ids);
+int FileListDirectoryRetry(ConstUnicode pathName,
+                           uint32 msecMaxWaitTime,
+                           Unicode **ids);
 
 #define FileAttributes(a, b)    FileAttributesRetry((a), 0, (b))
 #define FileRename(a, b)        FileRenameRetry((a), (b), 0)
@@ -150,20 +151,20 @@ EXTERN int FileListDirectoryRetry(ConstUnicode pathName,
 #define FileRemoveDirectoryRobust(a) \
                     FileRemoveDirectoryRetry((a), FILE_MAX_WAIT_TIME_MS)
 #else
-EXTERN char *FilePosixGetBlockDevice(char const *path);
+char *FilePosixGetBlockDevice(char const *path);
 
-EXTERN int FileAttributes(ConstUnicode pathName,
-                          FileData *fileData);
+int FileAttributes(ConstUnicode pathName,
+                   FileData *fileData);
 
-EXTERN int FileRename(ConstUnicode fromPath,
-                      ConstUnicode toPath);
+int FileRename(ConstUnicode fromPath,
+               ConstUnicode toPath);
 
-EXTERN int FileDeletion(ConstUnicode pathName,
-                        Bool handleLink);
+int FileDeletion(ConstUnicode pathName,
+                 Bool handleLink);
 
-EXTERN int FileCreateDirectory(ConstUnicode pathName);
+int FileCreateDirectory(ConstUnicode pathName);
 
-EXTERN int FileRemoveDirectory(ConstUnicode pathName);
+int FileRemoveDirectory(ConstUnicode pathName);
 
 #define FileListDirectoryRobust(a, b) File_ListDirectory((a), (b))
 #define FileAttributesRobust(a, b)    FileAttributes((a), (b))
@@ -207,64 +208,116 @@ typedef HANDLE FILELOCK_FILE_HANDLE;
 typedef int FILELOCK_FILE_HANDLE;
 #endif
 
-EXTERN uint32 FileSleeper(uint32 msecMinSleepTime,
-                          uint32 msecMaxSleepTime);
+uint32 FileSleeper(uint32 msecMinSleepTime,
+                   uint32 msecMaxSleepTime);
 
-EXTERN uint32 FileSimpleRandom(void);
+uint32 FileSimpleRandom(void);
 
-EXTERN const char *FileLockGetMachineID(void);
+const char *FileLockGetMachineID(void);
 
-EXTERN char *FileLockGetExecutionID(void);
+char *FileLockGetExecutionID(void);
 
-EXTERN Bool FileLockMachineIDMatch(char *host,
-                                   char *second);
+Bool FileLockMachineIDMatch(char *host,
+                            char *second);
 
-EXTERN int FileLockMemberValues(ConstUnicode lockDir, 
-                                ConstUnicode fileName,
-                                char *buffer,
-                                uint32 size,
-                                LockValues *memberValues);
+int FileLockMemberValues(ConstUnicode lockDir, 
+                         ConstUnicode fileName,
+                         char *buffer,
+                         uint32 size,
+                         LockValues *memberValues);
 
-EXTERN int FileLockHackVMX(ConstUnicode filePathName);
+int FileLockHackVMX(ConstUnicode filePathName);
 
-EXTERN int FileLockOpenFile(ConstUnicode pathName,
-                            int flags,
-                            FILELOCK_FILE_HANDLE *handle);
+int FileLockOpenFile(ConstUnicode pathName,
+                     int flags,
+                     FILELOCK_FILE_HANDLE *handle);
 
-EXTERN int FileLockCloseFile(FILELOCK_FILE_HANDLE handle);
+int FileLockCloseFile(FILELOCK_FILE_HANDLE handle);
 
-EXTERN int FileLockReadFile(FILELOCK_FILE_HANDLE handle,
-                            void *buf,
-                            uint32 requestedBytes,
-                            uint32 *resultantBytes);
+int FileLockReadFile(FILELOCK_FILE_HANDLE handle,
+                     void *buf,
+                     uint32 requestedBytes,
+                     uint32 *resultantBytes);
 
-EXTERN int FileLockWriteFile(FILELOCK_FILE_HANDLE handle,
-                             void *buf,
-                             uint32 requestedBytes,
-                             uint32 *resultantBytes);
+int FileLockWriteFile(FILELOCK_FILE_HANDLE handle,
+                      void *buf,
+                      uint32 requestedBytes,
+                      uint32 *resultantBytes);
 
-EXTERN FileLockToken *FileLockIntrinsic(ConstUnicode filePathName,
-                                        Bool exclusivity,
-                                        uint32 msecMaxWaitTime,
-                                        const char *payload,
-                                        int *err);
+FileLockToken *FileLockIntrinsic(ConstUnicode filePathName,
+                                 Bool exclusivity,
+                                 uint32 msecMaxWaitTime,
+                                 const char *payload,
+                                 int *err);
 
-EXTERN int FileUnlockIntrinsic(FileLockToken *tokenPtr);
+int FileUnlockIntrinsic(FileLockToken *tokenPtr);
 
-EXTERN Bool FileLockIsLocked(ConstUnicode filePath,
-                             int *err);
+Bool FileLockIsLocked(ConstUnicode filePath,
+                      int *err);
 
-EXTERN Bool FileLockValidOwner(const char *executionID,
-                               const char *payload);
+Bool FileLockValidOwner(const char *executionID,
+                        const char *payload);
 
-EXTERN Bool FileLockValidName(ConstUnicode fileName);
+Bool FileLockValidName(ConstUnicode fileName);
 
-EXTERN Bool FileIsWritableDir(ConstUnicode dirName);
+Bool FileIsWritableDir(ConstUnicode dirName);
+
+
+/*
+ * FileIOAligned_* are useful on hosted platforms where malloc/memalign/valloc
+ * for "large buffers" (in the range 64 KiB - 1 MiB) will generally fall
+ * through to mmap/munmap, which is expensive due to page table modifications
+ * and the attendant TLB flushing (which requires IPIs and possibly world
+ * switches) on other threads running in the same address space.  In particular,
+ * on Mac OS 10.6.6 on a Westmere-class Mac Pro, mmap + memcpy + munmap adds
+ * around a millisecond of CPU time and a hundred IPIs to a 512 KiB write.  See
+ * PR#685845.
+ *
+ * This isn't applicable to ESX because
+ * 1. we don't use this path for disk IO
+ * 2. we don't want to do extra large allocations
+ * 3. we don't have the same alignment constraints
+ * so simply define it away to nothing.
+ *
+ * Tools is another case, we can use this path for IO but we don't want to add
+ * MXUserExclLock dependencies.
+ */
+
+#if defined(VMX86_TOOLS) || defined(VMX86_SERVER)
+#define FileIOAligned_PoolInit()     /* nothing */
+#define FileIOAligned_PoolExit()     /* nothing */
+#define FileIOAligned_PoolMalloc(sz) NULL
+#define FileIOAligned_PoolFree(ptr)  FALSE
+#else
+void FileIOAligned_PoolInit(void);
+void FileIOAligned_PoolExit(void);
+void *FileIOAligned_PoolMalloc(size_t);
+Bool FileIOAligned_PoolFree(void *);
+#endif
+
+static INLINE void *
+FileIOAligned_Malloc(size_t sz)  // IN:
+{
+   void *buf = FileIOAligned_PoolMalloc(sz);
+
+   if (!buf) {
+      buf = Aligned_Malloc(sz);
+   }
+   return buf;
+}
+
+static INLINE void
+FileIOAligned_Free(void *ptr)  // IN:
+{
+   if (!FileIOAligned_PoolFree(ptr)) {
+      Aligned_Free(ptr);
+   }
+}
 
 #if defined(__APPLE__)
-EXTERN int PosixFileOpener(ConstUnicode pathName,
-                           int flags,
-                           mode_t mode);
+int PosixFileOpener(ConstUnicode pathName,
+                    int flags,
+                    mode_t mode);
 #else
 #define PosixFileOpener(a, b, c) Posix_Open(a, b, c);
 #endif
