@@ -185,7 +185,7 @@ RemoveLockingFile(ConstUnicode lockDir,   // IN:
          err = 0;
       } else {
          Warning(LGPFX" %s of '%s' failed: %s\n", __FUNCTION__,
-                 UTF8(path), Err_Errno2String(err));
+                 UTF8(path), strerror(err));
       }
    }
 
@@ -318,7 +318,7 @@ FileLockMemberValues(ConstUnicode lockDir,      // IN:
 
       if (err != ENOENT) {
          Warning(LGPFX" %s open failure on '%s': %s\n", __FUNCTION__,
-                 UTF8(path), Err_Errno2String(err));
+                 UTF8(path), strerror(err));
       }
 
       goto bail;
@@ -337,7 +337,7 @@ FileLockMemberValues(ConstUnicode lockDir,      // IN:
 
       if (err != ENOENT) {
          Warning(LGPFX" %s file size failure on '%s': %s\n", __FUNCTION__,
-                 UTF8(path), Err_Errno2String(err));
+                 UTF8(path), strerror(err));
       }
 
       FileLockCloseFile(handle);
@@ -362,7 +362,7 @@ FileLockMemberValues(ConstUnicode lockDir,      // IN:
 
    if (err != 0) {
       Warning(LGPFX" %s read failure on '%s': %s\n",
-              __FUNCTION__, UTF8(path), Err_Errno2String(err));
+              __FUNCTION__, UTF8(path), strerror(err));
 
       goto bail;
    }
@@ -977,7 +977,7 @@ FileUnlockIntrinsic(FileLockToken *tokenPtr)  // IN:
 
       if (err && vmx86_debug) {
          Log(LGPFX" %s failed for '%s': %s\n", __FUNCTION__,
-             UTF8(tokenPtr->lockFilePath), Err_Errno2String(err));
+             UTF8(tokenPtr->lockFilePath), strerror(err));
       }
 
       /*
@@ -1253,13 +1253,13 @@ CreateEntryDirectory(ConstUnicode lockDir,     // IN:
 
             if ((err != 0) && (err != EEXIST)) {
                Warning(LGPFX" %s creation failure on '%s': %s\n",
-                       __FUNCTION__, UTF8(lockDir), Err_Errno2String(err));
+                       __FUNCTION__, UTF8(lockDir), strerror(err));
 
                break;
             }
          } else {
             Warning(LGPFX" %s stat failure on '%s': %s\n",
-                    __FUNCTION__, UTF8(lockDir), Err_Errno2String(err));
+                    __FUNCTION__, UTF8(lockDir), strerror(err));
 
             break;
          }
@@ -1303,7 +1303,7 @@ CreateEntryDirectory(ConstUnicode lockDir,     // IN:
 
             if (vmx86_debug) {
                Log(LGPFX" %s stat failure on '%s': %s\n",
-                   __FUNCTION__, UTF8(*memberFilePath), Err_Errno2String(err));
+                   __FUNCTION__, UTF8(*memberFilePath), strerror(err));
              }
          }
 
@@ -1312,8 +1312,7 @@ CreateEntryDirectory(ConstUnicode lockDir,     // IN:
           if ((err != EEXIST) &&  // Another process/thread created it...
               (err != ENOENT)) {  // lockDir is gone...
              Warning(LGPFX" %s creation failure on '%s': %s\n",
-                     __FUNCTION__, UTF8(*entryDirectory),
-                     Err_Errno2String(err));
+                     __FUNCTION__, UTF8(*entryDirectory), strerror(err));
 
              break;
           }
@@ -1401,7 +1400,7 @@ CreateMemberFile(FILELOCK_FILE_HANDLE entryHandle,  // IN:
 
    if (err != 0) {
       Warning(LGPFX" %s write of '%s' failed: %s\n", __FUNCTION__,
-              UTF8(entryFilePath), Err_Errno2String(err));
+              UTF8(entryFilePath), strerror(err));
 
       FileLockCloseFile(entryHandle);
 
@@ -1412,7 +1411,7 @@ CreateMemberFile(FILELOCK_FILE_HANDLE entryHandle,  // IN:
 
    if (err != 0) {
       Warning(LGPFX" %s close of '%s' failed: %s\n", __FUNCTION__,
-              UTF8(entryFilePath), Err_Errno2String(err));
+              UTF8(entryFilePath), strerror(err));
 
       return err;
    }
@@ -1429,16 +1428,16 @@ CreateMemberFile(FILELOCK_FILE_HANDLE entryHandle,  // IN:
    if (err != 0) {
       Warning(LGPFX" %s FileRename of '%s' to '%s' failed: %s\n",
               __FUNCTION__, UTF8(entryFilePath), UTF8(memberFilePath),
-              Err_Errno2String(err));
+              strerror(err));
 
       if (vmx86_debug) {
          Log(LGPFX" %s FileLockFileType() of '%s': %s\n",
              __FUNCTION__, UTF8(entryFilePath),
-            Err_Errno2String(FileAttributesRobust(entryFilePath, NULL)));
+            strerror(FileAttributesRobust(entryFilePath, NULL)));
 
          Log(LGPFX" %s FileLockFileType() of '%s': %s\n",
              __FUNCTION__, UTF8(memberFilePath),
-            Err_Errno2String(FileAttributesRobust(memberFilePath, NULL)));
+            strerror(FileAttributesRobust(memberFilePath, NULL)));
       }
 
       return err;
@@ -1735,121 +1734,3 @@ FileLock_TokenPathName(const FileLockToken *lockToken)  // IN:
 
    return Unicode_Duplicate(lockToken->pathName);
 }
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * ScannerVMX --
- *
- *      VMX hack scanner
- *
- * Results:
- *      0       success
- *      > 0     error (errno)
- *
- * Side effects:
- *      None.
- *
- *-----------------------------------------------------------------------------
- */
-
-static int
-ScannerVMX(ConstUnicode lockDir,      // IN:
-           ConstUnicode fileName,     // IN:
-           LockValues *memberValues,  // IN:
-           LockValues *myValues)      // IN/OUT:
-{
-   ASSERT(lockDir);
-   ASSERT(fileName);
-
-   myValues->lamportNumber++;
-
-   return 0;
-}
-
-
-/*
- *----------------------------------------------------------------------
- *
- * FileLockHackVMX --
- *
- *      The VMX file delete primitive.
- *
- * Results:
- *      0       unlocked
- *      > 0     errno
- *
- * Side effects:
- *      Changes the host file system.
- *
- * Note:
- *      THIS IS A HORRIBLE HACK AND NEEDS TO BE REMOVED ASAP!!!
- *
- *----------------------------------------------------------------------
- */
-
-int
-FileLockHackVMX(ConstUnicode pathName)  // IN:
-{
-   int err;
-   LockValues myValues;
-
-   Unicode lockDir = NULL;
-   Unicode entryFilePath = NULL;
-   Unicode memberFilePath = NULL;
-   Unicode entryDirectory = NULL;
-
-   ASSERT(pathName);
-
-   /* first the locking directory path name */
-   lockDir = Unicode_Append(pathName, FILELOCK_SUFFIX);
-
-   /* establish our values */
-   myValues.machineID = (char *) FileLockGetMachineID(); // don't free this!
-   myValues.executionID = FileLockGetExecutionID();      // free this!
-   myValues.locationChecksum = FileLockLocationChecksum(lockDir); // free this!
-   myValues.lamportNumber = 0;
-   myValues.memberName = NULL;
-
-   LOG(1, ("%s on %s (%s, %s).\n", __FUNCTION__, UTF8(pathName),
-       myValues.machineID, myValues.executionID));
-
-   err = CreateEntryDirectory(lockDir, &entryDirectory, &entryFilePath,
-                              &memberFilePath, &myValues.memberName);
-
-   if (err != 0) {
-      goto bail;
-   }
-
-   /* Scan the lock directory */
-   err = Scanner(lockDir, ScannerVMX, &myValues, FALSE);
-
-   if (err == 0) {
-      /* if no members are valid, clean up */
-      if (myValues.lamportNumber == 1) {
-         FileDeletionRobust(pathName, FALSE);
-      }
-   } else {
-      if (vmx86_debug) {
-         Warning(LGPFX" %s clean-up failure for '%s': %s\n",
-                 __FUNCTION__, UTF8(pathName), Err_Errno2String(err));
-      }
-   }
-
-   /* clean up */
-   FileRemoveDirectoryRobust(entryDirectory);
-   FileRemoveDirectoryRobust(lockDir);
-
-bail:
-
-   Unicode_Free(lockDir);
-   Unicode_Free(entryDirectory);
-   Unicode_Free(entryFilePath);
-   Unicode_Free(memberFilePath);
-   Unicode_Free(myValues.memberName);
-   free(myValues.locationChecksum);
-   free(myValues.executionID);
-
-   return err;
-}
