@@ -1102,6 +1102,53 @@ bail:
 }
 
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * File_GetVMFSVersion --
+ *
+ *      Get the version number of the VMFS file system on which the
+ *      given file resides.
+ *
+ * Results:
+ *      Integer return value and version number.
+ *
+ * Side effects:
+ *      Will fail if file is not on VMFS or not enough memory for partition
+ *      query results.
+ *
+ *----------------------------------------------------------------------
+ */
+
+int
+File_GetVMFSVersion(ConstUnicode pathName,  // IN: File name to test
+                    uint32 *versionNum)     // OUT: Version number
+{
+   int ret = -1;
+   FS_PartitionListResult *fsAttrs = NULL;
+
+   if (!versionNum) {
+      errno = EINVAL;
+      goto exit;
+   }
+   
+   ret = File_GetVMFSAttributes(pathName, &fsAttrs);
+
+   if (ret < 0) {
+      Log(LGPFX" %s: File_GetVMFSAttributes failed\n", __func__);
+   } else {
+      *versionNum = fsAttrs->versionNumber;
+   }
+
+   if (fsAttrs) {
+      free(fsAttrs);
+   }
+
+exit:
+   return ret;
+}
+
+
 /*
  *----------------------------------------------------------------------
  *
@@ -1123,9 +1170,14 @@ int
 File_GetVMFSBlockSize(ConstUnicode pathName,  // IN: File name to test
                       uint32 *blockSize)      // IN/OUT: VMFS block size
 {
-   int ret;
+   int ret = -1;
    FS_PartitionListResult *fsAttrs = NULL;
 
+   if (!blockSize) {
+      errno = EINVAL;
+      goto exit;
+   }
+
    ret = File_GetVMFSAttributes(pathName, &fsAttrs);
 
    if (ret < 0) {
@@ -1138,6 +1190,7 @@ File_GetVMFSBlockSize(ConstUnicode pathName,  // IN: File name to test
       free(fsAttrs);
    }
 
+exit:
    return ret;
 }
 
@@ -2227,6 +2280,9 @@ FileCreateDirectory(ConstUnicode pathName)  // IN:
  *      The memory allocated for the array may be larger than necessary.
  *      The caller may trim it with realloc() if it cares.
  *
+ *      A file name that cannot be represented in the default encoding
+ *      will appear as a string of three UTF8 sustitution characters.
+ *
  *----------------------------------------------------------------------
  */
 
@@ -2269,7 +2325,24 @@ File_ListDirectory(ConstUnicode pathName,  // IN:
 
       /* Don't create the file list if we aren't providing it to the caller. */
       if (ids) {
-         Unicode id = Unicode_Alloc(entry->d_name, STRING_ENCODING_DEFAULT);
+         Unicode id;
+
+         if (Unicode_IsBufferValid(entry->d_name, -1,
+                                   STRING_ENCODING_DEFAULT)) {
+            id = Unicode_Alloc(entry->d_name, STRING_ENCODING_DEFAULT);
+         } else {
+            id = Unicode_EscapeBuffer(entry->d_name, -1,
+                                     STRING_ENCODING_DEFAULT);
+
+            Warning("%s: file '%s' in directory '%s' cannot be converted to "
+                    "UTF8\n", __FUNCTION__, pathName, id);
+
+            Unicode_Free(id);
+
+            id = Unicode_Duplicate(UNICODE_SUBSTITUTION_CHAR
+                                   UNICODE_SUBSTITUTION_CHAR
+                                   UNICODE_SUBSTITUTION_CHAR);
+         }
 
          DynBuf_Append(&b, &id, sizeof id);
       }
