@@ -231,6 +231,9 @@ HgfsConvertFromInternalStatus(HgfsInternalStatus status) // IN
       return HGFS_STATUS_OPERATION_NOT_SUPPORTED;
    case ERROR_INVALID_PARAMETER:
       return HGFS_STATUS_INVALID_PARAMETER;
+   case ERROR_NOT_SAME_DEVICE:
+      return HGFS_STATUS_NOT_SAME_DEVICE;
+   case ERROR_INTERNAL_ERROR:
    case HGFS_INTERNAL_STATUS_ERROR:
    default:
       return HGFS_STATUS_GENERIC_ERROR;
@@ -272,6 +275,9 @@ HgfsConvertFromInternalStatus(HgfsInternalStatus status) // IN
       return HGFS_STATUS_NAME_TOO_LONG;
    case EPARAMETERNOTSUPPORTED:
       return HGFS_STATUS_INVALID_PARAMETER;
+   case EXDEV:
+      return HGFS_STATUS_NOT_SAME_DEVICE;
+   case EINTERNAL:
    case HGFS_INTERNAL_STATUS_ERROR:
    default:
       return HGFS_STATUS_GENERIC_ERROR;
