@@ -147,6 +147,12 @@ typedef struct HgfsShareInfo {
 
    /* Write permissions for the shared folder, needed for handle => name conversions. */
    Bool writePermissions;
+
+   /*
+    *  Shared folder handle used by change directory notification code to identify
+    *  shared folder.
+    */
+   HgfsSharedFolderHandle handle;
 } HgfsShareInfo;
 
 /*
@@ -278,7 +284,6 @@ typedef enum {
    HGFS_SESSION_STATE_CLOSED,
 } HgfsSessionInfoState;
 
-
 typedef struct HgfsSessionInfo {
    /* Unique session id. */
    uint64 sessionId;
@@ -348,6 +353,13 @@ typedef struct HgfsSessionInfo {
    DblLnkLst_Links searchFreeList;
    /** END SEARCH ARRAY ****************************************************/
 
+   /* Array of session specific capabiities. */
+   HgfsCapability hgfsSessionCapabilities[HGFS_OP_MAX];
+
+   uint32 numberOfCapabilities;
+
+   Bool activeNotification;
+
 } HgfsSessionInfo;
 
 
@@ -438,6 +450,7 @@ typedef struct HgfsInputParam {
    HgfsSessionInfo *session;
    HgfsPacket *packet;
    void const *payload;
+   uint32 payloadOffset;
    size_t payloadSize;
    HgfsOp op;
    uint32 id;
@@ -787,6 +800,50 @@ HgfsPackDestorySessionReply(HgfsPacket *packet,        // IN/OUT: Hgfs Packet
                             char const *packetHeader,  // IN: packet header
                             size_t *payloadSize,       // OUT: size of packet
                             HgfsSessionInfo *session); // IN: Session Info
+void
+HgfsServerGetDefaultCapabilities(HgfsCapability *capabilities,   // OUT:
+                                 uint32 *numberOfCapabilities);  // OUT:
+Bool
+HgfsUnpackSetWatchRequest(void const *packet,      // IN: HGFS packet
+                          size_t packetSize,       // IN: request packet size
+                          HgfsOp op,               // IN: requested operation
+                          Bool *useHandle,         // OUT: handle or cpName
+                          char **cpName,           // OUT: cpName
+                          size_t *cpNameSize,      // OUT: cpName size
+                          uint32 *flags,           // OUT: flags for the new watch
+                          uint32 *events,          // OUT: event filter
+                          HgfsHandle *dir,         // OUT: direrctory handle
+                          uint32 *caseFlags);      // OUT: case-sensitivity flags
+Bool
+HgfsPackSetWatchReply(HgfsPacket *packet,           // IN/OUT: Hgfs Packet
+                      char const *packetHeader,     // IN: packet header
+                      HgfsOp     op,                // IN: operation code
+                      HgfsSubscriberHandle watchId, // IN: new watch id
+                      size_t *payloadSize,          // OUT: size of packet
+                      HgfsSessionInfo *session);    // IN: Session info
+Bool
+HgfsUnpackRemoveWatchRequest(void const *packet,             // IN: HGFS packet
+                             size_t packetSize,              // IN: packet size
+                             HgfsOp op,                      // IN: operation code
+                             HgfsSubscriberHandle *watchId); // OUT: watch Id
+Bool
+HgfsPackRemoveWatchReply(HgfsPacket *packet,           // IN/OUT: Hgfs Packet
+                         char const *packetHeader,     // IN: packet header
+                         HgfsOp     op,                // IN: operation code
+                         size_t *payloadSize,          // OUT: size of packet
+                         HgfsSessionInfo *session);    // IN: Session info
+size_t
+HgfsPackCalculateNotificationSize(char const *shareName, // IN: shared folder name
+                                  char *fileName);       // IN: file name
+Bool
+HgfsPackChangeNotificationRequest(void *packet,                    // IN/OUT: Hgfs Packet
+                                  HgfsSubscriberHandle subscriber, // IN: watch
+                                  char const *shareName,           // IN: share name
+                                  char *fileName,                  // IN: file name
+                                  uint32 mask,                     // IN: event mask
+                                  uint32 flags,                    // IN: notify flags
+                                  HgfsSessionInfo *session,        // IN: session
+                                  size_t *bufferSize);             // IN/OUT: packet size
 /* Node cache functions. */
 
 Bool
