@@ -65,7 +65,8 @@
 #if defined(__FreeBSD__) || \
     defined(VMX86_SERVER) || \
     defined(__APPLE__) || \
-    defined(N_PLAT_NLM)
+    defined(N_PLAT_NLM) || \
+    defined __ANDROID__
 #define CURRENT_IS_UTF8
 #endif
 
@@ -377,6 +378,11 @@ Bool CodeSet_UTF32ToUTF8(const char *utf32,
 
 int CodeSet_LengthInCodePoints(const char *utf8);
 
+int CodeSet_GetUtf8(const char *string,
+                    const char *end,
+                    uint32 *uchar);
+
+
 /*
  *-----------------------------------------------------------------------------
  *
