@@ -66,6 +66,11 @@
  */
 #define CONFGROUPNAME_GUESTINFO "guestinfo"
 
+/**
+ * Lets users disable just the perf monitor.
+ */
+#define CONFNAME_GUESTINFO_DISABLEPERFMON "disable-perf-mon"
+
 /**
  * Lets users disable just DiskInfo.
  *
