@@ -1831,27 +1831,37 @@ HgfsRequestSetWatchV4;
 /*
  * Fine grain notification event types.
  * HgfsRequestSetWatch events.
- * Events marked with (*) apply only to directories.
  */
-#define HGFS_NOTIFY_ACCESS        (1 << 0)    /* File accessed (read) */
-#define HGFS_NOTIFY_ATTRIB        (1 << 1)    /* Windows file attributes changed. */
-#define HGFS_NOTIFY_SIZE          (1 << 2)    /* File size changed. */
-#define HGFS_NOTIFY_ATIME         (1 << 3)    /* Access time changed. */
-#define HGFS_NOTIFY_MTIME         (1 << 4)    /* Modification time changed. */
-#define HGFS_NOTIFY_CTIME         (1 << 5)    /* Attribute change time changed. */
-#define HGFS_NOTIFY_CRTIME        (1 << 6)    /* Creation time changed. */
-#define HGFS_NOTIFY_NAME          (1 << 7)    /* File / Directory name changed. */
-#define HGFS_NOTIFY_OPEN          (1 << 8)    /* File opened */
-#define HGFS_NOTIFY_CLOSE_WRITE   (1 << 9)    /* File opened for writing closed */
-#define HGFS_NOTIFY_CLOSE_NOWRITE (1 << 10)   /* File opened for reading closed */
-#define HGFS_NOTIFY_CREATE        (1 << 11)   /* File / directory created */
-#define HGFS_NOTIFY_DELETE        (1 << 12)   /* (*) File / directory deleted */
-#define HGFS_NOTIFY_DELETE_SELF   (1 << 13)   /* Watched file / directory deleted */
-#define HGFS_NOTIFY_MODIFY        (1 << 14)   /* File or directory modified. */
-#define HGFS_NOTIFY_MOVE_SELF     (1 << 15)   /* Watched file / directory moved. */
-#define HGFS_NOTIFY_MOVE_FROM     (1 << 16)   /* (*) Moved out of watched directory. */
-#define HGFS_NOTIFY_MOVE_TO       (1 << 17)   /* (*) Moved into watched directory. */
-#define HGFS_NOTIFY_RENAME        (1 << 18)   /* File / directory name changed. */
+#define HGFS_NOTIFY_ACCESS                   (1 << 0)    /* File accessed (read) */
+#define HGFS_NOTIFY_ATTRIB                   (1 << 1)    /* File attributes changed. */
+#define HGFS_NOTIFY_SIZE                     (1 << 2)    /* File size changed. */
+#define HGFS_NOTIFY_ATIME                    (1 << 3)    /* Access time changed. */
+#define HGFS_NOTIFY_MTIME                    (1 << 4)    /* Modification time changed. */
+#define HGFS_NOTIFY_CTIME                    (1 << 5)    /* Attribute time changed. */
+#define HGFS_NOTIFY_CRTIME                   (1 << 6)    /* Creation time changed. */
+#define HGFS_NOTIFY_NAME                     (1 << 7)    /* File / Directory name. */
+#define HGFS_NOTIFY_OPEN                     (1 << 8)    /* File opened */
+#define HGFS_NOTIFY_CLOSE_WRITE              (1 << 9)    /* Modified file closed. */
+#define HGFS_NOTIFY_CLOSE_NOWRITE            (1 << 10)   /* Non-modified file closed. */
+#define HGFS_NOTIFY_CREATE_FILE              (1 << 11)   /* File created */
+#define HGFS_NOTIFY_CREATE_DIR               (1 << 12)   /* Directory created */
+#define HGFS_NOTIFY_DELETE_FILE              (1 << 13)   /* File deleted */
+#define HGFS_NOTIFY_DELETE_DIR               (1 << 14)   /* Directory deleted */
+#define HGFS_NOTIFY_DELETE_SELF              (1 << 15)   /* Watched directory deleted */
+#define HGFS_NOTIFY_MODIFY                   (1 << 16)   /* File modified. */
+#define HGFS_NOTIFY_MOVE_SELF                (1 << 17)   /* Watched directory moved. */
+#define HGFS_NOTIFY_OLD_FILE_NAME            (1 << 18)   /* Rename: old file name. */
+#define HGFS_NOTIFY_NEW_FILE_NAME            (1 << 19)   /* Rename: new file name. */
+#define HGFS_NOTIFY_OLD_DIR_NAME             (1 << 20)   /* Rename: old dir name. */
+#define HGFS_NOTIFY_NEW_DIR_NAME             (1 << 21)   /* Rename: new dir name. */
+#define HGFS_NOTIFY_CHANGE_EA                (1 << 22)   /* Extended attributes. */
+#define HGFS_NOTIFY_CHANGE_SECURITY          (1 << 23)   /* Security/permissions. */
+#define HGFS_NOTIFY_ADD_STREAM               (1 << 24)   /* Named stream created. */
+#define HGFS_NOTIFY_DELETE_STREAM            (1 << 25)   /* Named stream deleted. */
+#define HGFS_NOTIFY_CHANGE_STREAM_SIZE       (1 << 26)   /* Named stream size changed. */
+#define HGFS_NOTIFY_CHANGE_STREAM_LAST_WRITE (1 << 27)   /* Stream timestamp changed. */
+#define HGFS_NOTIFY_WATCH_DELETED            (1 << 28)   /* Dir with watch deleted. */
+#define HGFS_NOTIFY_EVENTS_DROPPED           (1 << 29)   /* Notifications dropped. */
 
 /* HgfsRequestSetWatch flags. */
 #define HGFS_NOTIFY_FLAG_WATCH_TREE  (1 << 0)    /* Watch the entire directory tree. */
@@ -1864,11 +1874,14 @@ HgfsRequestSetWatchV4;
                                                   * set.
                                                   */
 
+typedef uint64 HgfsSubscriberHandle;
+#define HGFS_INVALID_SUBSCRIBER_HANDLE         ((HgfsSubscriberHandle)~((HgfsSubscriberHandle)0))
+
 typedef
 #include "vmware_pack_begin.h"
 struct HgfsReplySetWatchV4 {
-    HgfsHandle watchId;    /* Watch identifier for subsequent references. */
-    uint64 reserved;       /* Reserved for future use. */
+    HgfsSubscriberHandle watchId; /* Watch identifier for subsequent references. */
+    uint64 reserved;              /* Reserved for future use. */
 }
 #include "vmware_pack_end.h"
 HgfsReplySetWatchV4;
@@ -1876,7 +1889,7 @@ HgfsReplySetWatchV4;
 typedef
 #include "vmware_pack_begin.h"
 struct HgfsRequestRemoveWatchV4 {
-    HgfsHandle watchId;    /* Watch identifier to remove. */
+    HgfsSubscriberHandle watchId;  /* Watch identifier to remove. */
 }
 #include "vmware_pack_end.h"
 HgfsRequestRemoveWatchV4;
@@ -1891,15 +1904,14 @@ HgfsReplyRemoveWatchV4;
 
 typedef
 #include "vmware_pack_begin.h"
-struct HgfsNotifyEvent {
+struct HgfsNotifyEventV4 {
    uint32 nextOffset;        /* Offset of next event; 0 if it i sthe last one. */
    uint64 mask;              /* Event occurred. */
    uint64 reserved;          /* Reserved for future use. */
    HgfsFileName fileName;    /* Filename. */
-   HgfsFileName oldName;     /* New filename for rename operation. */
 }
 #include "vmware_pack_end.h"
-HgfsNotifyEvent;
+HgfsNotifyEventV4;
 
 /* Too many events, some or all event were dropped by the server. */
 #define HGFS_NOTIFY_FLAG_OVERFLOW          (1 << 0)
@@ -1910,15 +1922,15 @@ HgfsNotifyEvent;
 
 typedef
 #include "vmware_pack_begin.h"
-struct HgfsRequestNotify {
-   HgfsHandle watchId;         /* Watch identifier. */
-   uint32 flags;               /* Various flags. */
-   uint32 count;               /* Number of events occured. */
-   uint64 reserved;            /* Reserved for future use. */
-   HgfsNotifyEvent events[1];  /* Events. HgfsNotifyEvent(s). */
+struct HgfsRequestNotifyV4 {
+   HgfsSubscriberHandle watchId; /* Watch identifier. */
+   uint32 flags;                 /* Various flags. */
+   uint32 count;                 /* Number of events occured. */
+   uint64 reserved;              /* Reserved for future use. */
+   HgfsNotifyEventV4 events[1];  /* Events. HgfsNotifyEvent(s). */
 }
 #include "vmware_pack_end.h"
-HgfsRequestNotify;
+HgfsRequestNotifyV4;
 
 // Query EA flags values.
 #define HGFS_QUERY_EA_INDEX_SPECIFIED (1 << 0)
@@ -1927,7 +1939,7 @@ HgfsRequestNotify;
 
 typedef
 #include "vmware_pack_begin.h"
-struct HgfsRequestQueryEA {
+struct HgfsRequestQueryEAV4 {
    uint32 flags;                 /* EA flags. */
    uint32 index;
    uint64 reserved;              /* Reserved for future use. */
@@ -1940,11 +1952,11 @@ struct HgfsRequestQueryEA {
                                   */
 }
 #include "vmware_pack_end.h"
-HgfsRequestQueryEA;
+HgfsRequestQueryEAV4;
 
 typedef
 #include "vmware_pack_begin.h"
-struct HgfsReplyQueryEA {
+struct HgfsReplyQueryEAV4 {
    uint32 nextOffset;            /* Offset of the next structure when more then
                                   * one record is returned.
                                   */
@@ -1955,37 +1967,37 @@ struct HgfsReplyQueryEA {
    char eaData[1];               /* NULL termianed EA name followed by EA value. */
 }
 #include "vmware_pack_end.h"
-HgfsReplyQueryEA;
+HgfsReplyQueryEAV4;
 
 
 typedef
 #include "vmware_pack_begin.h"
-struct HgfsEA {
+struct HgfsEAV4 {
    uint32 nextOffset;      /* Offset of the next structure in the chain. */
    uint32 valueLength;     /* EA value length. */
    char data[1];           /* NULL terminated EA name followed by EA value. */
 }
 #include "vmware_pack_end.h"
-HgfsEA;
+HgfsEAV4;
 
 typedef
 #include "vmware_pack_begin.h"
-struct HgfsRequestSetEA {
-   uint32 flags;           /* Flags, see below. */
-   uint64 reserved;        /* Reserved for future use. */
-   uint32 numEAs;          /* Number of EAs in this request. */
-   HgfsEA attributes[1];   /* Array of attributes. */
+struct HgfsRequestSetEAV4 {
+   uint32   flags;           /* Flags, see below. */
+   uint64   reserved;        /* Reserved for future use. */
+   uint32   numEAs;          /* Number of EAs in this request. */
+   HgfsEAV4 attributes[1];   /* Array of attributes. */
 }
 #include "vmware_pack_end.h"
-HgfsRequestSetEA;
+HgfsRequestSetEAV4;
 
 typedef
 #include "vmware_pack_begin.h"
-struct HgfsReplySetEA {
+struct HgfsReplySetEAV4 {
    uint64 reserved;        /* Reserved for future use. */
 }
 #include "vmware_pack_end.h"
-HgfsReplySetEA;
+HgfsReplySetEAV4;
 
 /*
  * EA Flags. When both flags are set EA is either created or replaced if it exists.
@@ -2007,7 +2019,7 @@ HgfsReplySetEA;
 
 typedef
 #include "vmware_pack_begin.h"
-struct HgfsRequestLockRange {
+struct HgfsRequestLockRangeV4 {
    HgfsHandle     fid;          /* File to take lock on. */
    uint32 flags;                /* Various flags. */
    uint64 start;                /* Starting offset in the file. */
@@ -2015,36 +2027,37 @@ struct HgfsRequestLockRange {
    uint64 reserved;             /* Reserved for future use. */
 }
 #include "vmware_pack_end.h"
-HgfsRequestLockRange;
+HgfsRequestLockRangeV4;
 
 typedef
 #include "vmware_pack_begin.h"
-struct HgfsReplyLockRange {
+struct HgfsReplyLockRangeV4 {
    uint64 reserved;             /* Reserved for future use. */
 }
 #include "vmware_pack_end.h"
-HgfsReplyLockRange;
+HgfsReplyLockRangeV4;
 
 #define HGFS_RANGE_LOCK_UNLOCK_ALL               (1 << 0)
 
 typedef
 #include "vmware_pack_begin.h"
-struct HgfsRequestUnlockRange {
+struct HgfsRequestUnlockRangeV4 {
    HgfsHandle     fid;          /* File to take lock on. */
    uint32 flags;                /* Various flags. */
    uint64 start;                /* Starting offset in the file. */
    uint64 length;               /* Number of bytes to lock. */
-}
-#include "vmware_pack_end.h"
-HgfsRequestUnlockRange;
-
-typedef
-#include "vmware_pack_begin.h"
-struct HgfsReplyUnlockRange {
    uint64 reserved;             /* Reserved for future use. */
 }
 #include "vmware_pack_end.h"
-HgfsReplyUnlockRange;
+HgfsRequestUnlockRangeV4;
+
+typedef
+#include "vmware_pack_begin.h"
+struct HgfsReplyUnlockRangeV4 {
+   uint64 reserved;             /* Reserved for future use. */
+}
+#include "vmware_pack_end.h"
+HgfsReplyUnlockRangeV4;
 
 /*
  * There are three types of oplocks: level 1, batch, and level 2. Both the level 1 and
@@ -2294,7 +2307,7 @@ struct HgfsRequestOpenV4 {
     * If the host file system does not support EA server should fail the request rather
     * then succeeding and silently dropping EA.
     */
-   HgfsRequestSetEA extendedAttributes;
+   HgfsRequestSetEAV4 extendedAttributes;
    uint32 aclLength;               /* Length of the acl field. */
    char acl[1];                    /* Multi-platform ACL as defined in RFC 3530. */
 }
@@ -2484,6 +2497,7 @@ struct HgfsReplyReadDirectoryV4 {
    uint32 numberEntriesReturned;
    uint32 offsetToContinue;
    HgfsDirectoryReadMaskV4 mask; /* Returned mask that may be a subset of requested mask. */
+   uint64 reserved;              /* Reserved for future use. */
    HgfsDirectoryEntryV4 entries[1];
 }
 #include "vmware_pack_end.h"
@@ -2513,13 +2527,25 @@ HgfsRequestStreamEntryV4;
 
 typedef
 #include "vmware_pack_begin.h"
-struct HgfsRequestGetAttrV4 {
+struct HgfsReplyEnumerateStreamsV4 {
+   uint32 numberEntriesReturned;
+   uint32 offsetToContinue;
+   uint64 reserved;
+   HgfsRequestStreamEntryV4 entries[1];
+}
+#include "vmware_pack_end.h"
+HgfsReplyEnumerateStreamsV4;
+
+typedef
+#include "vmware_pack_begin.h"
+struct HgfsRequestGetattrV4 {
    uint32 mask;
    uint32 flags;
+   uint64 reserved;
    HgfsFileNameV3 name;
 }
 #include "vmware_pack_end.h"
-HgfsRequestGetAttrV4;
+HgfsRequestGetattrV4;
 
 /*
  * V4 reports different file size for symlinks then V3 or V2.
@@ -2547,6 +2573,7 @@ struct HgfsAttrV4 {
    HgfsFileName shortName;
    HgfsFileName symlinkTarget;
    uint32 aclLength;
+   uint64 reserved;
    char acl[1];
 }
 #include "vmware_pack_end.h"
@@ -2554,19 +2581,30 @@ HgfsAttrV4;
 
 typedef
 #include "vmware_pack_begin.h"
-struct HgfsReplyGetAttrV4 {
+struct HgfsReplyGetattrV4 {
    HgfsAttrV4 attr;
 }
 #include "vmware_pack_end.h"
-HgfsReplyGetAttrV4;
+HgfsReplyGetattrV4;
 
 typedef
 #include "vmware_pack_begin.h"
-struct HgfsReplySetAttrV4 {
+struct HgfsRequestSetattrV4 {
+   HgfsAttrHint hints;
+   HgfsAttrV2 attr;
+   uint64 reserved;          /* Reserved for future use */
+   HgfsFileNameV3 fileName;  /* Filename used when file handle invalid. */
+}
+#include "vmware_pack_end.h"
+HgfsRequestSetattrV4;
+
+typedef
+#include "vmware_pack_begin.h"
+struct HgfsReplySetattrV4 {
    uint32 mask;                      /* Defines which attributes were set. */
 }
 #include "vmware_pack_end.h"
-HgfsReplySetAttrV4;
+HgfsReplySetattrV4;
 
 /*
  * Unlike V3 deletion this command can be used to delete both files and directories.
