@@ -53,7 +53,19 @@ extern char *Hostinfo_GetCpuDescription(uint32 cpuNumber);
 extern void Hostinfo_GetTimeOfDay(VmTimeType *time);
 extern VmTimeType Hostinfo_SystemUpTime(void);
 extern VmTimeType Hostinfo_SystemTimerNS(void);
-extern VmTimeType Hostinfo_SystemTimerUS(void);
+
+static INLINE VmTimeType
+Hostinfo_SystemTimerUS(void)
+{
+   return Hostinfo_SystemTimerNS() / 1000ULL;
+}
+
+static INLINE VmTimeType
+Hostinfo_SystemTimerMS(void)
+{
+   return Hostinfo_SystemTimerNS() / 1000000ULL;
+}
+
 extern int Hostinfo_OSVersion(unsigned int i);
 extern int Hostinfo_GetSystemBitness(void);
 extern const char *Hostinfo_OSVersionString(void);
@@ -82,7 +94,7 @@ extern char *Hostinfo_HypervisorCPUIDSig(void);
 #define HGMP_PRIVILEGE    0
 #define HGMP_NO_PRIVILEGE 1
 extern Unicode Hostinfo_GetModulePath(uint32 priv);
-
+extern char *Hostinfo_GetLibraryPath(void *addr);
 
 #if !defined(_WIN32)
 extern void Hostinfo_ResetProcessState(const int *keepFds, size_t numKeepFds);
@@ -123,10 +135,6 @@ typedef struct {
 
    uint32 features;
    uint32 extfeatures;
-
-   uint32 numPhysCPUs;
-   uint32 numCores;
-   uint32 numLogCPUs;
 } HostinfoCpuIdInfo;
 
 
@@ -134,8 +142,8 @@ extern uint32 Hostinfo_NumCPUs(void);
 extern char *Hostinfo_GetCpuidStr(void);
 extern Bool Hostinfo_GetCpuid(HostinfoCpuIdInfo *info);
 
-#if defined(VMX86_SERVER)
-extern Bool Hostinfo_HTDisabled(void);
+#if !defined(VMX86_SERVER)
+extern Bool Hostinfo_CPUCounts(uint32 *logical, uint32 *cores, uint32 *pkgs);
 #endif
 
 #if defined(_WIN32)
