@@ -105,6 +105,12 @@ typedef struct ProcMgr_ProcArgs {
     * environment.
     */
    char **envp;
+
+   /*
+    * If non-NULL, the directory to be changed to before the process is
+    * started.
+    */
+   char *workingDirectory;
 #endif
 } ProcMgr_ProcArgs;
 
@@ -139,5 +145,6 @@ Bool ProcMgr_ImpersonateUserStart(const char *user,      // UTF-8
                                   AuthToken token);
 Bool ProcMgr_ImpersonateUserStop(void);
 #endif
+Bool ProcMgr_GetImpersonatedUserInfo(char **username, char **homeDir);
 
 #endif /* __PROCMGR_H__ */
