@@ -83,8 +83,7 @@ unsigned int RpcIn_SetRetVals(char const **result, size_t *resultLen,
 #endif /* } */
 
 void RpcIn_Destruct(RpcIn *in);
-Bool RpcIn_stop(RpcIn *in);
-
+void RpcIn_stop(RpcIn *in);
 
 #ifdef __cplusplus
 } // extern "C"
