@@ -224,6 +224,23 @@
 #define SC_R13(uc) ((unsigned long) (uc)->uc_mcontext.gregs[GNU_REG_R13])
 #define SC_R14(uc) ((unsigned long) (uc)->uc_mcontext.gregs[GNU_REG_R14])
 #define SC_R15(uc) ((unsigned long) (uc)->uc_mcontext.gregs[GNU_REG_R15])
+#elif defined(__arm__)
+#define SC_R0(uc)  ((unsigned long) (uc)->uc_mcontext.arm_r0)
+#define SC_R1(uc)  ((unsigned long) (uc)->uc_mcontext.arm_r1)
+#define SC_R2(uc)  ((unsigned long) (uc)->uc_mcontext.arm_r2)
+#define SC_R3(uc)  ((unsigned long) (uc)->uc_mcontext.arm_r3)
+#define SC_R4(uc)  ((unsigned long) (uc)->uc_mcontext.arm_r4)
+#define SC_R5(uc)  ((unsigned long) (uc)->uc_mcontext.arm_r5)
+#define SC_R6(uc)  ((unsigned long) (uc)->uc_mcontext.arm_r6)
+#define SC_R7(uc)  ((unsigned long) (uc)->uc_mcontext.arm_r7)
+#define SC_R8(uc)  ((unsigned long) (uc)->uc_mcontext.arm_r8)
+#define SC_R9(uc)  ((unsigned long) (uc)->uc_mcontext.arm_r9)
+#define SC_R10(uc) ((unsigned long) (uc)->uc_mcontext.arm_r10)
+#define SC_FP(uc)  ((unsigned long) (uc)->uc_mcontext.arm_fp)
+#define SC_IP(uc)  ((unsigned long) (uc)->uc_mcontext.arm_ip)
+#define SC_SP(uc)  ((unsigned long) (uc)->uc_mcontext.arm_sp)
+#define SC_LR(uc)  ((unsigned long) (uc)->uc_mcontext.arm_lr)
+#define SC_PC(uc)  ((unsigned long) (uc)->uc_mcontext.arm_pc)
 #else
 #define SC_EAX(uc) ((unsigned long) (uc)->uc_mcontext.gregs[GNU_REG_EAX])
 #define SC_EBX(uc) ((unsigned long) (uc)->uc_mcontext.gregs[GNU_REG_EBX])
