@@ -65,6 +65,7 @@ EXTERN uint32 CRC_Compute(const uint8 *buf, int len);
 EXTERN uint32 Util_Checksum32(const uint32 *buf, int len);
 EXTERN uint32 Util_Checksum(const uint8 *buf, int len);
 EXTERN uint32 Util_Checksumv(void *iov, int numEntries);
+EXTERN uint32 Util_HashString(const char *str);
 EXTERN Unicode Util_ExpandString(ConstUnicode fileName);
 EXTERN void Util_ExitThread(int);
 EXTERN NORETURN void Util_ExitProcessAbruptly(int);
@@ -127,6 +128,20 @@ void Util_BacktraceToBuffer(uintptr_t *basePtr,
 
 int Util_CompareDotted(const char *s1, const char *s2);
 
+/*
+ * This enum defines how Util_GetOpt should handle non-option arguments:
+ *
+ * UTIL_NONOPT_PERMUTE: Permute argv so that all non-options are at the end.
+ * UTIL_NONOPT_STOP:    Stop when first non-option argument is seen.
+ * UTIL_NONOPT_ALL:     Return each non-option argument as if it were
+ *                      an option with character code 1.
+ */
+typedef enum { UTIL_NONOPT_PERMUTE, UTIL_NONOPT_STOP, UTIL_NONOPT_ALL } Util_NonOptMode;
+struct option;
+int Util_GetOpt(int argc, char * const *argv, const struct option *opts,
+                Util_NonOptMode mode);
+
+
 #if defined(VMX86_STATS)
 Bool Util_QueryCStResidency(uint32 *numCpus, uint32 *numCStates,
                             uint64 **transitns, uint64 **residency,
@@ -139,17 +154,92 @@ Bool Util_QueryCStResidency(uint32 *numCpus, uint32 *numCStates,
 EXTERN Bool Util_Throttle(uint32 count);
 EXTERN uint32 Util_FastRand(uint32 seed);
 
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * Util_ValidateBytes --
+ *
+ *      Check that memory is filled with the specified value.
+ *
+ * Results:
+ *      NULL   No error
+ *      !NULL  First address that doesn't have the proper value
+ *
+ * Side effects:
+ *      None.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+static INLINE void *
+Util_ValidateBytes(const void *ptr,  // IN: ptr to check
+                   size_t size,      // IN: size of ptr
+                   uint8 byteValue)  // IN: memory must be filled with this
+{
+   uint8 *p;
+   uint8 *end;
+   uint64 bigValue;
+
+   ASSERT(ptr);
+
+   if (size == 0) {
+      return NULL;
+   }
+
+   p = (uint8 *) ptr;
+   end = p + size;
+
+   /* Compare bytes until a "nice" boundary is achieved. */
+   while ((uintptr_t) p % sizeof bigValue) {
+      if (*p != byteValue) {
+         return p;
+      }
+
+      p++;
+
+      if (p == end) {
+         return NULL;
+      }
+   }
+
+   /* Compare using a "nice sized" chunk for a long as possible. */
+   memset(&bigValue, (int) byteValue, sizeof bigValue);
+
+   while (p + sizeof bigValue <= end) {
+      if (*((uint64 *) p) != bigValue) {
+         /* That's not right... let the loop below report the exact address. */
+         break;
+      }
+
+      size -= sizeof bigValue;
+      p += sizeof bigValue;
+   }
+
+   /* Handle any trailing bytes. */
+   while (p < end) {
+      if (*p != byteValue) {
+         return p;
+      }
+
+      p++;
+   }
+
+   return NULL;
+}
+
+
 /*
  *----------------------------------------------------------------------
  *
  * Util_BufferIsEmpty --
  *
- *    Determine wether or not the buffer of 'len' bytes starting at 'base' is
- *    empty (i.e. full of zeroes)
+ *    Determine if the specified buffer of 'len' bytes starting at 'base'
+ *    is empty (i.e. full of zeroes).
  *
  * Results:
- *    TRUE if yes
- *    FALSE if no
+ *    TRUE  Yes
+ *    FALSE No
  *
  * Side effects:
  *    None
@@ -157,45 +247,16 @@ EXTERN uint32 Util_FastRand(uint32 seed);
  *----------------------------------------------------------------------
  */
 
-static INLINE Bool Util_BufferIsEmpty(void const *base, // IN
-                                      size_t len)       // IN
+static INLINE Bool
+Util_BufferIsEmpty(void const *base,  // IN:
+                   size_t len)        // IN:
 {
-   uint32 const *p32;
-   uint32 const *e32;
-   uint16 const *p16;
-
-   ASSERT_ON_COMPILE(sizeof(uint32) == 4);
-
-   p32 = (uint32 const *)base;
-   e32 = p32 + len / 4;
-   for (; p32 < e32; p32++) {
-      if (*p32) {
-         return FALSE;
-      }
-   }
-
-   len &= 0x3;
-   p16 = (uint16 const *)p32;
-
-   if (len & 0x2) {
-      if (*p16) {
-         return FALSE;
-      }
-
-      p16++;
-   }
-
-   if (   len & 0x1
-       && *(uint8 const *)p16) {
-      return FALSE;
-   }
-
-   return TRUE;
-};
+   return Util_ValidateBytes(base, len, '\0') == NULL;
+}
 
 
 EXTERN Bool Util_MakeSureDirExistsAndAccessible(char const *path,
-						unsigned int mode);
+                                                unsigned int mode);
 
 #ifdef N_PLAT_NLM
 #   define DIRSEPS	      "\\"
@@ -314,10 +375,18 @@ EXTERN char *Util_SafeInternalStrndup(int bugNumber, const char *s, size_t n,
 
 static INLINE void
 Util_Zero(void *buf,       // OUT
-          size_t bufSize)  // IN 
+          size_t bufSize)  // IN
 {
    if (buf != NULL) {
+#if defined _WIN32 && defined USERLEVEL
+      /*
+       * Simple memset calls might be optimized out.  See CERT advisory
+       * MSC06-C.
+       */
+      SecureZeroMemory(buf, bufSize);
+#else
       memset(buf, 0, bufSize);
+#endif
    }
 }
 
@@ -339,10 +408,10 @@ Util_Zero(void *buf,       // OUT
  */
 
 static INLINE void
-Util_ZeroString(char *str)  // IN
+Util_ZeroString(char *str)  // IN/OUT
 {
    if (str != NULL) {
-      memset(str, 0, strlen(str));
+      Util_Zero(str, strlen(str));
    }
 }
 
@@ -366,11 +435,11 @@ Util_ZeroString(char *str)  // IN
 
 static INLINE void
 Util_ZeroFree(void *buf,       // OUT
-              size_t bufSize)  // IN 
+              size_t bufSize)  // IN
 {
    if (buf != NULL) {
-      memset(buf, 0, bufSize);
-      free(buf); 
+      Util_Zero(buf, bufSize);
+      free(buf);
    }
 }
 
@@ -396,8 +465,8 @@ static INLINE void
 Util_ZeroFreeString(char *str)  // IN
 {
    if (str != NULL) {
-      Util_Zero(str, strlen(str));
-      free(str); 
+      Util_ZeroString(str);
+      free(str);
    }
 }
 
@@ -424,8 +493,8 @@ static INLINE void
 Util_ZeroFreeStringW(wchar_t *str)  // IN
 {
    if (str != NULL) {
-      Util_Zero(str, wcslen(str) * sizeof(wchar_t));
-      free(str); 
+      Util_Zero(str, wcslen(str) * sizeof *str);
+      free(str);
    }
 }
 #endif // _WIN32
@@ -467,15 +536,15 @@ Util_FreeList(void **list,      // IN/OUT: the list to free
       ssize_t i;
 
       for (i = 0; i < length; i++) {
-	 free(list[i]);
-	 DEBUG_ONLY(list[i] = NULL);
+         free(list[i]);
+         DEBUG_ONLY(list[i] = NULL);
       }
    } else {
       void **s;
 
       for (s = list; *s != NULL; s++) {
-	 free(*s);
-	 DEBUG_ONLY(*s = NULL);
+         free(*s);
+         DEBUG_ONLY(*s = NULL);
       }
    }
    free(list);
