@@ -64,6 +64,7 @@
 #define PRODUCT_WORKSTATION_BRIEF_NAME "Workstation"
 #define PRODUCT_WORKSTATION_ENTERPRISE_BRIEF_NAME \
          PRODUCT_WORKSTATION_BRIEF_NAME " " "ACE Edition"
+#define PRODUCT_WORKSTATION_SERVER_BRIEF_NAME "Workstation Server"
 #define PRODUCT_PLAYER_BRIEF_NAME "Player"
 #define PRODUCT_ACE_PLAYER_BRIEF_NAME "ACE " PRODUCT_PLAYER_BRIEF_NAME
 #define PRODUCT_MAC_DESKTOP_BRIEF_NAME "Fusion"
@@ -87,6 +88,7 @@
 #define PRODUCT_GSX_SMP_NAME MAKE_NAME("Virtual SMP for GSX Server")
 #define PRODUCT_WORKSTATION_NAME MAKE_NAME(PRODUCT_WORKSTATION_BRIEF_NAME)
 #define PRODUCT_WORKSTATION_ENTERPRISE_NAME MAKE_NAME(PRODUCT_WORKSTATION_ENTERPRISE_BRIEF_NAME)
+#define PRODUCT_WORKSTATION_SERVER_NAME MAKE_NAME(PRODUCT_WORKSTATION_SERVER_BRIEF_NAME)
 #define PRODUCT_MUI_NAME MAKE_NAME("Management Interface")
 #define PRODUCT_CONSOLE_NAME MAKE_NAME("Server Console")
 #define PRODUCT_PLAYER_NAME MAKE_NAME(PRODUCT_PLAYER_BRIEF_NAME)
@@ -144,6 +146,8 @@
 #define PRODUCT_XVP_NAME MAKE_NAME("vCenter XVP Manager")
 #define PRODUCT_RMKSCONTAINER_NAME MAKE_NAME("Remote MKS Container")
 
+#define PRODUCT_BOOMERANG_NAME MAKE_NAME("Boomerang")
+
 // XXX VMvisor is the underlying technology for possibly several products,
 // XXX not the product. Fix when names are decided.
 // #define PRODUCT_VMVISOR_NAME MAKE_NAME("VMvisor")
@@ -174,8 +178,8 @@
 /*
  * VMRC ActiveX CLSIDs and ProgIDs
  */
-#define PRODUCT_VMRC_PLUGIN_GUID_EMBEDDED      0567856D-8D8E-43ad-9241-7772F392B3A3
-#define PRODUCT_VMRC_PLUGIN_GUID_TYPELIB       AFAD9CB7-273F-4f59-8C78-D8AF13845222
+#define PRODUCT_VMRC_PLUGIN_GUID_EMBEDDED      291BA977-564B-4626-B868-A968FB8D4591
+#define PRODUCT_VMRC_PLUGIN_GUID_TYPELIB       70ABCAB7-6B45-4555-ABFC-8BC686E5ACFE
 #define PRODUCT_VMRC_PLUGIN_PROGID_EMBEDDED_BASE "VMware.VMwareRemoteConsole"
 #define PRODUCT_VMRC_PLUGIN_PROGID_EMBEDDED    PRODUCT_VMRC_PLUGIN_PROGID_EMBEDDED_BASE "." \
                                                XSTR(VMRC_PLUGIN_VERSION_BASE)
@@ -192,18 +196,7 @@
 #endif
 
 #define PRODUCT_VMRC_PLUGIN_CURRENT_MIMETYPE \
-   "application/x-vmware-vmrc;version=" VMRC_PLUGIN_VERSION
-
-/*
- * previously shipped plugin mimetypes (updated as we ship them)
- *
- * XXX It is important that the mimetypes be listed in strictly decreasing
- *     order by version. Server 2.0 Web Access (WBC) only checks the first VMRC
- *     mimetype for compatibility, and others might make the same mistake in
- *     the future.
- */
-#define PRODUCT_VMRC_PLUGIN_PREVIOUS_MIMETYPES \
-   "application/x-vmware-vmrc;version=4.0.0.0"
+   "application/x-vmware-remote-console-2011"
 
 /*
  * legacy plugin mimetypes (currently unused but here for reference)
@@ -223,8 +216,7 @@
  * NB: See above for constraints on the ordering of this list.
  */
 #define PRODUCT_VMRC_PLUGIN_MIMETYPES \
-   PRODUCT_VMRC_PLUGIN_CURRENT_MIMETYPE PRODUCT_VMRC_MIMETYPE_SEPARATOR \
-   PRODUCT_VMRC_PLUGIN_PREVIOUS_MIMETYPES PRODUCT_VMRC_MIMETYPE_TERMINATOR
+   PRODUCT_VMRC_PLUGIN_CURRENT_MIMETYPE PRODUCT_VMRC_MIMETYPE_TERMINATOR
 
 /*
  * VMware USB Arbitration Service version definitions
@@ -264,7 +256,8 @@
       || defined(VMX86_P2V)      \
       || defined(VMX86_DDK)      \
       || defined(VMX86_VDDK)     \
-      || defined(VMX86_NETDUMP))
+      || defined(VMX86_NETDUMP) \
+      || defined(VMX86_BOOMERANG))
 #   if defined(_WIN32) || defined(__APPLE__)
       /*
        * XXX Make the product be Workstation by default if none of the defines
@@ -339,6 +332,8 @@
 # define PRODUCT_SHORT_NAME PRODUCT_VDDK_NAME
 #elif defined(VMX86_NETDUMP)
 # define PRODUCT_SHORT_NAME PRODUCT_NETDUMP_NAME
+#elif defined(VMX86_BOOMERANG)
+# define PRODUCT_SHORT_NAME PRODUCT_BOOMERANG_NAME
 #endif
 
 
