@@ -54,6 +54,7 @@
 #define SO_VMCI_PEER_HOST_VM_ID             3
 #define SO_VMCI_SERVICE_LABEL               4
 #define SO_VMCI_TRUSTED                     5
+#define SO_VMCI_CONNECT_TIMEOUT             6
 
 /*
  * The VMCI sockets address equivalents of INADDR_ANY.  The first works for
@@ -122,7 +123,18 @@ struct sockaddr_vm {
          }
          return afvalue;
       }
-   
+
+      static __inline int VMCISock_GetAFValueFd(int *outFd)
+      {
+         (void)outFd; /* Unused parameter. */
+         return VMCISock_GetAFValue();
+      }
+
+      static __inline void VMCISock_ReleaseAFValueFd(int fd)
+      {
+         (void)fd; /* Unused parameter. */
+      }
+
       static __inline unsigned int VMCISock_GetLocalCID(void)
       {
          unsigned int cid = VMADDR_CID_ANY;
