@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2009-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2009 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -58,15 +58,6 @@
 
 #else   // !WIN32
 
-/**
- * Signal emitted upon X I/O error callback firing.
- *
- * @param[in]   src     The source object.
- * @param[in]   ctx     ToolsAppCtx *: the application context.
- * @param[in]   data    Client data.
- */
-#define TOOLS_CORE_SIG_XIOERROR "tcs_de_xioerror"
-
 /**
  * Signal emitted upon SmcCallbacks::save_yourself.
  *
