@@ -28,11 +28,11 @@
 
 typedef struct
 {
-   VmTimeType              holdStart;
-
    MXUserAcquisitionStats  acquisitionStats;
    Atomic_Ptr              acquisitionHisto;
 
+   void                   *holder;
+   VmTimeType              holdStart;
    MXUserBasicStats        heldStats;
    Atomic_Ptr              heldHisto;
 } MXUserStats;
@@ -45,7 +45,6 @@ struct MXUserExclLock
 };
 
 
-#if defined(MXUSER_STATS)
 /*
  *-----------------------------------------------------------------------------
  *
@@ -113,7 +112,6 @@ MXUserStatsActionExcl(MXUserHeader *header)  // IN:
       }
    }
 }
-#endif
 
 
 /*
@@ -136,332 +134,20 @@ static void
 MXUserDumpExclLock(MXUserHeader *header)  // IN:
 {
    MXUserExclLock *lock = (MXUserExclLock *) header;
+   MXUserStats *stats = (MXUserStats *) Atomic_ReadPtr(&lock->statsMem);
 
    Warning("%s: Exclusive lock @ 0x%p\n", __FUNCTION__, lock);
 
    Warning("\tsignature 0x%X\n", lock->header.signature);
    Warning("\tname %s\n", lock->header.name);
    Warning("\trank 0x%X\n", lock->header.rank);
+   Warning("\tserial number %u\n", lock->header.serialNumber);
 
    Warning("\tcount %u\n", lock->recursiveLock.referenceCount);
 
-#if defined(MXUSER_DEBUG)
-   Warning("\tcaller 0x%p\n", lock->recursiveLock.ownerRetAddr);
-#endif
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * MXUser_CreateExclLock --
- *
- *      Create an exclusive lock.
- *
- * Results:
- *      NULL  Creation failed
- *      !NULL Creation succeeded
- *
- * Side effects:
- *      None
- *
- *-----------------------------------------------------------------------------
- */
-
-MXUserExclLock *
-MXUser_CreateExclLock(const char *userName,  // IN:
-                      MX_Rank rank)          // IN:
-{
-   char *properName;
-   MXUserStats *stats;
-   MXUserExclLock *lock;
-
-   lock = Util_SafeCalloc(1, sizeof(*lock));
-
-   if (userName == NULL) {
-      properName = Str_SafeAsprintf(NULL, "X-%p", GetReturnAddress());
-   } else {
-      properName = Util_SafeStrdup(userName);
+   if (stats && (stats->holder != NULL)) {
+      Warning("\tholder %p\n", stats->holder);
    }
-
-   if (!MXRecLockInit(&lock->recursiveLock)) {
-      free(properName);
-      free(lock);
-
-      return NULL;
-   }
-
-   lock->header.name = properName;
-   lock->header.signature = MXUSER_EXCL_SIGNATURE;
-   lock->header.rank = rank;
-   lock->header.dumpFunc = MXUserDumpExclLock;
-
-#if defined(MXUSER_STATS)
-   lock->header.statsFunc = MXUserStatsActionExcl;
-   lock->header.identifier = MXUserAllocID();
-
-   stats = Util_SafeCalloc(1, sizeof(*stats));
-
-   MXUserAcquisitionStatsSetUp(&stats->acquisitionStats);
-   MXUserBasicStatsSetUp(&stats->heldStats, MXUSER_STAT_CLASS_HELD);
-#else
-   stats = NULL;
-#endif
-
-   Atomic_WritePtr(&lock->statsMem, stats);
-
-   MXUserAddToList(&lock->header);
-
-   return lock;
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * MXUser_DestroyExclLock --
- *
- *      Destroy an exclusive lock.
- *
- * Results:
- *      Lock is destroyed. Don't use the pointer again.
- *
- * Side effects:
- *      None
- *
- *-----------------------------------------------------------------------------
- */
-
-void
-MXUser_DestroyExclLock(MXUserExclLock *lock)  // IN:
-{
-   if (lock != NULL) {
-      MXUserStats *stats;
-
-      ASSERT(lock->header.signature == MXUSER_EXCL_SIGNATURE);
-
-      if (MXRecLockCount(&lock->recursiveLock) > 0) {
-         MXUserDumpAndPanic(&lock->header,
-                            "%s: Destroy of an acquired exclusive lock\n",
-                            __FUNCTION__);
-      }
-
-      MXRecLockDestroy(&lock->recursiveLock);
-
-      MXUserRemoveFromList(&lock->header);
-
-      stats = (MXUserStats *) Atomic_ReadPtr(&lock->statsMem);
-
-      if (stats) {
-         MXUserAcquisitionStatsTearDown(&stats->acquisitionStats);
-         MXUserBasicStatsTearDown(&stats->heldStats);
-         MXUserHistoTearDown(Atomic_ReadPtr(&stats->acquisitionHisto));
-         MXUserHistoTearDown(Atomic_ReadPtr(&stats->heldHisto));
-
-         free(stats);
-      }
-
-      lock->header.signature = 0;  // just in case...
-      free((void *) lock->header.name);  // avoid const warnings
-      lock->header.name = NULL;
-      free(lock);
-   }
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * MXUser_AcquireExclLock --
- *
- *      An acquisition is made (lock is taken) on the specified exclusive lock.
- *
- * Results:
- *      The lock is acquired (locked).
- *
- * Side effects:
- *      None
- *
- *-----------------------------------------------------------------------------
- */
-
-void
-MXUser_AcquireExclLock(MXUserExclLock *lock)  // IN/OUT:
-{
-   MXUserStats *stats;
-
-   ASSERT(lock && (lock->header.signature == MXUSER_EXCL_SIGNATURE));
-
-   stats = (MXUserStats *) Atomic_ReadPtr(&lock->statsMem);
-
-   MXUserAcquisitionTracking(&lock->header, TRUE);
-
-   if (stats) {
-      VmTimeType begin = Hostinfo_SystemTimerNS();
-      VmTimeType value;
-      Bool contended;
-      MXUserHisto *histo;
-
-      contended = MXRecLockAcquire(&lock->recursiveLock, GetReturnAddress());
-
-      value = Hostinfo_SystemTimerNS() - begin;
-
-      MXUserAcquisitionSample(&stats->acquisitionStats, TRUE, contended,
-                              value);
-
-      histo = Atomic_ReadPtr(&stats->acquisitionHisto);
-
-      if (UNLIKELY(histo != NULL)) {
-         MXUserHistoSample(histo, value);
-      }
-   } else {
-      MXRecLockAcquire(&lock->recursiveLock, GetReturnAddress());
-   }
-
-   if (MXRecLockCount(&lock->recursiveLock) > 1) {
-      MXUserDumpAndPanic(&lock->header,
-                         "%s: Acquire on an acquired exclusive lock\n",
-                         __FUNCTION__);
-   }
-
-   if (stats) {
-      stats->holdStart = Hostinfo_SystemTimerNS();
-   }
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * MXUser_ReleaseExclLock --
- *
- *      Release (unlock) an exclusive lock.
- *
- * Results:
- *      The lock is released.
- *
- * Side effects:
- *      None
- *
- *-----------------------------------------------------------------------------
- */
-
-void
-MXUser_ReleaseExclLock(MXUserExclLock *lock)  // IN/OUT:
-{
-   MXUserStats *stats;
-
-   ASSERT(lock && (lock->header.signature == MXUSER_EXCL_SIGNATURE));
-
-   stats = (MXUserStats *) Atomic_ReadPtr(&lock->statsMem);
-
-   if (stats) {
-      VmTimeType value = Hostinfo_SystemTimerNS() - stats->holdStart;
-      MXUserHisto *histo;
-
-      MXUserBasicStatsSample(&stats->heldStats, value);
-
-      histo = Atomic_ReadPtr(&stats->heldHisto);
-
-      if (UNLIKELY(histo != NULL)) {
-         MXUserHistoSample(histo, value);
-      }
-   }
-
-   if (!MXRecLockIsOwner(&lock->recursiveLock)) {
-      uint32 lockCount = MXRecLockCount(&lock->recursiveLock);
-
-      MXUserDumpAndPanic(&lock->header,
-                         "%s: Non-owner release of an %s exclusive lock\n",
-                         __FUNCTION__,
-                         lockCount == 0 ? "unacquired" : "acquired");
-   }
-
-   MXUserReleaseTracking(&lock->header);
-
-   MXRecLockRelease(&lock->recursiveLock);
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * MXUser_TryAcquireExclLock --
- *
- *      An attempt is made to conditionally acquire (lock) an exclusive lock.
- *
- * Results:
- *      TRUE    Acquired (locked)
- *      FALSE   Not acquired
- *
- * Side effects:
- *      None
- *
- * NOTE:
- *      A "TryAcquire" does not rank check should the acquisition succeed.
- *      This duplicates the behavor of MX locks.
- *
- *-----------------------------------------------------------------------------
- */
-
-Bool
-MXUser_TryAcquireExclLock(MXUserExclLock *lock)  // IN/OUT:
-{
-   Bool success;
-   MXUserStats *stats;
-
-   ASSERT(lock && (lock->header.signature == MXUSER_EXCL_SIGNATURE));
-
-   if (MXUserTryAcquireFail(lock->header.name)) {
-      return FALSE;
-   }
-
-   success = MXRecLockTryAcquire(&lock->recursiveLock, GetReturnAddress());
-
-   if (success) {
-      MXUserAcquisitionTracking(&lock->header, FALSE);
-
-      if (MXRecLockCount(&lock->recursiveLock) > 1) {
-         MXUserDumpAndPanic(&lock->header,
-                            "%s: Acquire on an acquired exclusive lock\n",
-                            __FUNCTION__);
-      }
-   }
-
-   stats = (MXUserStats *) Atomic_ReadPtr(&lock->statsMem);
-
-   if (stats) {
-      MXUserAcquisitionSample(&stats->acquisitionStats, success, !success,
-                              0ULL);
-   }
-
-   return success;
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * MXUser_IsCurThreadHoldingExclLock --
- *
- *      Is an exclusive lock held by the calling thread?
- *
- * Results:
- *      TRUE    Yes
- *      FALSE   No
- *
- * Side effects:
- *      None
- *
- *-----------------------------------------------------------------------------
- */
-
-Bool
-MXUser_IsCurThreadHoldingExclLock(const MXUserExclLock *lock)  // IN:
-{
-   ASSERT(lock && (lock->header.signature == MXUSER_EXCL_SIGNATURE));
-
-   return MXRecLockIsOwner(&lock->recursiveLock);
 }
 
 
@@ -540,6 +226,32 @@ MXUser_ControlExclLock(MXUserExclLock *lock,  // IN/OUT:
       break;
    }
 
+   case MXUSER_CONTROL_ENABLE_STATS: {
+      MXUserStats *stats = (MXUserStats *) Atomic_ReadPtr(&lock->statsMem);
+
+      if (LIKELY(stats == NULL)) {
+         MXUserStats *before;
+
+         stats = Util_SafeCalloc(1, sizeof(*stats));
+
+         MXUserAcquisitionStatsSetUp(&stats->acquisitionStats);
+         MXUserBasicStatsSetUp(&stats->heldStats, MXUSER_STAT_CLASS_HELD);
+
+         before = (MXUserStats *) Atomic_ReadIfEqualWritePtr(&lock->statsMem,
+                                                             NULL,
+                                                             (void *) stats);
+
+         if (before) {
+            free(stats);
+         }
+
+         lock->header.statsFunc = MXUserStatsActionExcl;
+      }
+
+      result = TRUE;
+      break;
+   }
+
    default:
       result = FALSE;
    }
@@ -548,6 +260,316 @@ MXUser_ControlExclLock(MXUserExclLock *lock,  // IN/OUT:
 }
 
 
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * MXUser_CreateExclLock --
+ *
+ *      Create an exclusive lock.
+ *
+ * Results:
+ *      NULL  Creation failed
+ *      !NULL Creation succeeded
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+MXUserExclLock *
+MXUser_CreateExclLock(const char *userName,  // IN:
+                      MX_Rank rank)          // IN:
+{
+   char *properName;
+   MXUserExclLock *lock;
+
+   lock = Util_SafeCalloc(1, sizeof(*lock));
+
+   if (userName == NULL) {
+      properName = Str_SafeAsprintf(NULL, "X-%p", GetReturnAddress());
+   } else {
+      properName = Util_SafeStrdup(userName);
+   }
+
+   if (!MXRecLockInit(&lock->recursiveLock)) {
+      free(properName);
+      free(lock);
+
+      return NULL;
+   }
+
+   lock->header.signature = MXUSER_EXCL_SIGNATURE;
+   lock->header.name = properName;
+   lock->header.rank = rank;
+   lock->header.serialNumber = MXUserAllocSerialNumber();
+   lock->header.dumpFunc = MXUserDumpExclLock;
+
+   if (MXUserStatsEnabled()) {
+      MXUser_ControlExclLock(lock, MXUSER_CONTROL_ENABLE_STATS);
+   } else {
+      lock->header.statsFunc = NULL;
+      Atomic_WritePtr(&lock->statsMem, NULL);
+   }
+
+   MXUserAddToList(&lock->header);
+
+   return lock;
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * MXUser_DestroyExclLock --
+ *
+ *      Destroy an exclusive lock.
+ *
+ * Results:
+ *      Lock is destroyed. Don't use the pointer again.
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+void
+MXUser_DestroyExclLock(MXUserExclLock *lock)  // IN:
+{
+   if (lock != NULL) {
+      MXUserStats *stats;
+
+      ASSERT(lock->header.signature == MXUSER_EXCL_SIGNATURE);
+
+      if (MXRecLockCount(&lock->recursiveLock) > 0) {
+         MXUserDumpAndPanic(&lock->header,
+                            "%s: Destroy of an acquired exclusive lock\n",
+                            __FUNCTION__);
+      }
+
+      MXRecLockDestroy(&lock->recursiveLock);
+
+      MXUserRemoveFromList(&lock->header);
+
+      stats = (MXUserStats *) Atomic_ReadPtr(&lock->statsMem);
+
+      if (stats) {
+         MXUserAcquisitionStatsTearDown(&stats->acquisitionStats);
+         MXUserBasicStatsTearDown(&stats->heldStats);
+         MXUserHistoTearDown(Atomic_ReadPtr(&stats->acquisitionHisto));
+         MXUserHistoTearDown(Atomic_ReadPtr(&stats->heldHisto));
+
+         free(stats);
+      }
+
+      lock->header.signature = 0;  // just in case...
+      free(lock->header.name);
+      lock->header.name = NULL;
+      free(lock);
+   }
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * MXUser_AcquireExclLock --
+ *
+ *      An acquisition is made (lock is taken) on the specified exclusive lock.
+ *
+ * Results:
+ *      The lock is acquired (locked).
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+void
+MXUser_AcquireExclLock(MXUserExclLock *lock)  // IN/OUT:
+{
+   MXUserStats *stats;
+
+   ASSERT(lock && (lock->header.signature == MXUSER_EXCL_SIGNATURE));
+
+   stats = (MXUserStats *) Atomic_ReadPtr(&lock->statsMem);
+
+   MXUserAcquisitionTracking(&lock->header, TRUE);
+
+   if (stats) {
+      VmTimeType begin = Hostinfo_SystemTimerNS();
+      VmTimeType value;
+      Bool contended;
+      MXUserHisto *histo;
+
+      contended = MXRecLockAcquire(&lock->recursiveLock);
+
+      value = Hostinfo_SystemTimerNS() - begin;
+
+      MXUserAcquisitionSample(&stats->acquisitionStats, TRUE, contended,
+                              value);
+
+      stats->holder = GetReturnAddress();
+
+      histo = Atomic_ReadPtr(&stats->acquisitionHisto);
+
+      if (UNLIKELY(histo != NULL)) {
+         MXUserHistoSample(histo, value, stats->holder);
+      }
+   } else {
+      MXRecLockAcquire(&lock->recursiveLock);
+   }
+
+   if (MXRecLockCount(&lock->recursiveLock) > 1) {
+      MXUserDumpAndPanic(&lock->header,
+                         "%s: Acquire on an acquired exclusive lock\n",
+                         __FUNCTION__);
+   }
+
+   if (stats) {
+      stats->holdStart = Hostinfo_SystemTimerNS();
+   }
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * MXUser_ReleaseExclLock --
+ *
+ *      Release (unlock) an exclusive lock.
+ *
+ * Results:
+ *      The lock is released.
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+void
+MXUser_ReleaseExclLock(MXUserExclLock *lock)  // IN/OUT:
+{
+   MXUserStats *stats;
+
+   ASSERT(lock && (lock->header.signature == MXUSER_EXCL_SIGNATURE));
+
+   stats = (MXUserStats *) Atomic_ReadPtr(&lock->statsMem);
+
+   if (stats) {
+      VmTimeType value = Hostinfo_SystemTimerNS() - stats->holdStart;
+      MXUserHisto *histo;
+
+      MXUserBasicStatsSample(&stats->heldStats, value);
+
+      histo = Atomic_ReadPtr(&stats->heldHisto);
+
+      if (UNLIKELY(histo != NULL)) {
+         MXUserHistoSample(histo, value, stats->holder);
+         stats->holder = NULL;
+      }
+   }
+
+   if (!MXRecLockIsOwner(&lock->recursiveLock)) {
+      uint32 lockCount = MXRecLockCount(&lock->recursiveLock);
+
+      MXUserDumpAndPanic(&lock->header,
+                         "%s: Non-owner release of an %s exclusive lock\n",
+                         __FUNCTION__,
+                         lockCount == 0 ? "unacquired" : "acquired");
+   }
+
+   MXUserReleaseTracking(&lock->header);
+
+   MXRecLockRelease(&lock->recursiveLock);
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * MXUser_TryAcquireExclLock --
+ *
+ *      An attempt is made to conditionally acquire (lock) an exclusive lock.
+ *
+ * Results:
+ *      TRUE    Acquired (locked)
+ *      FALSE   Not acquired
+ *
+ * Side effects:
+ *      None
+ *
+ * NOTE:
+ *      A "TryAcquire" does not rank check should the acquisition succeed.
+ *      This duplicates the behavor of MX locks.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+Bool
+MXUser_TryAcquireExclLock(MXUserExclLock *lock)  // IN/OUT:
+{
+   Bool success;
+   MXUserStats *stats;
+
+   ASSERT(lock && (lock->header.signature == MXUSER_EXCL_SIGNATURE));
+
+   if (MXUserTryAcquireFail(lock->header.name)) {
+      return FALSE;
+   }
+
+   success = MXRecLockTryAcquire(&lock->recursiveLock);
+
+   if (success) {
+      MXUserAcquisitionTracking(&lock->header, FALSE);
+
+      if (MXRecLockCount(&lock->recursiveLock) > 1) {
+         MXUserDumpAndPanic(&lock->header,
+                            "%s: Acquire on an acquired exclusive lock\n",
+                            __FUNCTION__);
+      }
+   }
+
+   stats = (MXUserStats *) Atomic_ReadPtr(&lock->statsMem);
+
+   if (stats) {
+      MXUserAcquisitionSample(&stats->acquisitionStats, success, !success,
+                              0ULL);
+   }
+
+   return success;
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * MXUser_IsCurThreadHoldingExclLock --
+ *
+ *      Is an exclusive lock held by the calling thread?
+ *
+ * Results:
+ *      TRUE    Yes
+ *      FALSE   No
+ *
+ * Side effects:
+ *      None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+Bool
+MXUser_IsCurThreadHoldingExclLock(const MXUserExclLock *lock)  // IN:
+{
+   ASSERT(lock && (lock->header.signature == MXUSER_EXCL_SIGNATURE));
+
+   return MXRecLockIsOwner(&lock->recursiveLock);
+}
+
+
 /*
  *-----------------------------------------------------------------------------
  *
@@ -634,7 +656,8 @@ MXUser_CreateCondVarExclLock(MXUserExclLock *lock)
  *      As above.
  *
  * Side effects:
- *      None.
+ *      It is possible to return from this routine without the condtion
+ *      variable having been signalled (spurious wake up); code accordingly!
  *
  *-----------------------------------------------------------------------------
  */
@@ -660,22 +683,21 @@ MXUser_WaitCondVarExclLock(MXUserExclLock *lock,    // IN:
  *      blocking and is reacquired before returning from this function.
  *
  * Results:
- *      TRUE   condVar was signalled
- *      FALSE  timed out waiting for signal
+ *      As above
  *
  * Side effects:
- *      None.
+ *      It is possible to return from this routine without the condtion
+ *      variable having been signalled (spurious wake up); code accordingly!
  *
  *-----------------------------------------------------------------------------
  */
 
-Bool
+void
 MXUser_TimedWaitCondVarExclLock(MXUserExclLock *lock,    // IN:
                                 MXUserCondVar *condVar,  // IN:
                                 uint32 msecWait)         // IN:
 {
    ASSERT(lock && (lock->header.signature == MXUSER_EXCL_SIGNATURE));
 
-   return MXUserWaitCondVar(&lock->header, &lock->recursiveLock, condVar,
-                            msecWait);
+   MXUserWaitCondVar(&lock->header, &lock->recursiveLock, condVar, msecWait);
 }
