@@ -31,6 +31,12 @@
 
 #define MXUSER_RW_SIGNATURE 0x57524B4C // 'LKRW' in memory
 
+static void
+MXUserFreeHashEntry(void *data)  // IN:
+{
+   free(data);
+}
+
 
 /*
  * Environment specific implementations of portable read-write locks.
@@ -96,6 +102,8 @@ MXUserNativeRWSupported(void)
                    (pReleaseSRWLockShared != NULL) &&
                    (pAcquireSRWLockExclusive != NULL) &&
                    (pReleaseSRWLockExclusive != NULL));
+
+         COMPILER_MEM_BARRIER();
       } else {
          result = FALSE;
       }
@@ -235,8 +243,9 @@ typedef enum {
 } HolderState;
 
 typedef struct {
-   HolderState  state;
-   VmTimeType   holdStart;
+   HolderState   state;
+   void         *holder;
+   VmTimeType    holdStart;
 } HolderContext;
 
 typedef struct {
@@ -262,7 +271,6 @@ struct MXUserRWLock
 };
 
 
-#if defined(MXUSER_STATS)
 /*
  *-----------------------------------------------------------------------------
  *
@@ -331,7 +339,6 @@ MXUserStatsActionRW(MXUserHeader *header)  // IN:
       }
    }
 }
-#endif
 
 
 /*
@@ -360,21 +367,127 @@ MXUserDumpRWLock(MXUserHeader *header)  // IN:
    Warning("\tsignature 0x%X\n", lock->header.signature);
    Warning("\tname %s\n", lock->header.name);
    Warning("\trank 0x%X\n", lock->header.rank);
+   Warning("\tserial number %u\n", lock->header.serialNumber);
 
    if (LIKELY(lock->useNative)) {
       Warning("\tnativeLock 0x%p\n", &lock->nativeLock);
    } else {
       Warning("\tcount %u\n", lock->recursiveLock.referenceCount);
-
-#if defined(MXUSER_DEBUG)
-      Warning("\tcaller 0x%p\n", lock->recursiveLock.ownerRetAddr);
-#endif
    }
 
    Warning("\tholderCount %d\n", Atomic_Read(&lock->holderCount));
 }
 
 
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * MXUser_ControlRWLock --
+ *
+ *      Perform the specified command on the specified lock.
+ *
+ * Results:
+ *      TRUE    succeeded
+ *      FALSE   failed
+ *
+ * Side effects:
+ *      Depends on the command, no?
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+Bool
+MXUser_ControlRWLock(MXUserRWLock *lock,  // IN/OUT:
+                     uint32 command,      // IN:
+                     ...)                 // IN:
+{
+   Bool result;
+
+   ASSERT(lock && (lock->header.signature == MXUSER_RW_SIGNATURE));
+
+   switch (command) {
+   case MXUSER_CONTROL_ACQUISITION_HISTO: {
+      MXUserStats *stats = (MXUserStats *) Atomic_ReadPtr(&lock->statsMem);
+
+      if (stats) {
+         va_list a;
+         uint64 minValue;
+         uint32 decades;
+
+         va_start(a, command);
+         minValue = va_arg(a, uint64);
+         decades = va_arg(a, uint32);
+         va_end(a);
+
+         MXUserForceHisto(&stats->acquisitionHisto,
+                          MXUSER_STAT_CLASS_ACQUISITION, minValue, decades);
+
+         result = TRUE;
+      } else {
+         result = FALSE;
+      }
+
+      break;
+   }
+
+   case MXUSER_CONTROL_HELD_HISTO: {
+      MXUserStats *stats = (MXUserStats *) Atomic_ReadPtr(&lock->statsMem);
+
+      if (stats) {
+         va_list a;
+         uint32 minValue;
+         uint32 decades;
+
+         va_start(a, command);
+         minValue = va_arg(a, uint64);
+         decades = va_arg(a, uint32);
+         va_end(a);
+
+         MXUserForceHisto(&stats->heldHisto, MXUSER_STAT_CLASS_HELD,
+                          minValue, decades);
+
+         result = TRUE;
+      } else {
+         result = FALSE;
+      }
+
+      break;
+   }
+
+   case MXUSER_CONTROL_ENABLE_STATS: {
+      MXUserStats *stats = (MXUserStats *) Atomic_ReadPtr(&lock->statsMem);
+
+      if (LIKELY(stats == NULL)) {
+         MXUserStats *before;
+
+         stats = Util_SafeCalloc(1, sizeof(*stats));
+
+         MXUserAcquisitionStatsSetUp(&stats->acquisitionStats);
+         MXUserBasicStatsSetUp(&stats->heldStats, MXUSER_STAT_CLASS_HELD);
+
+         before = (MXUserStats *) Atomic_ReadIfEqualWritePtr(&lock->statsMem,
+                                                             NULL,
+                                                             (void *) stats);
+
+         if (before) {
+            free(stats);
+         }
+
+         lock->header.statsFunc = MXUserStatsActionRW;
+      }
+
+      result = TRUE;
+      break;
+   }
+
+   default:
+      result = FALSE;
+   }
+
+   return result;
+}
+
+
 /*
  *-----------------------------------------------------------------------------
  *
@@ -395,12 +508,6 @@ MXUserDumpRWLock(MXUserHeader *header)  // IN:
  *-----------------------------------------------------------------------------
  */
 
-static void
-MXUserFreeHashEntry(void *data)  // IN:
-{
-   free(data);
-}
-
 MXUserRWLock *
 MXUser_CreateRWLock(const char *userName,  // IN:
                     MX_Rank rank)          // IN:
@@ -423,6 +530,12 @@ MXUser_CreateRWLock(const char *userName,  // IN:
       properName = Util_SafeStrdup(userName);
    }
 
+   lock->header.signature = MXUSER_RW_SIGNATURE;
+   lock->header.name = properName;
+   lock->header.rank = rank;
+   lock->header.serialNumber = MXUserAllocSerialNumber();
+   lock->header.dumpFunc = MXUserDumpRWLock;
+
    /*
     * Always attempt to use native locks when they are available. If, for some
     * reason, a native lock should be available but isn't, fall back to using
@@ -431,48 +544,26 @@ MXUser_CreateRWLock(const char *userName,  // IN:
 
    lock->useNative = useNative && MXUserNativeRWInit(&lock->nativeLock);
 
-   if (lock->useNative) {
-#if defined(MXUSER_STATS)
-      /* stats builds need an internal recursive lock for data integrity */
-
-      lockInited = MXRecLockInit(&lock->recursiveLock);
-
-      if (!lockInited) {
-         MXUserNativeRWDestroy(&lock->nativeLock);
-      }
-#else
-      lockInited = TRUE;
-#endif
-   } else {
-      lockInited = MXRecLockInit(&lock->recursiveLock);
-   }
+   lockInited = MXRecLockInit(&lock->recursiveLock);
 
    if (LIKELY(lockInited)) {
-      MXUserStats *stats;
-
       lock->holderTable = HashTable_Alloc(256,
                                           HASH_INT_KEY | HASH_FLAG_ATOMIC,
                                           MXUserFreeHashEntry);
 
-      lock->header.name = properName;
-      lock->header.signature = MXUSER_RW_SIGNATURE;
-      lock->header.rank = rank;
-      lock->header.dumpFunc = MXUserDumpRWLock;
-
-#if defined(MXUSER_STATS)
-      lock->header.statsFunc = MXUserStatsActionRW;
-      lock->header.identifier = MXUserAllocID();
-
-      stats = Util_SafeCalloc(1, sizeof(*stats));
-
-      MXUserAcquisitionStatsSetUp(&stats->acquisitionStats);
-      MXUserBasicStatsSetUp(&stats->heldStats, MXUSER_STAT_CLASS_HELD);
-#else
-      stats = NULL;
-#endif
+      if (MXUserStatsEnabled()) {
+         MXUser_ControlRWLock(lock, MXUSER_CONTROL_ENABLE_STATS);
+      } else {
+         lock->header.statsFunc = NULL;
+         Atomic_WritePtr(&lock->statsMem, NULL);
+      }
 
       MXUserAddToList(&lock->header);
    } else {
+      if (lock->useNative) {
+         MXUserNativeRWDestroy(&lock->nativeLock);
+      }
+
       free(properName);
       free(lock);
       lock = NULL;
@@ -519,14 +610,10 @@ MXUser_DestroyRWLock(MXUserRWLock *lock)  // IN:
             MXUserDumpAndPanic(&lock->header, "%s: Internal error (%d)\n",
                                __FUNCTION__, err);
          }
-
-#if defined(MXUSER_STATS)
-         MXRecLockDestroy(&lock->recursiveLock);  
-#endif
-      } else {
-         MXRecLockDestroy(&lock->recursiveLock);  
       }
 
+      MXRecLockDestroy(&lock->recursiveLock);  
+
       MXUserRemoveFromList(&lock->header);
 
       stats = (MXUserStats *) Atomic_ReadPtr(&lock->statsMem);
@@ -542,7 +629,7 @@ MXUser_DestroyRWLock(MXUserRWLock *lock)  // IN:
 
       HashTable_FreeUnsafe(lock->holderTable);
       lock->header.signature = 0;  // just in case...
-      free((void *) lock->header.name);  // avoid const warnings
+      free(lock->header.name);
       lock->header.name = NULL;
       free(lock);
    }
@@ -578,6 +665,7 @@ MXUserGetHolderContext(MXUserRWLock *lock)  // IN:
       HolderContext *newContext = Util_SafeMalloc(sizeof(HolderContext));
 
       newContext->holdStart = 0;
+      newContext->holder = NULL;
       newContext->state = RW_UNLOCKED;
 
       result = HashTable_LookupOrInsert(lock->holderTable, threadID,
@@ -641,8 +729,7 @@ MXUserAcquisition(MXUserRWLock *lock,  // IN/OUT:
 
       contended = lock->useNative ? MXUserNativeRWAcquire(&lock->nativeLock,
                                                           forRead, &err) :
-                                    MXRecLockAcquire(&lock->recursiveLock,
-                                                     GetReturnAddress());
+                                    MXRecLockAcquire(&lock->recursiveLock);
 
       value = Hostinfo_SystemTimerNS() - begin;
 
@@ -657,7 +744,7 @@ MXUserAcquisition(MXUserRWLock *lock,  // IN/OUT:
        */
 
       if (forRead && lock->useNative) {
-         MXRecLockAcquire(&lock->recursiveLock, GetReturnAddress());
+         MXRecLockAcquire(&lock->recursiveLock);
       }
 
       MXUserAcquisitionSample(&stats->acquisitionStats, TRUE, contended,
@@ -666,7 +753,7 @@ MXUserAcquisition(MXUserRWLock *lock,  // IN/OUT:
       histo = Atomic_ReadPtr(&stats->acquisitionHisto);
 
       if (UNLIKELY(histo != NULL)) {
-         MXUserHistoSample(histo, value);
+         MXUserHistoSample(histo, value, GetReturnAddress());
       }
 
       if (forRead && lock->useNative) {
@@ -676,7 +763,7 @@ MXUserAcquisition(MXUserRWLock *lock,  // IN/OUT:
       if (LIKELY(lock->useNative)) {
          MXUserNativeRWAcquire(&lock->nativeLock, forRead, &err);
       } else {
-         MXRecLockAcquire(&lock->recursiveLock, GetReturnAddress());
+         MXRecLockAcquire(&lock->recursiveLock);
       }
    }
 
@@ -688,6 +775,7 @@ MXUserAcquisition(MXUserRWLock *lock,  // IN/OUT:
    myContext->state = forRead ? RW_LOCKED_FOR_READ : RW_LOCKED_FOR_WRITE;
 
    if (stats) {
+      myContext->holder = GetReturnAddress();
       myContext->holdStart = Hostinfo_SystemTimerNS();
    }
 }
@@ -780,11 +868,7 @@ MXUser_IsCurThreadHoldingRWLock(MXUserRWLock *lock,  // IN:
       return myContext->state != RW_UNLOCKED;
 
    default:
-#if defined(MXUSER_DEBUG)
       Panic("%s: unknown query type %d\n", __FUNCTION__, queryType);
-#else   
-      return FALSE;
-#endif  
    }    
 }
 
@@ -814,29 +898,20 @@ MXUser_ReleaseRWLock(MXUserRWLock *lock)  // IN/OUT:
    ASSERT(lock && (lock->header.signature == MXUSER_RW_SIGNATURE));
 
    myContext = MXUserGetHolderContext(lock);
+
    stats = (MXUserStats *) Atomic_ReadPtr(&lock->statsMem);
 
    if (stats) {
       VmTimeType duration = Hostinfo_SystemTimerNS() - myContext->holdStart;
       MXUserHisto *histo;
 
-      if (UNLIKELY(myContext->state == RW_UNLOCKED)) {
-         uint32 lockCount = Atomic_Read(&lock->holderCount);
-
-         MXUserDumpAndPanic(&lock->header,
-                            "%s: Non-owner release of an %s read-write lock\n",
-                            __FUNCTION__,
-                            lockCount == 0 ? "unacquired" : "acquired");
-      }
-
-
       /*
        * The statistics are not always atomically safe so protect them
        * when necessary
        */
 
       if ((myContext->state == RW_LOCKED_FOR_READ) && lock->useNative) {
-         MXRecLockAcquire(&lock->recursiveLock, GetReturnAddress());
+         MXRecLockAcquire(&lock->recursiveLock);
       }
 
       MXUserBasicStatsSample(&stats->heldStats, duration);
@@ -844,7 +919,8 @@ MXUser_ReleaseRWLock(MXUserRWLock *lock)  // IN/OUT:
       histo = Atomic_ReadPtr(&stats->heldHisto);
 
       if (UNLIKELY(histo != NULL)) {
-         MXUserHistoSample(histo, duration);
+         MXUserHistoSample(histo, duration, myContext->holder);
+         myContext->holder = NULL;
       }
 
       if ((myContext->state == RW_LOCKED_FOR_READ) && lock->useNative) {
@@ -852,12 +928,22 @@ MXUser_ReleaseRWLock(MXUserRWLock *lock)  // IN/OUT:
       }
    }
 
+   if (UNLIKELY(myContext->state == RW_UNLOCKED)) {
+      uint32 lockCount = Atomic_Read(&lock->holderCount);
+
+      MXUserDumpAndPanic(&lock->header,
+                         "%s: Non-owner release of an %s read-write lock\n",
+                         __FUNCTION__,
+                         lockCount == 0 ? "unacquired" : "acquired");
+   }
+
    MXUserReleaseTracking(&lock->header);
 
    Atomic_Dec(&lock->holderCount);
 
    if (LIKELY(lock->useNative)) {
-      int err = MXUserNativeRWRelease(&lock->nativeLock, myContext->state);
+      int err = MXUserNativeRWRelease(&lock->nativeLock,
+                                      myContext->state == RW_LOCKED_FOR_READ);
 
       if (UNLIKELY(err != 0)) {
          MXUserDumpAndPanic(&lock->header, "%s: Internal error (%d)\n",
