@@ -91,9 +91,13 @@
 #define CODESET_CAN_FALLBACK_ON_NON_ICU TRUE
 
 #if defined(__APPLE__)
-#define POSIX_ICU_DIR DEFAULT_LIBDIRECTORY "/icu"
+#   define POSIX_ICU_DIR DEFAULT_LIBDIRECTORY "/icu"
 #elif !defined(WIN32)
-#define POSIX_ICU_DIR "/etc/vmware/icu"
+#   if defined(VMX86_TOOLS)
+#      define POSIX_ICU_DIR "/etc/vmware-tools/icu"
+#   else
+#      define POSIX_ICU_DIR "/etc/vmware/icu"
+#   endif
 #endif
 
 /*
@@ -1614,7 +1618,7 @@ CodeSet_IsEncodingSupported(const char *name) // IN
 
 Bool
 CodeSet_Validate(const char *buf,   // IN: the string
-                 size_t size,	   // IN: length of string
+                 size_t size,	    // IN: length of string
                  const char *code)  // IN: encoding
 {
    UConverter *cv;
