@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2010-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2010 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -25,34 +25,8 @@
 #include <stdlib.h>
 #include "vmware.h"
 #include "codeset.h"
-#include "codesetOld.h"
 #include "util.h"
 
-
-/*
- *-----------------------------------------------------------------------------
- *
- * CodeSet_GetCurrentCodeSet --
- *
- *    Return native code set name. Always calls down to
- *    CodeSetOld_GetCurrentCodeSet. See there for more details.
- *
- * Results:
- *    See CodeSetOld_GetCurrentCodeSet.
- *
- * Side effects:
- *    See CodeSetOld_GetCurrentCodeSet.
- *
- *-----------------------------------------------------------------------------
- */
-
-const char *
-CodeSet_GetCurrentCodeSet(void)
-{
-   return CodeSetOld_GetCurrentCodeSet();
-}
-
-
 /*
  *-----------------------------------------------------------------------------
  *
@@ -73,7 +47,7 @@ CodeSet_GetCurrentCodeSet(void)
 int
 CodeSet_GetUtf8(const char *string,  // IN: string
                 const char *end,     // IN: end of string
-                uint32 *uchar)       // OUT/OPT: the Unicode character
+                uint32 *uchar)       // OUT: the Unicode character
 {
    uint8 *p = (uint8 *) string;
    uint8 *e;
@@ -181,7 +155,8 @@ CodeSet_LengthInCodePoints(const char *utf8)  // IN:
    end = p + strlen(utf8);
 
    while (p < end) {
-      uint32 len = CodeSet_GetUtf8(p, end, NULL);
+      uint32 utf32;
+      uint32 len = CodeSet_GetUtf8(p, end, &utf32);
 
       if (len == 0) {
          return -1;
@@ -195,55 +170,6 @@ CodeSet_LengthInCodePoints(const char *utf8)  // IN:
 }
 
 
-/*
- *-----------------------------------------------------------------------------
- *
- * CodeSet_CodePointOffsetToByteOffset --
- *
- *    Return the byte offset of the character at the given codepoint
- *    offset.
- *
- * Results:
- *    -1 on error
- *
- * Side effects:
- *    None
- *
- *-----------------------------------------------------------------------------
- */
-
-int
-CodeSet_CodePointOffsetToByteOffset(const char *utf8,    // IN
-                                    int codePointOffset) // IN
-{
-   const char *p;
-   const char *end;
-
-   ASSERT(utf8);
-
-   p = utf8;
-   end = p + strlen(utf8);
-
-   while (p < end && codePointOffset > 0) {
-      uint32 utf32;
-      uint32 len = CodeSet_GetUtf8(p, end, &utf32);
-
-      if (len == 0) {
-         return -1;
-      }
-
-      p += len;
-      codePointOffset--;
-   }
-
-   if (codePointOffset == 0) {
-      return p - utf8;
-   } else {
-      return -1;
-   }
-}
-
-
 /*
  *-----------------------------------------------------------------------------
  *
@@ -289,7 +215,7 @@ CodeSet_UTF8ToUTF32(const char *utf8,  // IN:
    p = (char *) utf8;
    end = p + strlen(utf8);
 
-   ptr = Util_SafeMalloc(sizeof *ptr * (codePoints + 1));
+   ptr = Util_SafeMalloc(sizeof(*ptr) * (codePoints + 1));
    *utf32 = (char *) ptr;
 
    while (p < end) {
