@@ -311,8 +311,7 @@ HashTable_AllocOnce(Atomic_Ptr *var,          // IN/OUT: the atomic var
       if (ht == NULL) {
          ht = new;
       } else {
-         new->atomic = FALSE;
-         HashTable_Free(new);
+         HashTable_FreeUnsafe(new);
       }
    }
    ASSERT(ht == Atomic_ReadPtr(var));
@@ -321,35 +320,6 @@ HashTable_AllocOnce(Atomic_Ptr *var,          // IN/OUT: the atomic var
 }
 
 
-/*
- *----------------------------------------------------------------------
- *
- * HashTable_Free --
- *
- *      Free the hash table skeleton.
- *
- * Results:
- *      None.
- *
- * Side effects:
- *      None.
- *
- *----------------------------------------------------------------------
- */
-
-void
-HashTable_Free(HashTable *ht)  // IN/OUT:
-{
-   ASSERT(ht);
-   ASSERT(!ht->atomic);
-
-   HashTable_Clear(ht);
-
-   free(ht->buckets);
-   free(ht);
-}
-
-
 /*
  *----------------------------------------------------------------------
  *
@@ -366,14 +336,11 @@ HashTable_Free(HashTable *ht)  // IN/OUT:
  *----------------------------------------------------------------------
  */
 
-void
-HashTable_Clear(HashTable *ht)  // IN/OUT:
+static void
+HashTableClearInternal(HashTable *ht)  // IN/OUT:
 {
    int i;
 
-   ASSERT(ht);
-   ASSERT(!ht->atomic);
-
    ht->numElements = 0;
 
    for (i = 0; i < ht->numEntries; i++) {
@@ -393,6 +360,76 @@ HashTable_Clear(HashTable *ht)  // IN/OUT:
 }
 
 
+void
+HashTable_Clear(HashTable *ht)  // IN/OUT:
+{
+   ASSERT(ht);
+   ASSERT(!ht->atomic);
+
+   HashTableClearInternal(ht);
+}
+
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * HashTable_Free --
+ *
+ *      Free the hash table skeleton. Do not call this if the hashTable is
+ *      atomic; use HashTable_FreeUnsafe and HEED THE RESTRICTIONS!
+ *
+ * Results:
+ *      None.
+ *
+ * Side effects:
+ *      None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+void
+HashTable_Free(HashTable *ht)  // IN/OUT:
+{
+   ASSERT(ht);
+   ASSERT(!ht->atomic);
+
+   HashTableClearInternal(ht);
+
+   free(ht->buckets);
+   free(ht);
+}
+
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * HashTable_FreeUnsafe --
+ *
+ *      HashTable_Free for atomic hash tables.  Non-atomic hash tables
+ *      should always use HashTable_Free.  The caller should make sure
+ *      no other thread may access the hash table when this is called.
+ *
+ * Results:
+ *      None.
+ *
+ * Side effects:
+ *      None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+void
+HashTable_FreeUnsafe(HashTable *ht)  // IN/OUT:
+{
+   ASSERT(ht);
+
+   HashTableClearInternal(ht);
+
+   free(ht->buckets);
+   free(ht);
+}
+
+
 /*
  *----------------------------------------------------------------------
  *
