@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2011-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2011 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -62,7 +62,7 @@ Hostinfo_HypervisorCPUIDSig(void)
    CPUIDRegs regs;
 
    __GET_CPUID(1, &regs);
-   if (!CPUID_ISSET(1, ECX, HYPERVISOR, regs.ecx)) {
+   if (!(regs.ecx & CPUID_FEATURE_COMMON_ID1ECX_HYPERVISOR)) {
       return NULL;
    }
 
@@ -489,7 +489,7 @@ Hostinfo_VCPUInfoBackdoor(unsigned bit)
 /*
  *----------------------------------------------------------------------
  *
- * Hostinfo_TouchBackDoor --
+ *  Hostinfo_TouchBackDoor --
  *
  *      Access the backdoor. This is used to determine if we are
  *      running in a VM or on a physical host. On a physical host
@@ -505,7 +505,7 @@ Hostinfo_VCPUInfoBackdoor(unsigned bit)
  *      if not.
  *
  * Side effects:
- *      Exception if not in a VM.
+ *	Exception if not in a VM.
  *
  *----------------------------------------------------------------------
  */
@@ -513,7 +513,13 @@ Hostinfo_VCPUInfoBackdoor(unsigned bit)
 Bool
 Hostinfo_TouchBackDoor(void)
 {
-#if defined(__i386__) || defined(__x86_64__)
+   /*
+    * XXX: This can cause Apple's Crash Reporter to erroneously display
+    * a crash, even though the process has caught the SIGILL and handled
+    * it.
+    */
+
+#if !defined(__APPLE__) && (defined(__i386__) || defined(__x86_64__))
    uint32 eax;
    uint32 ebx;
    uint32 ecx;
