@@ -16,6 +16,7 @@
  *
  *********************************************************/
 
+#define _GNU_SOURCE
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -79,6 +80,10 @@
 #include <paths.h>
 #endif
 
+#ifdef __linux__
+#include <dlfcn.h>
+#endif
+
 #if !defined(_PATH_DEVNULL)
 #define _PATH_DEVNULL "/dev/null"
 #endif
@@ -156,7 +161,6 @@ typedef struct distro_info {
    char *filename;
 } DistroInfo;
 
-
 static const DistroInfo distroArray[] = {
    {"RedHat",             "/etc/redhat-release"},
    {"RedHat",             "/etc/redhat_version"},
@@ -436,6 +440,7 @@ Hostinfo_GetSystemBitness(void)
 }
 
 
+#if !defined __APPLE__
 /*
  *-----------------------------------------------------------------------------
  *
@@ -555,13 +560,14 @@ HostinfoGetOSShortName(char *distro,         // IN: full distro name
       } else if (strstr(distroLower, "5.0")) {
          Str_Strcpy(distroShort, STR_OS_DEBIAN_5, distroShortSize);
       }
-   } else if (StrUtil_StartsWith(distroLower, "enterprise linux")) {
+   } else if (StrUtil_StartsWith(distroLower, "enterprise linux") ||
+              StrUtil_StartsWith(distroLower, "oracle")) {
       /*
        * [root@localhost ~]# lsb_release -sd
        * "Enterprise Linux Enterprise Linux Server release 5.4 (Carthage)"
        *
        * Not sure why they didn't brand their releases as "Oracle Enterprise
-       * Linux".  Oh well.
+       * Linux". Oh well. It's fixed in 6.0, though.
        */
       Str_Strcpy(distroShort, STR_OS_ORACLE, distroShortSize);
    } else if (strstr(distroLower, "fedora")) {
@@ -715,6 +721,7 @@ out:
 
    return ret;
 }
+#endif
 
 
 /*
@@ -801,9 +808,6 @@ HostinfoGetCmdOutput(const char *cmd)  // IN:
  *
  *      Determine the OS short (.vmx format) and long names.
  *
- *      First retrieve OS information using uname, then look in
- *      /etc/xxx-release file to get the distro info.
- *
  * Return value:
  *      Returns TRUE on success and FALSE on failure.
  *
@@ -818,12 +822,8 @@ HostinfoOSData(void)
 {
    struct utsname buf;
    unsigned int lastCharPos;
-   const char *lsbCmd = "lsb_release -sd 2>/dev/null";
-   char *lsbOutput = NULL;
-
    char osName[MAX_OS_NAME_LEN];
    char osNameFull[MAX_OS_FULLNAME_LEN];
-
    static Atomic_uint32 mutex = {0};
 
    /*
@@ -844,18 +844,66 @@ HostinfoOSData(void)
    }
 
    Str_Strcpy(osName, STR_OS_EMPTY, sizeof osName);
-   Str_Sprintf(osNameFull, sizeof osNameFull, "%s %s", buf.sysname, buf.release);
+   Str_Sprintf(osNameFull, sizeof osNameFull, "%s %s", buf.sysname,
+               buf.release);
 
-   /*
-    * Check to see if this is Linux
-    * If yes, determine the distro by looking for /etc/xxx file.
-    */
+#if defined __APPLE__
+   {
+      /*
+       * The easiest way is to invoke "system_profiler" and hope that the
+       * format of its unlocalized output will never change.
+       *
+       * Alternatively, we could do what system_profiler does and use the
+       * CFPropertyList/CFDIctionary APIs to parse
+       * /System/Library/CoreServices/{Server,System}Version.plist.
+       *
+       * On a MacBookPro4,1 (and possibly other models), invoking
+       * "system_profiler" can take several seconds: it seems to spin up the CD
+       * drive as a side-effect. So use "system_profiler SPSoftwareDataType"
+       * instead.
+       */
+      SInt32 major;
+      char *sysname = HostinfoGetCmdOutput(
+                         "/usr/sbin/system_profiler SPSoftwareDataType"
+                      " | /usr/bin/grep 'System Version:'"
+                      " | /usr/bin/cut -d : -f 2");
 
+      if (sysname) {
+         char *trimmed = Unicode_Trim(sysname);
+
+         ASSERT_MEM_ALLOC(trimmed);
+         free(sysname);
+         Str_Snprintf(osNameFull, sizeof osNameFull, "%s", trimmed);
+         free(trimmed);
+      } else {
+         Log("%s: Failed to get output of system_profiler.\n", __FUNCTION__);
+         /* Fall back to returning the original osNameFull. */
+      }
+
+      if (Gestalt(gestaltSystemVersionMajor, &major) == 0) {
+         /*
+          * XXX: some places in the code refer to Mac OS X Lion (10.7?)
+          * as "darwin11", which is not what this code is doing. Once it's
+          * released and we support it, this code may need some tweaking.
+          */
+         Str_Snprintf(osName, sizeof osName, "%s%d", STR_OS_MACOS, (int) major);
+      } else if (Gestalt(gestaltSystemVersion, &major) == 0) {
+         /* Pre 10.3 Mac OS doesn't have gestaltSystemVersionMajor. */
+         NOT_TESTED();
+         major = (major & 0xFF00) >> 8;
+         Str_Snprintf(osName, sizeof osName, "%s%x", STR_OS_MACOS, (int) major);
+      } else {
+         Log("Call to Gestalt() failed!\n");
+         Str_Snprintf(osName, sizeof osName, "%s", STR_OS_MACOS);
+      }
+   }
+#else
+   // XXX Use compile-time instead of run-time checks for these as well.
    if (strstr(osNameFull, "Linux")) {
       char distro[DISTRO_BUF_SIZE];
       char distroShort[DISTRO_BUF_SIZE];
-      int i = 0;
-      int distroSize = sizeof distro;
+      static int const distroSize = sizeof distro;
+      char *lsbOutput;
 
       /*
        * Write default distro string depending on the kernel version. If
@@ -877,9 +925,10 @@ HostinfoOSData(void)
        * Try to get OS detailed information from the lsb_release command.
        */
 
-      lsbOutput = HostinfoGetCmdOutput(lsbCmd);
-
+      lsbOutput = HostinfoGetCmdOutput("lsb_release -sd 2>/dev/null");
       if (!lsbOutput) {
+         int i;
+
          /*
           * Try to get more detailed information from the version file.
           */
@@ -981,6 +1030,7 @@ HostinfoOSData(void)
       Str_Snprintf(osName, sizeof osName, "%s%s", STR_OS_SOLARIS,
                    solarisRelease);
    }
+#endif
 
    if (Hostinfo_GetSystemBitness() == 64) {
       if (strlen(osName) + sizeof STR_OS_64BIT_SUFFIX > sizeof osName) {
@@ -1020,98 +1070,93 @@ HostinfoOSData(void)
 }
 
 
-#if defined(VMX86_SERVER)
 /*
- *----------------------------------------------------------------------
+ *-----------------------------------------------------------------------------
  *
- * HostinfoReadProc --
+ * Hostinfo_CPUCounts --
  *
- *      Depending on what string is passed to it, this function parses the
- *      /proc/vmware/sched/ncpus node and returns the requested value.
+ *      Get a count of CPUs for the host.
+ *        pkgs := total number of sockets/packages
+ *        cores := total number of actual cores (not including hyperthreads)
+ *        logical := total schedulable threads, as seen by host scheduler
+ *      Depending on available host OS interfaces, these numbers may be
+ *      either "active" or "possible", so do not depend upon them for
+ *      precision.
+ *
+ *      As an example, a 2 socket Nehalem (4 cores + HT) would return:
+ *        pkgs = 2, cores = 8, logical = 16
+ *
+ *      Again, this interface is generally not useful b/c of its potential
+ *      inaccuracy (especially with hotplug!) and because it is only implemented
+ *      for a few OSes.
+ *
+ *      If you are trying to use this interface, it probably means you are doing
+ *      something 'clever' with licensing.  Don't.
  *
  * Results:
- *      A postive value on success, -1 (0xFFFFFFFF) on failure.
+ *      TRUE if sane numbers are populated, FALSE otherwise.
  *
  * Side effects:
- *      None.
+ *      None
  *
- *----------------------------------------------------------------------
- */
-
-static uint32
-HostinfoReadProc(const char *str)  // IN:
-{
-   /* XXX this should use sysinfo!! (bug 59849)
-    */
-   FILE *f;
-   char *line;
-   uint32 count;
-
-   ASSERT(!strcmp("logical", str) || !strcmp("cores", str) ||
-          !strcmp("packages", str));
-
-   ASSERT(!HostType_OSIsVMK()); // Don't use /proc/vmware
-
-   f = Posix_Fopen("/proc/vmware/sched/ncpus", "r");
-
-   if (f != NULL) {
-      while (StdIO_ReadNextLine(f, &line, 0, NULL) == StdIO_Success) {
-         if (strstr(line, str)) {
-            if (sscanf(line, "%d ", &count) == 1) {
-               free(line);
-               break;
-            }
-         }
-         free(line);
-      }
-      fclose(f);
-
-      if (count > 0) {
-         return count;
-      }
-   }
-
-   return -1;
-}
-
-
-/*
- *----------------------------------------------------------------------
- *
- * Hostinfo_HTDisabled --
- *
- *      Figure out if hyperthreading is enabled
- *
- * Results:
- *      TRUE if hyperthreading is disabled, FALSE otherwise
- *
- * Side effects:
- *      None.
- *
- *----------------------------------------------------------------------
+ *-----------------------------------------------------------------------------
  */
 
 Bool
-Hostinfo_HTDisabled(void)
+Hostinfo_CPUCounts(uint32 *logical,  // OUT
+                   uint32 *cores,    // OUT
+                   uint32 *pkgs)     // OUT
 {
-   static uint32 logical = 0, cores = 0;
+#if defined __APPLE__
+   /*
+    * Lame logic.  Because Apple doesn't really expose this info,
+    * we'd only use it for licensing anyway, and we just plain
+    * don't need it on Apple except that VMHS stuffs it somewhere
+    * and may result in a division-by-zero if we don't provide it.
+    */
+   *logical = Hostinfo_NumCPUs();
+   *pkgs = *logical > 4 ? 2 : 1;
+   *cores = *logical / *pkgs;
 
-   if (HostType_OSIsVMK()) {
-      return VMKernel_HTEnabledCPU() != VMK_OK;
+   return TRUE;
+#elif defined __linux__
+   FILE *f;
+   char *line;
+   unsigned count = 0, coresPerProc = 0, siblingsPerProc = 0;
+
+   f = Posix_Fopen("/proc/cpuinfo", "r");
+   if (f == NULL) {
+      return FALSE;
    }
 
-   if (logical == 0 && cores == 0) {
-      logical = HostinfoReadProc("logical");
-      cores = HostinfoReadProc("cores");
-
-      if (logical <= 0 || cores <= 0) {
-         logical = cores = 0;
+   while (StdIO_ReadNextLine(f, &line, 0, NULL) == StdIO_Success) {
+      if (strncmp(line, "processor", strlen("processor")) == 0) {
+         count++;
       }
+      /* Assume all processors are identical, so just read the first. */
+      if (coresPerProc == 0) {
+         sscanf(line, "cpu cores : %u", &coresPerProc);
+      }
+      if (siblingsPerProc == 0) {
+         sscanf(line, "siblings : %u", &siblingsPerProc);
+      }
+      free(line);
    }
 
-   return logical == cores;
+   fclose(f);
+
+   *logical = count;
+   *pkgs = siblingsPerProc > 0 ? count / siblingsPerProc : count;
+   *cores = coresPerProc > 0 ? *pkgs * coresPerProc : *pkgs;
+
+   Log(LGPFX" This machine has %u physical CPUS, %u total cores, and %u "
+            "logical CPUs.\n", *pkgs, *cores, *logical);
+
+   return TRUE;
+#else
+   NOT_IMPLEMENTED();
+#endif
 }
-#endif /*ifdef VMX86_SERVER*/
 
 
 /*
@@ -1200,6 +1245,9 @@ Hostinfo_NumCPUs(void)
    static int count = 0;
 
    if (count <= 0) {
+      FILE *f;
+      char *line;
+
 #if defined(VMX86_SERVER)
       if (HostType_OSIsVMK()) {
          VMK_ReturnStatus status = VMKernel_GetNumCPUsUsed(&count);
@@ -1209,19 +1257,10 @@ Hostinfo_NumCPUs(void)
 
             return -1;
          }
-      } else {
-         count = HostinfoReadProc("logical");
 
-         if (count <= 0) {
-            count = 0;
-
-            return -1;
-         }
+         return count;
       }
-#else /* ifdef VMX86_SERVER */
-      FILE *f;
-      char *line;
-
+#endif
       f = Posix_Fopen("/proc/cpuinfo", "r");
       if (f == NULL) {
 	 return -1;
@@ -1239,7 +1278,6 @@ Hostinfo_NumCPUs(void)
       if (count == 0) {
 	 return -1;
       }
-#endif /* ifdef VMX86_SERVER */
    }
 
    return count;
@@ -1772,207 +1810,6 @@ Hostinfo_LogMemUsage(void)
 }
 
 
-/*
- *----------------------------------------------------------------------
- *
- *  Hostinfo_TouchBackDoor --
- *
- *      Access the backdoor. This is used to determine if we are
- *      running in a VM or on a physical host. On a physical host
- *      this should generate a GP which we catch and thereby determine
- *      that we are not in a VM. However some OSes do not handle the
- *      GP correctly and the process continues running returning garbage.
- *      In this case we check the EBX register which should be
- *      BDOOR_MAGIC if the IN was handled in a VM. Based on this we
- *      return either TRUE or FALSE.
- *
- * Results:
- *      TRUE if we succesfully accessed the backdoor, FALSE or segfault
- *      if not.
- *
- * Side effects:
- *	Exception if not in a VM.
- *
- *----------------------------------------------------------------------
- */
-
-Bool
-Hostinfo_TouchBackDoor(void)
-{
-   /*
-    * XXX: This can cause Apple's Crash Reporter to erroneously display
-    * a crash, even though the process has caught the SIGILL and handled
-    * it.
-    */
-
-#if !defined(__APPLE__) && (defined(__i386__) || defined(__x86_64__))
-   uint32 eax;
-   uint32 ebx;
-   uint32 ecx;
-
-   __asm__ __volatile__(
-#   if defined __PIC__ && !vm_x86_64 // %ebx is reserved by the compiler.
-      "xchgl %%ebx, %1" "\n\t"
-      "inl %%dx, %%eax" "\n\t"
-      "xchgl %%ebx, %1"
-      : "=a" (eax),
-        "=&rm" (ebx),
-#   else
-      "inl %%dx, %%eax"
-      : "=a" (eax),
-        "=b" (ebx),
-#   endif
-        "=c" (ecx)
-      :	"0" (BDOOR_MAGIC),
-        "1" (~BDOOR_MAGIC),
-        "2" (BDOOR_CMD_GETVERSION),
-        "d" (BDOOR_PORT)
-   );
-   if (ebx == BDOOR_MAGIC) {
-      return TRUE;
-   }
-#endif
-
-   return FALSE;
-}
-
-
-/*
- *----------------------------------------------------------------------
- *
- *  Hostinfo_TouchVirtualPC --
- *
- *      Access MS Virtual PC's backdoor. This is used to determine if 
- *      we are running in a MS Virtual PC or on a physical host.  Works
- *      the same as Hostinfo_TouchBackDoor, except the entry to MS VPC
- *      is an invalid opcode instead or writing to a port.  Since
- *      MS VPC is 32-bit only, the 64-bit path returns FALSE.
- *      See: http://www.codeproject.com/KB/system/VmDetect.aspx
- *
- * Results:
- *      TRUE if we succesfully accessed MS Virtual PC, FALSE or 
- *      segfault if not.
- *
- * Side effects:
- *      Exception if not in a VM.
- *
- *----------------------------------------------------------------------
- */
-
-Bool
-Hostinfo_TouchVirtualPC(void)
-{
-#if defined vm_x86_64
-   return FALSE;
-#else
-
-   uint32 ebxval;
-
-   __asm__ __volatile__ (
-#  if defined __PIC__        // %ebx is reserved by the compiler.
-     "xchgl %%ebx, %1" "\n\t"
-     ".long 0x0B073F0F" "\n\t"
-     "xchgl %%ebx, %1"
-     : "=&rm" (ebxval)
-     : "a" (1),
-       "0" (0)
-#  else
-     ".long 0x0B073F0F"
-     : "=b" (ebxval)
-     : "a" (1),
-       "b" (0)
-#  endif
-  );
-  return !ebxval; // %%ebx is zero if inside Virtual PC
-#endif
-}
-
-
-/*
- *----------------------------------------------------------------------
- *
- *  Hostinfo_NestingSupported --
- *
- *      Access the backdoor with a nesting control query. This is used
- *      to determine if we are running inside a VM that supports nesting.
- *      This function should only be called after determining that the
- *	backdoor is present with Hostinfo_TouchBackdoor().
- *
- * Results:
- *      TRUE if the outer VM supports nesting.
- *	FALSE otherwise.
- *
- * Side effects:
- *	Exception if not in a VM, so don't do that!
- *
- *----------------------------------------------------------------------
- */
-
-Bool
-Hostinfo_NestingSupported(void)
-{
-#if defined(__i386__) || defined(__x86_64__)
-   uint32 cmd = NESTING_CONTROL_QUERY << 16 | BDOOR_CMD_NESTING_CONTROL;
-   uint32 result;
-
-   __asm__ __volatile__(
-      "inl %%dx, %%eax"
-      : "=a" (result)
-      :	"0"  (BDOOR_MAGIC),
-        "c"  (cmd),
-        "d"  (BDOOR_PORT)
-   );
-
-   if (result >= NESTING_CONTROL_QUERY && result != ~0U) {
-      return TRUE;
-   }
-#endif
-
-   return FALSE;
-}
-
-
-/*
- *----------------------------------------------------------------------
- *
- *  Hostinfo_VCPUInfoBackdoor --
- *
- *      Access the backdoor with an VCPU info query. This is used to
- *      determine whether a VCPU supports a particular feature,
- *      determined by 'bit'.  This function should only be called after
- *      determining that the backdoor is present with
- *      Hostinfo_TouchBackdoor().
- *
- * Results:
- *      TRUE if the outer VM supports the feature.
- *	FALSE otherwise.
- *
- * Side effects:
- *	Exception if not in a VM, so don't do that!
- *
- *----------------------------------------------------------------------
- */
-
-Bool
-Hostinfo_VCPUInfoBackdoor(unsigned bit)
-{
-#if defined(__i386__) || defined(__x86_64__)
-   uint32 result;
-   __asm__ __volatile__(
-      "inl %%dx, %%eax"
-      : "=a" (result)
-      :	"0"  (BDOOR_MAGIC),
-        "c"  (BDOOR_CMD_GET_VCPU_INFO),
-        "d"  (BDOOR_PORT)
-   );
-   /* If reserved bit is 1, this command wasn't implemented. */
-   return (result & (1 << BDOOR_CMD_VCPU_RESERVED)) == 0 &&
-          (result & (1 << bit))                     != 0;
-#endif
-   return FALSE;
-}
-
-
 /*
  *----------------------------------------------------------------------
  *
@@ -3435,3 +3272,39 @@ Hostinfo_GetModulePath(uint32 priv)  // IN:
 
    return path;
 }
+
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Hostinfo_GetLibraryPath --
+ *
+ *      Try and deduce the path to the library where the specified
+ *      address resides. Expected usage is that the caller will pass
+ *      in the address of one of the caller's own functions.
+ *
+ *      Not implemented on MacOS.
+ *
+ * Results:
+ *      The path (which MAY OR MAY NOT BE ABSOLUTE) or NULL on failure.
+ *
+ * Side effects:
+ *      Memory is allocated.
+ *
+ *----------------------------------------------------------------------
+ */
+
+char *
+Hostinfo_GetLibraryPath(void *addr)  // IN
+{
+#ifdef __linux__
+   Dl_info info;
+
+   if (dladdr(addr, &info)) {
+      return Unicode_Alloc(info.dli_fname, STRING_ENCODING_DEFAULT);
+   }
+   return NULL;
+#else
+   return NULL;
+#endif
+}
