@@ -459,9 +459,22 @@ Posix_Access(ConstUnicode pathName,  // IN:
       return -1;
    }
 
+#if defined(VMX86_SERVER)
+   /*
+    * ESX can return EINTR making retries a necessity. This is a bug in
+    * ESX that is being worked around here - POSIX says access cannot return
+    * EINTR.
+    */
+
+   do {
+      ret = access(path, mode);
+   } while ((ret == -1) && (errno == EINTR));
+#else
    ret = access(path, mode);
+#endif
 
    free(path);
+
    return ret;
 }
 
@@ -490,15 +503,14 @@ Posix_EuidAccess(ConstUnicode pathName,  // IN:
 #if defined(GLIBC_VERSION_24)
    char *path;
    int ret;
-   int err;
 
    if (!PosixConvertToCurrent(pathName, &path)) {
       return -1;
    }
+
    ret = euidaccess(path, mode);
-   err = errno;
+
    free(path);
-   errno = err;
    return ret;
 #else
    errno = ENOSYS;
@@ -561,7 +573,7 @@ Posix_Utime(ConstUnicode pathName,        // IN:
  */
 
 void
-Posix_Perror(ConstUnicode str)         // IN:
+Posix_Perror(ConstUnicode str)  // IN:
 {
    char *tmpstr = Unicode_GetAllocBytes(str, STRING_ENCODING_DEFAULT);
 
@@ -1660,8 +1672,10 @@ GetpwInternal(struct passwd *pw)  // IN:
    spw.pw_dir = NULL;
    free(spw.pw_name);
    spw.pw_name = NULL;
+#if !defined __ANDROID__
    free(spw.pw_gecos);
    spw.pw_gecos = NULL;
+#endif
    free(spw.pw_shell);
    spw.pw_shell = NULL;
 #if defined(__FreeBSD__)
@@ -1698,11 +1712,13 @@ GetpwInternal(struct passwd *pw)  // IN:
                                     STRING_ENCODING_DEFAULT)) == NULL) {
       goto exit;
    }
+#if !defined __ANDROID__
    if (pw->pw_gecos &&
        (spw.pw_gecos = Unicode_Alloc(pw->pw_gecos,
                                      STRING_ENCODING_DEFAULT)) == NULL) {
       goto exit;
    }
+#endif
    if (pw->pw_shell &&
        (spw.pw_shell = Unicode_Alloc(pw->pw_shell,
                                      STRING_ENCODING_DEFAULT)) == NULL) {
@@ -1991,9 +2007,11 @@ PasswdCopy(struct passwd *orig, // IN
    if (!CopyFieldIntoBuf(orig->pw_passwd, &new->pw_passwd, &buf, &bufLen)) {
       return NULL;
    }
+#if !defined __ANDROID__
    if (!CopyFieldIntoBuf(orig->pw_gecos, &new->pw_gecos, &buf, &bufLen)) {
       return NULL;
    }
+#endif
    if (!CopyFieldIntoBuf(orig->pw_dir, &new->pw_dir, &buf, &bufLen)) {
       return NULL;
    }
@@ -2390,7 +2408,9 @@ GetpwInternal_r(struct passwd *pw,    // IN:
    int ret;
    char *pwname = NULL;
    char *passwd = NULL;
+#if !defined __ANDROID__
    char *gecos = NULL;
+#endif
    char *dir = NULL;
    char *shell = NULL;
    size_t n;
@@ -2417,11 +2437,13 @@ GetpwInternal_r(struct passwd *pw,    // IN:
                                STRING_ENCODING_DEFAULT)) == NULL) {
       goto exit;
    }
+#if !defined __ANDROID__
    if (pw->pw_gecos &&
        (gecos = Unicode_Alloc(pw->pw_gecos,
                               STRING_ENCODING_DEFAULT)) == NULL) {
       goto exit;
    }
+#endif
    if (pw->pw_dir &&
        (dir = Unicode_Alloc(pw->pw_dir,
                             STRING_ENCODING_DEFAULT)) == NULL) {
@@ -2459,7 +2481,7 @@ GetpwInternal_r(struct passwd *pw,    // IN:
       pw->pw_passwd = memcpy(buf + n, passwd, len);
       n += len;
    }
-
+#if !defined __ANDROID__
    if (gecos) {
       size_t len = strlen(gecos) + 1;
 
@@ -2469,7 +2491,7 @@ GetpwInternal_r(struct passwd *pw,    // IN:
       pw->pw_gecos = memcpy(buf + n, gecos, len);
       n += len;
    }
-
+#endif
    if (dir) {
       size_t len = strlen(dir) + 1;
 
@@ -2495,7 +2517,9 @@ exit:
    free(passwd);
    free(dir);
    free(pwname);
+#if !defined __ANDROID__
    free(gecos);
+#endif
    free(shell);
 
    return ret;
@@ -2884,9 +2908,16 @@ Posix_Setmntent(ConstUnicode pathName,  // IN:
    if (!PosixConvertToCurrent(pathName, &path)) {
       return NULL;
    }
-
+#if defined __ANDROID__
+   /*
+    * Android doesn't support setmntent().
+    */
+   NOT_TESTED();
+   errno = ENOSYS;
+   return NULL;
+#else
    stream = setmntent(path, mode);
-
+#endif
    free(path);
 
    return stream;
@@ -2990,6 +3021,14 @@ Posix_Getmntent_r(FILE *fp,          // IN:
                   char *buf,         // IN:
                   int size)          // IN:
 {
+#if defined __ANDROID__
+   /*
+    * Android doesn't support getmntent_r();
+    * using getmntent() will break thread safety.
+    */
+   errno = ENOSYS;
+   return NULL;
+#else
    int ret;
    char *fsname = NULL;
    char *dir = NULL;
@@ -3088,6 +3127,7 @@ exit:
    }
 
    return m;
+#endif // defined __ANDROID__
 }
 
 
