@@ -175,6 +175,9 @@ Random_Crypto(unsigned int size,  // IN:
  * Side Effects:
  *      None
  *
+ * NOTE:
+ *      Despite the look of the code this RNG is extremely fast.
+ *
  *-----------------------------------------------------------------------------
  */
 
@@ -188,15 +191,15 @@ Random_Crypto(unsigned int size,  // IN:
 #define C 0xDB8B0000
 #define L 16
 
-struct rngstate {
+struct rqContext {
   uint32 x[N];
   int p, q;
 };
 
-void *
+rqContext *
 Random_QuickSeed(uint32 seed)  // IN:
 {
-   struct rngstate *rs;
+   struct rqContext *rs;
 
    const uint32 xx[N] = {
       0x95F24DAB, 0x0B685215, 0xE76CCAE7, 0xAF3EC239, 0x715FAD23,
@@ -206,7 +209,7 @@ Random_QuickSeed(uint32 seed)  // IN:
       0x512C0C03, 0xEA857CCD, 0x4CC1D30F, 0x8891A8A1, 0xA6B7AADB
    };
 
-   rs = (struct rngstate *) malloc(sizeof *rs);
+   rs = (struct rqContext *) malloc(sizeof *rs);
 
    if (rs != NULL) {
       uint32 i;
@@ -219,7 +222,7 @@ Random_QuickSeed(uint32 seed)  // IN:
       rs->q = N - M - 1;
    }
 
-   return (void *) rs;
+   return rs;
 }
 
 
@@ -239,17 +242,18 @@ Random_QuickSeed(uint32 seed)  // IN:
  * Side Effects:
  *      The RNG context is modified for later use by Random_Quick.
  *
+ * NOTE:
+ *      Despite the look of the code this RNG is extremely fast.
+ *
  *-----------------------------------------------------------------------------
  */
 
 uint32
-Random_Quick(void *context)  // IN/OUT:
+Random_Quick(rqContext *rs)  // IN/OUT:
 {
    uint32 y, z;
 
-   struct rngstate *rs = (struct rngstate *) context;
-
-   ASSERT(context);
+   ASSERT(rs);
 
    if (rs->p == N - 1) {
       rs->p = 0;
@@ -284,3 +288,36 @@ Random_Quick(void *context)  // IN/OUT:
    return y;
 }
 
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Random_Simple --
+ *
+ *      Generates the next random number in the pseudo-random sequence
+ *      defined by the multiplicative linear congruential generator
+ *      S' = 33614 * S mod (2^31 - 1).  This is the ACM "minimal standard
+ *      random number generator". Based on method described by D.G. Carta
+ *      in CACM, January 1990.
+ *
+ *      Usage: provide previous random number as the seed for next one.
+ *
+ * Results:
+ *      A random integer number is returned.
+ *
+ * Side Effects:
+ *      None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+int
+Random_Simple(int seed)  // IN:
+{
+   uint64 product    = 33614 * (uint64) seed;
+   uint32 product_lo = (uint32) (product & 0xFFFFFFFF) >> 1;
+   uint32 product_hi = product >> 32;
+   int32  test       = product_lo + product_hi;
+
+   return test > 0 ? test : (test & 0x7FFFFFFF) + 1;
+}
