@@ -36,6 +36,7 @@
 # include <io.h>
 # include <process.h>
 # include "coreDump.h"
+# include "getoptWin32.h"
 #endif
 
 #include <fcntl.h>
@@ -48,6 +49,7 @@
 
 #if !defined(_WIN32) && !defined(N_PLAT_NLM)
 #  include <unistd.h>
+#  include <getopt.h>
 #  include <pwd.h>
 #  include <dlfcn.h>
 #endif
@@ -85,10 +87,6 @@ struct UtilVector {
    int len;
 };
 
-#if defined(_WIN32)
-#include "win32util.h"
-static int UtilTokenHasGroup(HANDLE token, SID *group);
-#endif
 
 #ifdef VM_X86_64
 #   if defined(__GNUC__) && (!defined(USING_AUTOCONF) || defined(HAVE_UNWIND_H))
@@ -252,6 +250,37 @@ Util_Checksumv(void *iov,      // IN
 }
 
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * Util_HashString --
+ *
+ *      Get a hash of the given NUL terminated string using the djb2
+ *      hash algorithm.
+ *
+ * Results:
+ *      The hashed value.
+ *
+ * Side effects:
+ *      None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+uint32
+Util_HashString(const char *str)  // IN:
+{
+   uint32 hash = 5381;
+   int c;
+
+   while ((c = *str++) != 0) {
+      hash = ((hash << 5) + hash) + c;
+   }
+
+   return hash;
+}
+
+
 /*
  *-----------------------------------------------------------------------------
  *
@@ -908,269 +937,141 @@ Util_CompareDotted(const char *s1, const char *s2)
 }
 
 
-#if defined(_WIN32)
 /*
  *-----------------------------------------------------------------------------
  *
- * UtilTokenHasGroup --
+ * Util_GetOpt --
  *
- *    Determine if the specified token has a particular group
+ *      A wrapper around getopt_long that avoids needing separate long and
+ *      short option lists.
+ *
+ *      To use this, the array of option structs must:
+ *      * Store the short option name in the 'val' member.
+ *      * Set the 'name' member to NULL if the option has a short name but no
+ *        long name.
+ *      * For options that have only a long name, 'val' should be set to a
+ *        unique value greater than UCHAR_MAX.
+ *      * Terminate the array with a sentinel value that zero-initializes both
+ *        'name' and 'val'.
  *
  * Results:
- *    1 if yes
- *    0 if no
- *    <0 on error
+ *      See getopt_long.
  *
  * Side effects:
- *    None
+ *      None
  *
  *-----------------------------------------------------------------------------
  */
 
-static int
-UtilTokenHasGroup(HANDLE token,
-		  SID *group)
+int
+Util_GetOpt(int argc,                  // IN
+            char * const *argv,        // IN
+            const struct option *opts, // IN
+            Util_NonOptMode mode)      // IN
 {
-   /*
-    * The code is from
-    * http://support.microsoft.com/support/kb/articles/Q118/6/26.ASP
-    * (HOWTO: Determine Whether a Thread Is Running in User Context of Local
-    * Administrator Account), modified as follows:
-    *
-    * . Removed the exception stuff
-    *
-    * . Fixed the token handle leak
-    *
-    * . Used DuplicateToken() which does just what we need instead of
-    *   ImpersonateSelf()/RevertToSelf() which does more, and which I believe
-    *   will not work if we are already impersonating ourselves
-    *
-    * . Allocated the SD on the stack
-    *
-    * . Got rid of the hardcoded DWORD in the computation of the ACL size
-    *
-    * . Used malloc()/free() instead of Local*()
-    *
-    * . Added comments
-    *
-    *  --hpreg
-    */
+   int ret = -1;
+
+   struct option *longOpts = NULL;
+   char *shortOptString = NULL;
 
    /*
-    * Make up private access rights --hpreg
+    * In the worst case, each character needs "::" to indicate that it takes
+    * an optional argument.
     */
-#define Util_HasAdminPriv_Read  (1 << 0)
-#define Util_HasAdminPriv_Write (1 << 1)
+   const size_t maxCharsPerShortOption = 3;
+   const size_t modePrefixSize = 1;
 
-   int ret;
-   HANDLE iToken;
-   SECURITY_DESCRIPTOR sd;
-   DWORD aclLen;
-   ACL *acl;
-   GENERIC_MAPPING gm;
-   PRIVILEGE_SET ps;
-   DWORD psLen;
-   DWORD granted;
-   BOOL status;
+   size_t n = 0;
+   size_t shortOptStringSize;
 
-   iToken = INVALID_HANDLE_VALUE;
-   acl = NULL;
-
-   /*
-    * Duplicate the token, because AccessCheck() requires an impersonation
-    * token --hpreg
-    */
-
-   if (DuplicateToken(token, SecurityImpersonation, &iToken) == 0) {
-      ret = -3;
-      goto end;
-   }
-
-   /*
-    * Construct a Security Descriptor with a Discretionary Access Control List
-    * that contains an Access Control Entry for the administrator group's
-    * Security IDentifier --hpreg
-    */
-
-   if (InitializeSecurityDescriptor(&sd, SECURITY_DESCRIPTOR_REVISION) == 0) {
-      ret = -5;
-      goto end;
-   }
-
-   /*
-    * This magic formula comes from the documentation for
-    * InitializeAcl() --hpreg
-    */
-
-   aclLen  =    sizeof(ACL)
-             + (  sizeof(ACCESS_ALLOWED_ACE)
-                - sizeof(((ACCESS_ALLOWED_ACE *)0)->SidStart)
-                + GetLengthSid(group));
-   acl = malloc(aclLen);
-   if (acl == NULL) {
-      ret = -6;
-      goto end;
-   }
-
-   if (InitializeAcl(acl, aclLen, ACL_REVISION) == 0) {
-      ret = -7;
-      goto end;
-   }
-
-   if (AddAccessAllowedAce(acl, ACL_REVISION,
-          Util_HasAdminPriv_Read | Util_HasAdminPriv_Write, group) == 0) {
-      ret = -8;
-      goto end;
-   }
-
-   if (SetSecurityDescriptorDacl(&sd, TRUE, acl, FALSE) == 0) {
-      ret = -9;
-      goto end;
-   }
-
-   /*
-    * Set the owner and group of the SD, because AccessCheck() requires
-    * it --hpreg
-    */
-
-   if (SetSecurityDescriptorGroup(&sd, group, FALSE) == 0) {
-      ret = -10;
-      goto end;
-   }
-
-   if (SetSecurityDescriptorOwner(&sd, group, FALSE) == 0) {
-      ret = -11;
-      goto end;
-   }
-
-   /*
-    * Finally, check if the SD grants access to the calling thread --hpreg
-    */
-
-   gm.GenericRead    = Util_HasAdminPriv_Read;
-   gm.GenericWrite   = Util_HasAdminPriv_Write;
-   gm.GenericExecute = 0;
-   gm.GenericAll     = Util_HasAdminPriv_Read | Util_HasAdminPriv_Write;
-
-   psLen = sizeof(ps);
-   if (AccessCheck(&sd, iToken, Util_HasAdminPriv_Read, &gm, &ps, &psLen,
-          &granted, &status) == 0) {
-      ret = -12;
-      goto end;
-   }
-
-   ret = status ? 1 : 0;
-
-end:
-
-   if (iToken != INVALID_HANDLE_VALUE) {
-      if (CloseHandle(iToken) == 0 && ret >= 0) {
-         ret = -14;
+   while (!(opts[n].name == NULL && opts[n].val == 0)) {
+      if (UNLIKELY(n == SIZE_MAX)) {
+         /*
+          * Avoid integer overflow.  If you have this many options, you're
+          * doing something wrong.
+          */
+         ASSERT(FALSE);
+         goto exit;
       }
+      n++;
    }
 
-   free(acl);
-
-   return ret;
-
-#undef Util_HasAdminPriv_Read
-#undef Util_HasAdminPriv_Write
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * Util_TokenHasAdminPriv --
- *
- *    Determine if the specified token has administrator privileges
- *
- * Results:
- *    1 if yes
- *    0 if no
- *    <0 on error
- *
- * Side effects:
- *    None
- *
- *-----------------------------------------------------------------------------
- */
-
-int
-Util_TokenHasAdminPriv(HANDLE token)
-{
-   SID_IDENTIFIER_AUTHORITY sidIdentAuth = SECURITY_NT_AUTHORITY;
-   SID *adminGrp = NULL;
-   int ret;
-
-   /*
-    * Build the Security IDentifier of the administrator group --hpreg
-    */
-
-   if (AllocateAndInitializeSid(&sidIdentAuth, 2, SECURITY_BUILTIN_DOMAIN_RID,
-			        DOMAIN_ALIAS_RID_ADMINS, 0, 0, 0, 0, 0, 0,
-				&adminGrp) == 0) {
-      ret = -4;
-      goto end;
+   if (UNLIKELY(n > SIZE_MAX / sizeof *longOpts - 1)) {
+      /* Avoid integer overflow. */
+      ASSERT(FALSE);
+      goto exit;
+   }
+   longOpts = malloc((n + 1) * sizeof *longOpts);
+   if (longOpts == NULL) {
+      goto exit;
    }
 
-   ret = UtilTokenHasGroup(token, adminGrp);
-
-end:
-   if (adminGrp) {
-      FreeSid(adminGrp);
+   if (UNLIKELY(n > (SIZE_MAX - modePrefixSize - 1 /* NUL */) /
+                maxCharsPerShortOption)) {
+      /* Avoid integer overflow. */
+      ASSERT(FALSE);
+      goto exit;
    }
+   shortOptStringSize = n * maxCharsPerShortOption + modePrefixSize + 1 /* NUL */;
+   shortOptString = malloc(shortOptStringSize);
+   if (shortOptString == NULL) {
+      goto exit;
+   } else {
+      struct option empty = { 0 };
+
+      size_t i;
+      struct option *longOptOut = longOpts;
+      char *shortOptOut = shortOptString;
+
+      switch (mode) {
+         case UTIL_NONOPT_STOP:
+            *shortOptOut++ = '+';
+            break;
+         case UTIL_NONOPT_ALL:
+            *shortOptOut++ = '-';
+            break;
+         default:
+            break;
+      }
+
+      for (i = 0; i < n; i++) {
+         int val = opts[i].val;
+
+         if (opts[i].name != NULL) {
+            *longOptOut++ = opts[i];
+         }
+
+         if (val > 0 && val <= UCHAR_MAX) {
+            int argSpec = opts[i].has_arg;
+
+            *shortOptOut++ = (char) val;
+
+            if (argSpec != no_argument) {
+               *shortOptOut++ = ':';
+
+               if (argSpec == optional_argument) {
+                  *shortOptOut++ = ':';
+               }
+            }
+         }
+      }
+
+      ASSERT(longOptOut - longOpts <= n);
+      *longOptOut = empty;
+
+      ASSERT(shortOptOut - shortOptString < shortOptStringSize);
+      *shortOptOut = '\0';
+   }
+
+   ret = getopt_long(argc, argv, shortOptString, longOpts, NULL);
+
+exit:
+   free(longOpts);
+   free(shortOptString);
    return ret;
 }
 
 
-/*
- *-----------------------------------------------------------------------------
- *
- * Util_TokenHasInteractPriv --
- *
- *    Determine if the specified token is logged in interactively.
- *    -- local logons and Remote Desktops logons
- *
- * Results:
- *    1 if yes
- *    0 if no
- *    <0 on error
- *
- * Side effects:
- *    None
- *
- *-----------------------------------------------------------------------------
- */
-
-int
-Util_TokenHasInteractPriv(HANDLE token)
-{
-   SID_IDENTIFIER_AUTHORITY sidIdentAuth = SECURITY_NT_AUTHORITY;
-   SID *interactiveGrp = NULL;
-   int ret;
-
-   /*
-    * Build the Security IDentifier of the administrator group --hpreg
-    */
-
-   if (AllocateAndInitializeSid(&sidIdentAuth, 1, SECURITY_INTERACTIVE_RID,
-			        0, 0, 0, 0, 0, 0, 0,
-				&interactiveGrp) == 0) {
-      ret = -4;
-      goto end;
-   }
-
-   ret = UtilTokenHasGroup(token, interactiveGrp);
-
-end:
-   if (interactiveGrp) {
-      FreeSid(interactiveGrp);
-   }
-   return ret;
-}
-
-#endif
 
 
 /*
