@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2010-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2010 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -25,12 +25,9 @@
 #include <sys/types.h>
 #include <sys/wait.h>
 
-#include <ctype.h>
 #include <stdio.h>
 #include <stdlib.h>
 
-#include <glib.h>
-
 #include "vmware.h"
 #include "vmstdio.h"
 #include "xdg.h"
@@ -60,10 +57,10 @@ static const char xdgDetectDEExec[] = "vmware-xdg-detect-de";
  *
  * Results:
  *      Returns a pointer to a string specifying the desktop environment on
- *      success or "" on failure.
+ *      success or NULL on failure.
  *
  *      This function only guarantees that the returned string matches the
- *      pattern ^[A-Za-z0-9]*$.
+ *      pattern ^[A-Z]*$.
  *
  * Side effects:
  *      Allocates memory for outbuf on first call for duration of program.
@@ -86,13 +83,13 @@ Xdg_DetectDesktopEnv(void)
          size_t outLen;         // Doesn't include NUL.
          int status;
 
-         if (   StdIO_ReadNextLine(cmdPipe, &outbuf, maxSize, &outLen)
+         if (StdIO_ReadNextLine(cmdPipe, &outbuf, maxSize, &outLen)
              == StdIO_Success) {
-            int i;
+            char *i;
 
-            for (i = 0; i < outLen; i++) {
-               if (!isalnum(outbuf[i])) {
-                  g_debug("%s: received malformed input\n", __func__);
+            for (i = outbuf; i < &outbuf[outLen]; i++) {
+               /* We expect a string in all capitals. */
+               if (*i < 'A' || *i > 'Z') {
                   free(outbuf);
                   outbuf = NULL;
                   break;
@@ -102,16 +99,10 @@ Xdg_DetectDesktopEnv(void)
 
          status = pclose(cmdPipe);
          if (!WIFEXITED(status) || WEXITSTATUS(status) != 0) {
-            g_debug("%s: %s did not exit cleanly (%x/%x)\n", __func__, xdgDetectDEExec,
-                    status, WEXITSTATUS(status));
             free(outbuf);
             outbuf = NULL;
          }
       }
-
-      if (outbuf == NULL) {
-         outbuf = "";
-      }
    }
 
    return outbuf;
