@@ -40,6 +40,7 @@
 #  include <sys/time.h>
 #endif
 
+#include "log.h"
 #if defined(G_PLATFORM_WIN32)
 #  include "coreDump.h"
 #endif
@@ -350,8 +351,8 @@ VMToolsLog(const gchar *domain,
           * message. Use the error handler to do it, and ignore any
           * errors.
           */
-         VMToolsError(domain, level | G_LOG_FLAG_RECURSION, message,
-                      gErrorData, VMToolsError);
+         gErrorData->logfn(domain, level | G_LOG_FLAG_RECURSION, msg,
+                           gErrorData, VMToolsError);
       }
       g_free(msg);
    }
@@ -425,7 +426,12 @@ VMToolsConfigLogDomain(const gchar *domain,
 
       if (configfn == NULL) {
          g_warning("Unknown log handler '%s', using default.", handler);
-         goto exit;
+         if (strcmp(domain, gLogDomain) == 0) {
+            configfn = DEFAULT_HANDLER->configfn;
+            hid = DEFAULT_HANDLER->id;
+         } else {
+            goto exit;
+         }
       }
 
       data = configfn(gLogDomain, domain, handler, cfg);
@@ -637,6 +643,28 @@ VMToolsRestoreLogging(LogHandlerData *oldDefault,
 
 
 #if defined(_WIN32)
+
+
+/**
+ * Checks whether given standard device (standard input, standard output,
+ * or standard error) has been redirected to an on-disk file/pipe.
+ * Win32-only.
+ *
+ * @param[in] nStdHandle          The standard device number.
+ *
+ * @return TRUE if device redirected to a file/pipe.
+ */
+
+static gboolean
+VMToolsIsRedirected(DWORD nStdHandle)
+{
+   HANDLE handle = GetStdHandle(nStdHandle);
+   DWORD type = handle ? GetFileType(handle) : FILE_TYPE_UNKNOWN;
+
+   return type == FILE_TYPE_DISK || type == FILE_TYPE_PIPE;
+}
+
+
 /**
  * Attaches a console to the current process. If the parent process already has
  * a console open, reuse it. Otherwise, create a new console for the current
@@ -655,34 +683,45 @@ gboolean
 VMTools_AttachConsole(void)
 {
    typedef BOOL (WINAPI *AttachConsoleFn)(DWORD);
-   gboolean ret = FALSE;
+   gboolean ret = TRUE;
    AttachConsoleFn _AttachConsole;
+   BOOL reopenStdout, reopenStderr;
 
    if (GetConsoleWindow() != NULL) {
-      return TRUE;
+      goto exit;
+   }
+
+   reopenStdout = !VMToolsIsRedirected(STD_OUTPUT_HANDLE);
+   reopenStderr = !VMToolsIsRedirected(STD_ERROR_HANDLE);
+   if (!reopenStdout && !reopenStderr) {
+      goto exit;
    }
 
    _AttachConsole = (AttachConsoleFn) GetProcAddress(GetModuleHandleW(L"kernel32.dll"),
                                                      "AttachConsole");
    if ((_AttachConsole != NULL && _AttachConsole(ATTACH_PARENT_PROCESS)) ||
        AllocConsole()) {
-      FILE* fptr;
+      FILE *fptr;
 
-      fptr = _wfreopen(L"CONOUT$", L"a", stdout);
-      if (fptr == NULL) {
-         g_warning("_wfreopen failed for stdout/CONOUT$: %d (%s)",
-                   errno, strerror(errno));
-         goto exit;
+      if (reopenStdout) {
+         fptr = _wfreopen(L"CONOUT$", L"a", stdout);
+         if (fptr == NULL) {
+            g_warning("_wfreopen failed for stdout/CONOUT$: %d (%s)",
+                      errno, strerror(errno));
+            ret = FALSE;
+         }
       }
 
-      fptr = _wfreopen(L"CONOUT$", L"a", stderr);
-      if (fptr == NULL) {
-         g_warning("_wfreopen failed for stderr/CONOUT$: %d (%s)",
-                   errno, strerror(errno));
-         goto exit;
+      if (reopenStderr) {
+         fptr = _wfreopen(L"CONOUT$", L"a", stderr);
+         if (fptr == NULL) {
+            g_warning("_wfreopen failed for stderr/CONOUT$: %d (%s)",
+                      errno, strerror(errno));
+            ret = FALSE;
+         } else {
+            setvbuf(fptr, NULL, _IONBF, 0);
+         }
       }
-      setvbuf(fptr, NULL, _IONBF, 0);
-      ret = TRUE;
    }
 
 exit:
@@ -881,6 +920,53 @@ Log(const char *fmt, ...)
 }
 
 
+/**
+ * Logs a message with the given log level.
+ *
+ * Translates lib/log levels into glib levels, and sends the message to the log
+ * implementation.
+ *
+ * @param[in]  level    Log level.
+ * @param[in]  fmt      Log message format.
+ * @param[in]  args     Log message arguments.
+ */
+
+void
+LogV(int level,
+     const char *fmt,
+     va_list args)
+{
+   int glevel;
+
+   switch (level) {
+   case VMW_LOG_PANIC:
+      glevel = G_LOG_LEVEL_ERROR;
+      break;
+
+   case VMW_LOG_ERROR:
+      glevel = G_LOG_LEVEL_CRITICAL;
+      break;
+
+   case VMW_LOG_WARNING:
+      glevel = G_LOG_LEVEL_WARNING;
+      break;
+
+   case VMW_LOG_INFO:
+      glevel = G_LOG_LEVEL_MESSAGE;
+      break;
+
+   case VMW_LOG_VERBOSE:
+      glevel = G_LOG_LEVEL_INFO;
+      break;
+
+   default:
+      glevel = G_LOG_LEVEL_DEBUG;
+   }
+
+   g_logv(gLogDomain, glevel, fmt, args);
+}
+
+
 /**
  * Logs a message using the G_LOG_LEVEL_ERROR level. In the default
  * configuration, this will cause the application to terminate and,
