@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2010-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2010 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -26,8 +26,9 @@
 #include "vmware/tools/guestrpc.h"
 
 typedef struct VMXLoggerData {
-   GlibLogger     handler;
-   RpcChannel    *chan;
+   LogHandlerData    handler;
+   GStaticMutex      lock;
+   RpcChannel       *chan;
 } VMXLoggerData;
 
 
@@ -35,7 +36,7 @@ typedef struct VMXLoggerData {
  *******************************************************************************
  * VMXLoggerLog --                                                        */ /**
  *
- * Logs a message to the VMX using RpcChannel.
+ * Logs a message to the VMX using the backdoor.
  *
  * The logger uses its own RpcChannel, opening and closing the channel for each
  * log message sent. This is not optimal, especially if the application already
@@ -46,39 +47,42 @@ typedef struct VMXLoggerData {
  * @param[in] domain    Unused.
  * @param[in] level     Log level.
  * @param[in] message   Message to log.
- * @param[in] data      VMX logger data.
+ * @param[in] _data     VMX logger data.
+ * @param[in] errfn     Unused.
+ *
+ * @return TRUE if successfully logged the message.
  *
  *******************************************************************************
  */
 
-static void
+static gboolean
 VMXLoggerLog(const gchar *domain,
              GLogLevelFlags level,
              const gchar *message,
-             gpointer data)
+             LogHandlerData *_data,
+             LogErrorFn errfn)
 {
-   VMXLoggerData *logger = data;
+   gboolean ret = FALSE;
+   VMXLoggerData *data = (VMXLoggerData *) _data;
 
-   VMTools_AcquireLogStateLock();
-   /*
-    * To avoid nested logging inside of RpcChannel, we need to disable logging
-    * here. See bug 1069390.
-    */
-
-   VMTools_StopLogging();
-
-   if (RpcChannel_Start(logger->chan)) {
+   g_static_mutex_lock(&data->lock);
+   if (RpcChannel_Start(data->chan)) {
       gchar *msg;
       gint cnt = VMToolsAsprintf(&msg, "log %s", message);
 
-      RpcChannel_Send(logger->chan, msg, cnt, NULL, NULL);
+      /*
+       * XXX: RpcChannel_Send() can log stuff in certain situations, which will
+       * cause this to blow up. Hopefully we won't hit those too often since
+       * we're stopping / starting the channel for each log message.
+       */
+      ret = RpcChannel_Send(data->chan, msg, cnt, NULL, NULL);
 
       g_free(msg);
-      RpcChannel_Stop(logger->chan);
+      RpcChannel_Stop(data->chan);
    }
+   g_static_mutex_unlock(&data->lock);
 
-   VMTools_RestartLogging();
-   VMTools_ReleaseLogStateLock();
+   return ret;
 }
 
 
@@ -88,40 +92,52 @@ VMXLoggerLog(const gchar *domain,
  *
  * Cleans up the internal state of a VMX logger.
  *
- * @param[in] data   VMX logger data.
+ * @param[in] _data     VMX logger data.
  *
  *******************************************************************************
  */
 
 static void
-VMXLoggerDestroy(gpointer data)
+VMXLoggerDestroy(LogHandlerData *_data)
 {
-   VMXLoggerData *logger = data;
-   RpcChannel_Destroy(logger->chan);
-   g_free(logger);
+   VMXLoggerData *data = (VMXLoggerData *) _data;
+   RpcChannel_Destroy(data->chan);
+   g_static_mutex_free(&data->lock);
+   g_free(data);
 }
 
 
 /*
  *******************************************************************************
- * VMToolsCreateVMXLogger --                                              */ /**
+ * VMXLoggerConfig --                                                     */ /**
  *
  * Configures a new VMX logger.
  *
+ * @param[in] defaultDomain   Unused.
+ * @param[in] domain          Unused.
+ * @param[in] name            Unused.
+ * @param[in] cfg             Unused.
+ *
  * @return The VMX logger data.
  *
  *******************************************************************************
  */
 
-GlibLogger *
-VMToolsCreateVMXLogger(void)
+LogHandlerData *
+VMXLoggerConfig(const gchar *defaultDomain,
+                const gchar *domain,
+                const gchar *name,
+                GKeyFile *cfg)
 {
    VMXLoggerData *data = g_new0(VMXLoggerData, 1);
    data->handler.logfn = VMXLoggerLog;
-   data->handler.addsTimestamp = TRUE;
+   data->handler.convertToLocal = FALSE;
+   data->handler.timestamp = FALSE;
    data->handler.shared = TRUE;
+   data->handler.copyfn = NULL;
    data->handler.dtor = VMXLoggerDestroy;
-   data->chan = RpcChannel_New();
+   g_static_mutex_init(&data->lock);
+   data->chan = BackdoorChannel_New();
    return &data->handler;
 }
 
