@@ -280,7 +280,7 @@ static inline int compat_unregister_netdevice_notifier(struct notifier_block *nb
         unregister_netdevice_notifier(_nb);
 #endif
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 24)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 24) || defined(__VMKLNX__)
 
 #   define compat_netif_napi_add(dev, napi, poll, quota) \
       netif_napi_add(dev, napi, poll, quota)
@@ -340,4 +340,10 @@ static inline int compat_unregister_netdevice_notifier(struct notifier_block *nb
 #   define compat_netif_tx_unlock(dev) do {} while (0)
 #endif
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 37)
+#   define COMPAT_VLAN_GROUP_ARRAY_LEN VLAN_N_VID
+#else
+#   define COMPAT_VLAN_GROUP_ARRAY_LEN VLAN_GROUP_ARRAY_LEN
+#endif
+
 #endif /* __COMPAT_NETDEVICE_H__ */
