@@ -88,6 +88,13 @@ typedef enum VMCIIntrType {
 #define VMCI_INTR_DATAGRAM     0
 #define VMCI_INTR_NOTIFICATION 1
 
+
+/*
+ * A single VMCI device has an upper limit of 128MB on the amount of
+ * memory that can be used for queue pairs.
+ */
+#define VMCI_MAX_GUEST_QP_MEMORY (128 * 1024 * 1024)
+
 /*
  * We have a fixed set of resource IDs available in the VMX.
  * This allows us to have a very simple implementation since we statically
@@ -116,7 +123,8 @@ typedef uint32 VMCI_Resource;
  * WS 7.0/7.1 and ESX 4.1
  */
 #define VMCI_HGFS_TRANSPORT       13
-#define VMCI_RESOURCE_MAX         14
+#define VMCI_UNITY_PBRPC_REGISTER 14
+#define VMCI_RESOURCE_MAX         15
 
 /* VMCI Ids. */
 typedef uint32 VMCIId;
@@ -301,12 +309,28 @@ static const VMCIHandle VMCI_INVALID_HANDLE = {VMCI_INVALID_ID,
 /* VMCI reserved events. */
 typedef uint32 VMCI_Event;
 
-#define VMCI_EVENT_CTX_ID_UPDATE  0
-#define VMCI_EVENT_CTX_REMOVED    1
-#define VMCI_EVENT_QP_RESUMED     2
-#define VMCI_EVENT_QP_PEER_ATTACH 3
-#define VMCI_EVENT_QP_PEER_DETACH 4
-#define VMCI_EVENT_MAX            5
+#define VMCI_EVENT_CTX_ID_UPDATE  0  // Only applicable to guest endpoints
+#define VMCI_EVENT_CTX_REMOVED    1  // Applicable to guest and host
+#define VMCI_EVENT_QP_RESUMED     2  // Only applicable to guest endpoints
+#define VMCI_EVENT_QP_PEER_ATTACH 3  // Applicable to guest and host
+#define VMCI_EVENT_QP_PEER_DETACH 4  // Applicable to guest and host
+#define VMCI_EVENT_MEM_ACCESS_ON  5  // Only applicable to VMX endpoints
+#define VMCI_EVENT_MEM_ACCESS_OFF 6  // Only applicable to VMX endpoints
+#define VMCI_EVENT_MAX            7
+
+/*
+ * Of the above events, a few are reserved for use in the VMX, and
+ * other endpoints (guest and host kernel) should not use them. For
+ * the rest of the events, we allow both host and guest endpoints to
+ * subscribe to them, to maintain the same API for host and guest
+ * endpoints.
+ */
+
+#define VMCI_EVENT_VALID_VMX(_event) (_event == VMCI_EVENT_MEM_ACCESS_ON || \
+                                      _event == VMCI_EVENT_MEM_ACCESS_OFF)
+
+#define VMCI_EVENT_VALID(_event) (_event < VMCI_EVENT_MAX && \
+                                  !VMCI_EVENT_VALID_VMX(_event))
 
 /* Reserved guest datagram resource ids. */
 #define VMCI_EVENT_HANDLER 0
@@ -338,42 +362,6 @@ typedef uint32 VMCIPrivilegeFlags;
 #define VMCI_LEAST_PRIVILEGE_FLAGS         VMCI_PRIVILEGE_FLAG_RESTRICTED
 #define VMCI_MAX_PRIVILEGE_FLAGS           VMCI_PRIVILEGE_FLAG_TRUSTED
 
-/* VMCI Discovery Service. */
-
-/* Well-known handle to the discovery service. */
-#define VMCI_DS_RESOURCE_ID 1 /* Reserved resource ID for discovery service. */
-#define VMCI_DS_HANDLE VMCI_MAKE_HANDLE(VMCI_WELL_KNOWN_CONTEXT_ID, \
-					VMCI_DS_RESOURCE_ID)
-#define VMCI_DS_CONTEXT VMCI_MAKE_HANDLE(VMCI_WELL_KNOWN_CONTEXT_ID, \
-					 VMCI_CONTEXT_RESOURCE_ID)
-
-/* Maximum length of a DS message. */
-#define VMCI_DS_MAX_MSG_SIZE        300
-
-/* Command actions. */
-#define VMCI_DS_ACTION_LOOKUP         0
-#define VMCI_DS_ACTION_REGISTER       1
-#define VMCI_DS_ACTION_UNREGISTER     2
-
-/* Defines wire-protocol format for a request send to the DS from a context. */
-typedef struct VMCIDsRequestHeader {
-   int32       action;
-   int32       msgid;
-   VMCIHandle  handle;
-   int32       nameLen;
-   char        name[1];
-} VMCIDsRequestHeader;
-
-
-/* Defines the wire-protocol format for a request send from the DS to a context. */
-typedef struct VMCIDsReplyHeader {
-   int32       msgid;
-   int32       code;
-   VMCIHandle  handle;
-   int32       msgLen;
-   int8        msg[1];
-} VMCIDsReplyHeader;
-
 #define VMCI_PUBLIC_GROUP_NAME "vmci public group"
 /* 0 through VMCI_RESERVED_RESOURCE_ID_MAX are reserved. */
 #define VMCI_RESERVED_RESOURCE_ID_MAX 1023
