@@ -40,16 +40,20 @@
 typedef enum {
    VMCIOBJ_VMX_VM = 10,
    VMCIOBJ_CONTEXT,
-   VMCIOBJ_PROCESS,
-   VMCIOBJ_DATAGRAM_PROCESS,
    VMCIOBJ_SOCKET,
    VMCIOBJ_NOT_SET,
 } VMCIObjType;
 
+/* For storing VMCI structures in file handles. */
+typedef struct VMCIObj {
+   void *ptr;
+   VMCIObjType type;
+} VMCIObj;
+
 /* Guestcalls currently support a maximum of 8 uint64 arguments. */
 #define VMCI_GUESTCALL_MAX_ARGS_SIZE 64
 
-/* 
+/*
  * Structure used for checkpointing the doorbell mappings. It is
  * written to the checkpoint as is, so changing this structure will
  * break checkpoint compatibility.
