@@ -183,15 +183,21 @@ enum IOCTLCmd_VMCI {
 
    /* BEGIN VMCI */
    IOCTLCMD(INIT_CONTEXT),
-   IOCTLCMD(CREATE_PROCESS),
-   IOCTLCMD(CREATE_DATAGRAM_PROCESS),
+
+   /*
+    * The following two were used for process and datagram process creation.
+    * They are not used anymore and reserved for future use.
+    * They will fail if issued.
+    */
+   IOCTLCMD(RESERVED1),
+   IOCTLCMD(RESERVED2),
 
    /*
     * The following two used to be for shared memory.  They are now unused and
     * and are reserved for future use.  They will fail if issued.
     */
-   IOCTLCMD(RESERVED1),
-   IOCTLCMD(RESERVED2),
+   IOCTLCMD(RESERVED3),
+   IOCTLCMD(RESERVED4),
 
    /*
     * The follwoing two were also used to be for shared memory. An old
@@ -302,18 +308,25 @@ enum IOCTLCmd_VMCIMacOS {
  * Windows VMCI ioctl definitions.
  */
 
-/*
- * The first is the device name in user-mode.  The next two are for registering
- * or opening the device in kernel-mode, and are always in UNICODE.
- */
-#define VMCI_DEVICE_NAME         TEXT("\\\\.\\VMCI")
-#define VMCI_DEVICE_NAME_NT      L"\\??\\VMCI"
-#define VMCI_DEVICE_NAME_PATH    L"\\Device\\vmci"
-#define VMCI_DEVICE_LINK_PATH    L"\\DosDevices\\vmci"
+/* PUBLIC: For VMCISockets user-mode clients that use CreateFile(). */
+#define VMCI_INTERFACE_VSOCK_PUBLIC_NAME TEXT("\\\\.\\VMCI")
+
+/* PUBLIC: For VMCISockets user-mode clients that use NtCreateFile(). */
+#define VMCI_INTERFACE_VSOCK_PUBLIC_NAME_NT L"\\??\\VMCI"
+
+/* PUBLIC: For the VMX, which uses CreateFile(). */
+#define VMCI_INTERFACE_VMX_PUBLIC_NAME TEXT("\\\\.\\VMCIDev\\VMX")
+
+/* PRIVATE NAMES */
+#define VMCI_DEVICE_VMCI_LINK_PATH  L"\\DosDevices\\VMCIDev"
+#define VMCI_DEVICE_VSOCK_LINK_PATH L"\\DosDevices\\vmci"
+#define VMCI_DEVICE_HOST_NAME_PATH  L"\\Device\\VMCIHostDev"
+#define VMCI_DEVICE_GUEST_NAME_PATH L"\\Device\\VMCIGuestDev"
+/* PRIVATE NAMES */
 
 /* These values cannot be changed since some of the ioctl values are public. */
-#define FILE_DEVICE_VMCI         0x8103
-#define VMCI_IOCTL_BASE_INDEX    0x801
+#define FILE_DEVICE_VMCI      0x8103
+#define VMCI_IOCTL_BASE_INDEX 0x801
 #define VMCIIOCTL_BUFFERED(name) \
       CTL_CODE(FILE_DEVICE_VMCI, \
 	       VMCI_IOCTL_BASE_INDEX + IOCTLCMD_VMCI_ ## name, \
@@ -335,10 +348,6 @@ enum IOCTLCmd_VMCIWin32 {
 /* BEGIN VMCI */
 #define IOCTL_VMCI_INIT_CONTEXT \
                VMCIIOCTL_BUFFERED(INIT_CONTEXT)
-#define IOCTL_VMCI_CREATE_PROCESS \
-               VMCIIOCTL_BUFFERED(CREATE_PROCESS)
-#define IOCTL_VMCI_CREATE_DATAGRAM_PROCESS \
-               VMCIIOCTL_BUFFERED(CREATE_DATAGRAM_PROCESS)
 #define IOCTL_VMCI_HYPERCALL \
                VMCIIOCTL_BUFFERED(HYPERCALL)
 #define IOCTL_VMCI_CREATE_DATAGRAM_HANDLE  \
@@ -434,8 +443,8 @@ typedef struct VMCIInitBlock {
 typedef struct VMCISharedMemInfo {
    VMCIHandle handle;
    uint32     size;
-   uint32     result;     
-   VA64       va; /* Currently only used in the guest. */ 
+   uint32     result;
+   VA64       va; /* Currently only used in the guest. */
    char       pageFileName[VMCI_PATH_MAX];
 } VMCISharedMemInfo;
 
@@ -449,7 +458,7 @@ typedef struct VMCIQueuePairAllocInfo {
    VA64       producePageFile; /* User VA. */
    VA64       consumePageFile; /* User VA. */
    uint64     producePageFileSize; /* Size of the file name array. */
-   uint64     consumePageFileSize; /* Size of the file name array. */ 
+   uint64     consumePageFileSize; /* Size of the file name array. */
 #else
    PPN *      PPNs;
    uint64     numPPNs;
@@ -520,19 +529,6 @@ typedef struct VMCIDatagramSendRecvInfo {
    int32  result;
 } VMCIDatagramSendRecvInfo;
 
-/* Used to create datagram endpoints in guest or host userlevel. */
-typedef struct VMCIDatagramCreateProcessInfo {
-   VMCIId      resourceID;
-   uint32      flags;
-#ifdef _WIN32
-   int         eventHnd;
-#else
-   int         _unused;
-#endif
-   int         result;     // result of handle create operation
-   VMCIHandle  handle;     // handle if successfull
-} VMCIDatagramCreateProcessInfo;
-
 /* Used to add/remove well-known datagram mappings. */
 typedef struct VMCIDatagramMapInfo {
    VMCIId      wellKnownID;
@@ -561,12 +557,12 @@ typedef struct VMCISetNotifyInfo {
    uint32      _pad;
 } VMCISetNotifyInfo;
 
-#define VMCI_NOTIFY_RESOURCE_QUEUE_PAIR 0 
+#define VMCI_NOTIFY_RESOURCE_QUEUE_PAIR 0
 #define VMCI_NOTIFY_RESOURCE_DOOR_BELL  1
 
-#define VMCI_NOTIFY_RESOURCE_ACTION_NOTIFY  0 
-#define VMCI_NOTIFY_RESOURCE_ACTION_CREATE  1 
-#define VMCI_NOTIFY_RESOURCE_ACTION_DESTROY 2 
+#define VMCI_NOTIFY_RESOURCE_ACTION_NOTIFY  0
+#define VMCI_NOTIFY_RESOURCE_ACTION_CREATE  1
+#define VMCI_NOTIFY_RESOURCE_ACTION_DESTROY 2
 
 /*
  * Used to create and destroy doorbells, and generate a notification
@@ -666,8 +662,6 @@ typedef struct VMCIDeviceGetInfo {
 enum VMCrossTalkSockOpt {
    VMCI_SO_VERSION = 0,
    VMCI_SO_CONTEXT                  = IOCTL_VMCI_INIT_CONTEXT,
-   VMCI_SO_PROCESS                  = IOCTL_VMCI_CREATE_PROCESS,
-   VMCI_SO_DATAGRAM_PROCESS         = IOCTL_VMCI_CREATE_DATAGRAM_PROCESS,
    VMCI_SO_NOTIFY_RESOURCE          = IOCTL_VMCI_NOTIFY_RESOURCE,
    VMCI_SO_NOTIFICATIONS_RECEIVE    = IOCTL_VMCI_NOTIFICATIONS_RECEIVE,
    VMCI_SO_VERSION2                 = IOCTL_VMCI_VERSION2,
@@ -675,13 +669,13 @@ enum VMCrossTalkSockOpt {
    VMCI_SO_QUEUEPAIR_SETPAGEFILE    = IOCTL_VMCI_QUEUEPAIR_SETPAGEFILE,
    VMCI_SO_QUEUEPAIR_DETACH         = IOCTL_VMCI_QUEUEPAIR_DETACH,
    VMCI_SO_DATAGRAM_SEND            = IOCTL_VMCI_DATAGRAM_SEND,
-   VMCI_SO_DATAGRAM_RECEIVE         = IOCTL_VMCI_DATAGRAM_RECEIVE, 
+   VMCI_SO_DATAGRAM_RECEIVE         = IOCTL_VMCI_DATAGRAM_RECEIVE,
    VMCI_SO_DATAGRAM_REQUEST_MAP     = IOCTL_VMCI_DATAGRAM_REQUEST_MAP,
-   VMCI_SO_DATAGRAM_REMOVE_MAP      = IOCTL_VMCI_DATAGRAM_REMOVE_MAP, 
-   VMCI_SO_CTX_ADD_NOTIFICATION     = IOCTL_VMCI_CTX_ADD_NOTIFICATION, 
-   VMCI_SO_CTX_REMOVE_NOTIFICATION  = IOCTL_VMCI_CTX_REMOVE_NOTIFICATION, 
-   VMCI_SO_CTX_GET_CPT_STATE        = IOCTL_VMCI_CTX_GET_CPT_STATE, 
-   VMCI_SO_CTX_SET_CPT_STATE        = IOCTL_VMCI_CTX_SET_CPT_STATE, 
+   VMCI_SO_DATAGRAM_REMOVE_MAP      = IOCTL_VMCI_DATAGRAM_REMOVE_MAP,
+   VMCI_SO_CTX_ADD_NOTIFICATION     = IOCTL_VMCI_CTX_ADD_NOTIFICATION,
+   VMCI_SO_CTX_REMOVE_NOTIFICATION  = IOCTL_VMCI_CTX_REMOVE_NOTIFICATION,
+   VMCI_SO_CTX_GET_CPT_STATE        = IOCTL_VMCI_CTX_GET_CPT_STATE,
+   VMCI_SO_CTX_SET_CPT_STATE        = IOCTL_VMCI_CTX_SET_CPT_STATE,
    VMCI_SO_GET_CONTEXT_ID           = IOCTL_VMCI_GET_CONTEXT_ID,
    VMCI_SO_USERFD,
 };
