@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2006-2012 VMware, Inc. All rights reserved.
+ * Copyright (C) 2006 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -26,6 +26,7 @@
 #define _VMCI_COMMONINT_H_
 
 #define INCLUDE_ALLOW_MODULE
+#define INCLUDE_ALLOW_VMMON
 #define INCLUDE_ALLOW_VMCORE
 #define INCLUDE_ALLOW_VMKERNEL
 #include "includeCheck.h"
@@ -37,6 +38,7 @@
 #include "vmci_handle_array.h"
 #include "vmci_kernel_if.h"
 
+
 /*
  *  The DatagramQueueEntry is a queue header for the in-kernel VMCI
  *  datagram queues. It is allocated in non-paged memory, as the
@@ -53,20 +55,6 @@ typedef struct DatagramQueueEntry {
 } DatagramQueueEntry;
 
 
-/*
- * The VMCIFilterState captures the state of all VMCI filters in one
- * direction. The ranges array contains all filter list in a single
- * memory chunk, and the filter list pointers in the VMCIProtoFilters
- * point into the ranges array.
- */
-
-typedef struct VMCIFilterState {
-   VMCIProtoFilters filters;
-   VMCIIdRange *ranges;
-   size_t rangesSize;
-} VMCIFilterState;
-
-
 struct VMCIContext {
    VMCIListItem       listItem;         /* For global VMCI list. */
    VMCIId             cid;
@@ -78,11 +66,8 @@ struct VMCIContext {
                                          * Version of the code that created
                                          * this context; e.g., VMX.
                                          */
-   VMCILock           lock;             /*
-                                         * Locks datagramQueue, inFilters,
-                                         * doorbellArray, pendingDoorbellArray
-                                         * and notifierArray.
-                                         */
+   VMCILock           lock;             /* Locks callQueue and handleArrays. */
+   VMCIHandleArray    *wellKnownArray;  /* WellKnown mappings owned by context. */
    VMCIHandleArray    *queuePairArray;  /*
                                          * QueuePairs attached to.  The array of
                                          * handles for queue pairs is accessed
@@ -91,7 +76,7 @@ struct VMCIContext {
                                          * is also accessed from the context
                                          * clean up path, which does not
                                          * require a lock.  VMCILock is not
-                                         * used to protect the QP array.
+                                         * used to protect the QP array field.
                                          */
    VMCIHandleArray    *doorbellArray;   /* Doorbells created by context. */
    VMCIHandleArray    *pendingDoorbellArray; /* Doorbells pending for context. */
@@ -101,14 +86,9 @@ struct VMCIContext {
    VMCIHostUser       user;
    Bool               validUser;
 #ifdef VMKERNEL
+   char               domainName[VMCI_DOMAIN_NAME_MAXLEN];
    Bool               isQuiesced;       /* Whether current VM is quiesced */
    VMCIId             migrateCid;       /* The migrate cid if it is migrating */
-   VMCIMutex          guestMemMutex;    /*
-                                         * Coordinates guest memory
-                                         * registration/release during FSR.
-                                         */
-   VMCIGuestMemID     curGuestMemID;    /* ID of current registered guest mem */
-   VMCIFilterState    *inFilters;       /* Ingoing filters for VMCI traffic. */
 #endif
 #ifndef VMX86_SERVER
    Bool               *notify;          /* Notify flag pointer - hosted only. */
@@ -126,7 +106,8 @@ struct VMCIContext {
  *
  *     Utilility function that checks whether two entities are allowed
  *     to interact. If one of them is restricted, the other one must
- *     be trusted.
+ *     be trusted. On ESX, the vmci domain must match for unrestricted
+ *     domains.
  *
  *  Result:
  *     TRUE if the two entities are not allowed to interact. FALSE otherwise.
@@ -139,12 +120,46 @@ struct VMCIContext {
 
 static INLINE Bool
 VMCIDenyInteraction(VMCIPrivilegeFlags partOne,  // IN
-                    VMCIPrivilegeFlags partTwo)  // IN
+                    VMCIPrivilegeFlags partTwo,  // IN
+                    const char *srcDomain,       // IN:  Unused on hosted
+                    const char *dstDomain)       // IN:  Unused on hosted
 {
+#ifndef VMKERNEL
    return (((partOne & VMCI_PRIVILEGE_FLAG_RESTRICTED) &&
             !(partTwo & VMCI_PRIVILEGE_FLAG_TRUSTED)) ||
            ((partTwo & VMCI_PRIVILEGE_FLAG_RESTRICTED) &&
             !(partOne & VMCI_PRIVILEGE_FLAG_TRUSTED)));
+#else
+   /*
+    * If source or destination is trusted (hypervisor), we allow the
+    * communication.
+    */
+   if ((partOne & VMCI_PRIVILEGE_FLAG_TRUSTED) ||
+       (partTwo & VMCI_PRIVILEGE_FLAG_TRUSTED)) {
+      return FALSE;
+   }
+   /*
+    * If source or destination is restricted, we deny the communication.
+    */
+   if ((partOne & VMCI_PRIVILEGE_FLAG_RESTRICTED) ||
+       (partTwo & VMCI_PRIVILEGE_FLAG_RESTRICTED)) {
+      return TRUE;
+   }
+   /*
+    * We are here, means that neither of source or destination are trusted, and
+    * both are unrestricted.
+    */
+   ASSERT(!(partOne & VMCI_PRIVILEGE_FLAG_TRUSTED) &&
+          !(partTwo & VMCI_PRIVILEGE_FLAG_TRUSTED));
+   ASSERT(!(partOne & VMCI_PRIVILEGE_FLAG_RESTRICTED) &&
+          !(partTwo & VMCI_PRIVILEGE_FLAG_RESTRICTED));
+   /*
+    * We now compare the source and destination domain names, and allow
+    * communication iff they match.
+    */
+   return strcmp(srcDomain, dstDomain) ? TRUE : /* Deny. */
+                                         FALSE; /* Allow. */
+#endif
 }
 
 #endif /* _VMCI_COMMONINT_H_ */
