@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2006-2013,2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2006 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -19,7 +19,7 @@
 /*
  * vmciContext.h --
  *
- *    VMCI state to enable sending calls between VMs.
+ *	VMCI state to enable sending calls between VMs.
  */
 
 #ifndef _VMCI_CONTEXT_H_
@@ -35,8 +35,6 @@
 #include "vmci_call_defs.h"
 #include "vmci_handle_array.h"
 #include "vmci_infrastructure.h"
-#include "vmci_kernel_if.h"
-#include "vmciCommonInt.h"
 
 #define MAX_QUEUED_GUESTCALLS_PER_VM  100
 
@@ -48,6 +46,9 @@ int VMCIContext_InitContext(VMCIId cid, VMCIPrivilegeFlags flags,
                             uintptr_t eventHnd, int version,
                             VMCIHostUser *user, VMCIContext **context);
 #ifdef VMKERNEL
+int VMCIContext_SetDomainName(VMCIContext *context, const char *domainName);
+int VMCIContext_GetDomainName(VMCIId contextID, char *domainName,
+                              size_t domainNameBufSize);
 void VMCIContext_SetFSRState(VMCIContext *context,
                              Bool isQuiesced,
                              VMCIId migrateCid,
@@ -56,43 +57,40 @@ void VMCIContext_SetFSRState(VMCIContext *context,
 VMCIContext *VMCIContext_FindAndUpdateSrcFSR(VMCIId migrateCid,
                                              uintptr_t eventHnd,
                                              uintptr_t *srcEventHnd);
-Bool VMCIContext_IsActiveHnd(VMCIContext *context, uintptr_t eventHnd);
-uintptr_t VMCIContext_GetActiveHnd(VMCIContext *context);
-void VMCIContext_SetInactiveHnd(VMCIContext *context, uintptr_t eventHnd);
+Bool VMCIContext_IsActiveHnd(VMCIContext *context,
+                             uintptr_t eventHnd);
+void VMCIContext_SetInactiveHnd(VMCIContext *context,
+                                uintptr_t eventHnd);
 Bool VMCIContext_RemoveHnd(VMCIContext *context,
                            uintptr_t eventHnd,
                            uint32 *numOld,
                            uint32 *numNew);
 void VMCIContext_ClearDatagrams(VMCIContext *context);
 void VMCIContext_SetId(VMCIContext *context, VMCIId cid);
-void VMCIContext_NotifyGuestPaused(VMCIId cid, Bool paused);
-void VMCIContext_NotifyMemoryAccess(VMCIId cid, Bool on);
 #endif
 Bool VMCIContext_SupportsHostQP(VMCIContext *context);
 void VMCIContext_ReleaseContext(VMCIContext *context);
-int VMCIContext_EnqueueDatagram(VMCIId cid, VMCIDatagram *dg, Bool notify);
+int VMCIContext_EnqueueDatagram(VMCIId cid, VMCIDatagram *dg);
 int VMCIContext_DequeueDatagram(VMCIContext *context, size_t *maxSize,
-                                VMCIDatagram **dg);
+				VMCIDatagram **dg);
 int VMCIContext_PendingDatagrams(VMCIId cid, uint32 *pending);
 VMCIContext *VMCIContext_Get(VMCIId cid);
 void VMCIContext_Release(VMCIContext *context);
 Bool VMCIContext_Exists(VMCIId cid);
 
 VMCIId VMCIContext_GetId(VMCIContext *context);
+int VMCIContext_AddGroupEntry(VMCIContext *context,
+                              VMCIHandle entryHandle);
+VMCIHandle VMCIContext_RemoveGroupEntry(VMCIContext *context,
+                                        VMCIHandle entryHandle);
+int VMCIContext_AddWellKnown(VMCIId contextID, VMCIId wellKnownID);
+int VMCIContext_RemoveWellKnown(VMCIId contextID, VMCIId wellKnownID);
 int VMCIContext_AddNotification(VMCIId contextID, VMCIId remoteCID);
 int VMCIContext_RemoveNotification(VMCIId contextID, VMCIId remoteCID);
 int VMCIContext_GetCheckpointState(VMCIId contextID, uint32 cptType,
                                    uint32 *numCIDs, char **cptBufPtr);
 int VMCIContext_SetCheckpointState(VMCIId contextID, uint32 cptType,
                                    uint32 numCIDs, char *cptBuf);
-void VMCIContext_RegisterGuestMem(VMCIContext *context, VMCIGuestMemID gid);
-void VMCIContext_ReleaseGuestMem(VMCIContext *context, VMCIGuestMemID gid,
-                                 Bool powerOff);
-
-int VMCIContext_QueuePairCreate(VMCIContext *context, VMCIHandle handle);
-int VMCIContext_QueuePairDestroy(VMCIContext *context, VMCIHandle handle);
-Bool VMCIContext_QueuePairExists(VMCIContext *context, VMCIHandle handle);
-
 #ifndef VMX86_SERVER
 void VMCIContext_CheckAndSignalNotify(VMCIContext *context);
 #  ifdef __linux__
@@ -116,8 +114,8 @@ void VMCIContext_ReceiveNotificationsRelease(VMCIId contextID,
                                              Bool success);
 #if defined(VMKERNEL)
 void VMCIContext_SignalPendingDoorbells(VMCIId contextID);
-void VMCIContext_SignalPendingDatagrams(VMCIId contextID);
-int VMCIContext_FilterSet(VMCIId cid, VMCIFilterState *filterState);
-int VMCI_Uuid2ContextId(const char *uuidString, VMCIId *contextID);
-#endif // VMKERNEL
+
+int VMCIContextID2HostVmID(VMCIId contextID, void *hostVmID, size_t hostVmIDLen);
+#endif
+
 #endif // _VMCI_CONTEXT_H_
