@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2006-2011,2014 VMware, Inc. All rights reserved.
+ * Copyright (C) 2006 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -58,6 +58,12 @@ typedef struct DatagramEntry {
    VMCIPrivilegeFlags  privFlags;
 } DatagramEntry;
 
+/* Mapping between wellknown resource and context. */
+typedef struct DatagramWKMapping {
+   VMCIHashEntry entry;
+   VMCIId        contextID;
+} DatagramWKMapping;
+
 typedef struct VMCIDelayedDatagramInfo {
    Bool inDGHostQueue;
    DatagramEntry *entry;
@@ -65,6 +71,9 @@ typedef struct VMCIDelayedDatagramInfo {
 } VMCIDelayedDatagramInfo;
 
 
+/* Wellknown mapping hashtable. */
+static VMCIHashTable *wellKnownTable = NULL;
+
 static Atomic_uint32 delayedDGHostQueueSize;
 
 static int VMCIDatagramGetPrivFlagsInt(VMCIId contextID, VMCIHandle handle,
@@ -72,6 +81,9 @@ static int VMCIDatagramGetPrivFlagsInt(VMCIId contextID, VMCIHandle handle,
 static void DatagramFreeCB(void *resource);
 static int DatagramReleaseCB(void *clientData);
 
+static DatagramWKMapping *DatagramGetWellKnownMap(VMCIId wellKnownID);
+static void DatagramReleaseWellKnownMap(DatagramWKMapping *wkMap);
+
 
 /*------------------------------ Helper functions ----------------------------*/
 
@@ -166,7 +178,7 @@ DatagramCreateHnd(VMCIId resourceID,            // IN:
       if ((flags & VMCI_FLAG_ANYCID_DG_HND) != 0) {
          contextID = VMCI_INVALID_ID;
       } else {
-         contextID = vmci_get_context_id();
+         contextID = VMCI_GetContextID();
          if (contextID == VMCI_INVALID_ID) {
             return VMCI_ERROR_NO_RESOURCES;
          }
@@ -242,6 +254,12 @@ DatagramCreateHnd(VMCIId resourceID,            // IN:
 int
 VMCIDatagram_Init(void)
 {
+   /* Create hash table for wellknown mappings. */
+   wellKnownTable = VMCIHashTable_Create(32);
+   if (wellKnownTable == NULL) {
+      return VMCI_ERROR_NO_RESOURCES;
+   }
+
    Atomic_Write(&delayedDGHostQueueSize, 0);
    return VMCI_SUCCESS;
 }
@@ -266,6 +284,10 @@ VMCIDatagram_Init(void)
 void
 VMCIDatagram_Exit(void)
 {
+   if (wellKnownTable != NULL) {
+      VMCIHashTable_Destroy(wellKnownTable);
+      wellKnownTable = NULL;
+   }
 }
 
 
@@ -274,7 +296,7 @@ VMCIDatagram_Exit(void)
 /*
  *------------------------------------------------------------------------------
  *
- * vmci_datagram_create_handle --
+ * VMCIDatagram_CreateHnd --
  *
  *      Creates a host context datagram endpoint and returns a handle to it.
  *
@@ -287,14 +309,14 @@ VMCIDatagram_Exit(void)
  *------------------------------------------------------------------------------
  */
 
-VMCI_EXPORT_SYMBOL(vmci_datagram_create_handle)
+VMCI_EXPORT_SYMBOL(VMCIDatagram_CreateHnd)
 int
-vmci_datagram_create_handle(
-   VMCIId resourceID,         // IN: Optional, generated if VMCI_INVALID_ID
-   uint32 flags,              // IN:
-   VMCIDatagramRecvCB recvCB, // IN:
-   void *clientData,          // IN:
-   VMCIHandle *outHandle)     // OUT: newly created handle
+VMCIDatagram_CreateHnd(VMCIId resourceID,          // IN: Optional, generated
+                                                   //     if VMCI_INVALID_ID
+                       uint32 flags,               // IN:
+                       VMCIDatagramRecvCB recvCB,  // IN:
+                       void *clientData,           // IN:
+                       VMCIHandle *outHandle)      // OUT: newly created handle
 {
    if (outHandle == NULL) {
       return VMCI_ERROR_INVALID_ARGS;
@@ -314,7 +336,7 @@ vmci_datagram_create_handle(
 /*
  *------------------------------------------------------------------------------
  *
- * vmci_datagram_create_handle_priv --
+ * VMCIDatagram_CreateHndPriv --
  *
  *      Creates a host context datagram endpoint and returns a handle to it.
  *
@@ -327,15 +349,15 @@ vmci_datagram_create_handle(
  *------------------------------------------------------------------------------
  */
 
-VMCI_EXPORT_SYMBOL(vmci_datagram_create_handle_priv)
+VMCI_EXPORT_SYMBOL(VMCIDatagram_CreateHndPriv)
 int
-vmci_datagram_create_handle_priv(
-   VMCIId resourceID,            // IN: Optional, generated if VMCI_INVALID_ID
-   uint32 flags,                 // IN:
-   VMCIPrivilegeFlags privFlags, // IN:
-   VMCIDatagramRecvCB recvCB,    // IN:
-   void *clientData,             // IN:
-   VMCIHandle *outHandle)        // OUT: newly created handle
+VMCIDatagram_CreateHndPriv(VMCIId resourceID,           // IN: Optional, generated
+                                                        //     if VMCI_INVALID_ID
+                           uint32 flags,                // IN:
+                           VMCIPrivilegeFlags privFlags,// IN:
+                           VMCIDatagramRecvCB recvCB,   // IN:
+                           void *clientData,            // IN:
+                           VMCIHandle *outHandle)       // OUT: newly created handle
 {
    if (outHandle == NULL) {
       return VMCI_ERROR_INVALID_ARGS;
@@ -359,7 +381,7 @@ vmci_datagram_create_handle_priv(
 /*
  *------------------------------------------------------------------------------
  *
- * vmci_datagram_destroy_handle --
+ * VMCIDatagram_DestroyHnd --
  *
  *      Destroys a handle.
  *
@@ -372,9 +394,9 @@ vmci_datagram_create_handle_priv(
  *------------------------------------------------------------------------------
  */
 
-VMCI_EXPORT_SYMBOL(vmci_datagram_destroy_handle)
+VMCI_EXPORT_SYMBOL(VMCIDatagram_DestroyHnd)
 int
-vmci_datagram_destroy_handle(VMCIHandle handle) // IN
+VMCIDatagram_DestroyHnd(VMCIHandle handle)       // IN
 {
    DatagramEntry *entry;
    VMCIResource *resource = VMCIResource_Get(handle,
@@ -394,6 +416,10 @@ vmci_datagram_destroy_handle(VMCIHandle handle) // IN
     */
    VMCI_WaitOnEvent(&entry->destroyEvent, DatagramReleaseCB, entry);
 
+   if ((entry->flags & VMCI_FLAG_WELLKNOWN_DG_HND) != 0) {
+      VMCIDatagramRemoveWellKnownMap(handle.resource, VMCI_HOST_CONTEXT_ID);
+   }
+
    /*
     * We know that we are now the only reference to the above entry so
      * can safely free it.
@@ -444,7 +470,7 @@ VMCIDatagramGetPrivFlagsInt(VMCIId contextID,              // IN
    } else if (contextID == VMCI_HYPERVISOR_CONTEXT_ID) {
       *privFlags = VMCI_MAX_PRIVILEGE_FLAGS;
    } else {
-      *privFlags = vmci_context_get_priv_flags(contextID);
+      *privFlags = VMCIContext_GetPrivFlags(contextID);
    }
 
    return VMCI_SUCCESS;
@@ -544,7 +570,10 @@ VMCIDatagramDispatchAsHost(VMCIId contextID,  // IN:
 {
    int retval;
    size_t dgSize;
+   VMCIId dstContext;
    VMCIPrivilegeFlags srcPrivFlags;
+   char srcDomain[VMCI_DOMAIN_NAME_MAXLEN]; /* Not used on hosted. */
+   char dstDomain[VMCI_DOMAIN_NAME_MAXLEN]; /* Not used on hosted. */
 
    ASSERT(dg);
    ASSERT(VMCI_HostPersonalityActive());
@@ -553,7 +582,6 @@ VMCIDatagramDispatchAsHost(VMCIId contextID,  // IN:
 
    if (contextID == VMCI_HOST_CONTEXT_ID &&
        dg->dst.context == VMCI_HYPERVISOR_CONTEXT_ID) {
-      VMCI_DEBUG_LOG(4, (LGPFX"Host cannot talk to hypervisor\n"));
       return VMCI_ERROR_DST_UNREACHABLE;
    }
 
@@ -569,10 +597,45 @@ VMCIDatagramDispatchAsHost(VMCIId contextID,  // IN:
     * Check that source handle matches sending context.
     */
    if (dg->src.context != contextID) {
-      VMCI_DEBUG_LOG(4, (LGPFX"Sender context (ID=0x%x) is not owner of src "
-                         "datagram entry (handle=0x%x:0x%x).\n",
-                         contextID, dg->src.context, dg->src.resource));
-      return VMCI_ERROR_NO_ACCESS;
+      if (dg->src.context == VMCI_WELL_KNOWN_CONTEXT_ID) {
+         /* Determine mapping. */
+         DatagramWKMapping *wkMap = DatagramGetWellKnownMap(dg->src.resource);
+         if (wkMap == NULL) {
+            VMCI_DEBUG_LOG(4, (LGPFX"Sending from invalid well-known resource "
+                               "(handle=0x%x:0x%x).\n",
+                               dg->src.context, dg->src.resource));
+            return VMCI_ERROR_INVALID_RESOURCE;
+         }
+         if (wkMap->contextID != contextID) {
+            VMCI_DEBUG_LOG(4, (LGPFX"Sender context (ID=0x%x) is not owner of "
+                               "well-known src datagram entry "
+                               "(handle=0x%x:0x%x).\n",
+                               contextID, dg->src.context, dg->src.resource));
+            DatagramReleaseWellKnownMap(wkMap);
+            return VMCI_ERROR_NO_ACCESS;
+         }
+         DatagramReleaseWellKnownMap(wkMap);
+      } else {
+         VMCI_DEBUG_LOG(4, (LGPFX"Sender context (ID=0x%x) is not owner of src "
+                            "datagram entry (handle=0x%x:0x%x).\n",
+                            contextID, dg->src.context, dg->src.resource));
+         return VMCI_ERROR_NO_ACCESS;
+      }
+   }
+
+   if (dg->dst.context == VMCI_WELL_KNOWN_CONTEXT_ID) {
+      /* Determine mapping. */
+      DatagramWKMapping *wkMap = DatagramGetWellKnownMap(dg->dst.resource);
+      if (wkMap == NULL) {
+         VMCI_DEBUG_LOG(4, (LGPFX"Sending to invalid wellknown destination "
+                            "(handle=0x%x:0x%x).\n",
+                            dg->dst.context, dg->dst.resource));
+         return VMCI_ERROR_DST_UNREACHABLE;
+      }
+      dstContext = wkMap->contextID;
+      DatagramReleaseWellKnownMap(wkMap);
+   } else {
+      dstContext = dg->dst.context;
    }
 
    /*
@@ -586,8 +649,26 @@ VMCIDatagramDispatchAsHost(VMCIId contextID,  // IN:
       return retval;
    }
 
+#ifdef VMKERNEL
+   /*
+    * In the vmkernel, all communicating contexts except the
+    * hypervisor context must belong to the same domain. If the
+    * hypervisor is the source, the domain doesn't matter.
+    */
+
+   if (contextID != VMCI_HYPERVISOR_CONTEXT_ID) {
+      retval = VMCIContext_GetDomainName(contextID, srcDomain,
+                                         sizeof srcDomain);
+      if (retval < VMCI_SUCCESS) {
+         VMCI_WARNING((LGPFX"Failed to get domain name for context (ID=0x%x).\n",
+                       contextID));
+         return retval;
+      }
+   }
+#endif
+
    /* Determine if we should route to host or guest destination. */
-   if (dg->dst.context == VMCI_HOST_CONTEXT_ID) {
+   if (dstContext == VMCI_HOST_CONTEXT_ID) {
       /* Route to host datagram entry. */
       DatagramEntry *dstEntry;
       VMCIResource *resource;
@@ -605,7 +686,18 @@ VMCIDatagramDispatchAsHost(VMCIId contextID,  // IN:
          return VMCI_ERROR_INVALID_RESOURCE;
       }
       dstEntry = RESOURCE_CONTAINER(resource, DatagramEntry, resource);
-      if (VMCIDenyInteraction(srcPrivFlags, dstEntry->privFlags)) {
+#ifdef VMKERNEL
+      retval = VMCIContext_GetDomainName(VMCI_HOST_CONTEXT_ID, dstDomain,
+                                         sizeof dstDomain);
+      if (retval < VMCI_SUCCESS) {
+         VMCI_WARNING((LGPFX"Failed to get domain name for context (ID=0x%x).\n",
+                       VMCI_HOST_CONTEXT_ID));
+         VMCIResource_Release(resource);
+         return retval;
+      }
+#endif
+      if (VMCIDenyInteraction(srcPrivFlags, dstEntry->privFlags, srcDomain,
+                              dstDomain)) {
          VMCIResource_Release(resource);
          return VMCI_ERROR_NO_ACCESS;
       }
@@ -622,7 +714,7 @@ VMCIDatagramDispatchAsHost(VMCIId contextID,  // IN:
            VMCI_CanScheduleDelayedWork())) {
          VMCIDelayedDatagramInfo *dgInfo;
 
-         if (Atomic_ReadInc32(&delayedDGHostQueueSize) ==
+         if (Atomic_FetchAndAdd(&delayedDGHostQueueSize, 1) ==
              VMCI_MAX_DELAYED_DG_HOST_QUEUE_SIZE) {
             Atomic_Dec(&delayedDGHostQueueSize);
             VMCIResource_Release(resource);
@@ -659,43 +751,33 @@ VMCIDatagramDispatchAsHost(VMCIId contextID,  // IN:
          }
       }
    } else {
-      /*
-       * Route to destination VM context.
-       */
-
+      /* Route to destination VM context. */
       VMCIDatagram *newDG;
 
-      if (contextID != dg->dst.context) {
-         if (VMCIDenyInteraction(srcPrivFlags,
-                              vmci_context_get_priv_flags(dg->dst.context))) {
-            VMCI_DEBUG_LOG(4, (LGPFX"Interaction denied (%X/%X - %X/%X)\n",
-                               contextID, srcPrivFlags,
-                               dg->dst.context,
-                               vmci_context_get_priv_flags(dg->dst.context)));
-            return VMCI_ERROR_NO_ACCESS;
-         } else if (VMCI_CONTEXT_IS_VM(contextID)) {
-            /*
-             * If the sending context is a VM, it cannot reach another VM.
-             */
-
-            VMCI_DEBUG_LOG(4, (LGPFX"Datagram communication between VMs not "
-                               "supported (src=0x%x, dst=0x%x).\n",
-                               contextID, dg->dst.context));
-            return VMCI_ERROR_DST_UNREACHABLE;
-         }
+#ifdef VMKERNEL
+      retval = VMCIContext_GetDomainName(dstContext, dstDomain,
+                                         sizeof dstDomain);
+      if (retval < VMCI_SUCCESS) {
+         VMCI_DEBUG_LOG(4, (LGPFX"Failed to get domain name for context "
+                            "(ID=0x%x).\n", dstContext));
+         return retval;
+      }
+#endif
+      if (contextID != dstContext &&
+         VMCIDenyInteraction(srcPrivFlags, VMCIContext_GetPrivFlags(dstContext),
+                             srcDomain, dstDomain)) {
+         return VMCI_ERROR_NO_ACCESS;
       }
 
       /* We make a copy to enqueue. */
       newDG = VMCI_AllocKernelMem(dgSize, VMCI_MEMORY_NORMAL);
       if (newDG == NULL) {
-         VMCI_DEBUG_LOG(4, (LGPFX"No memory for datagram\n"));
          return VMCI_ERROR_NO_MEM;
       }
       memcpy(newDG, dg, dgSize);
-      retval = VMCIContext_EnqueueDatagram(dg->dst.context, newDG, TRUE);
+      retval = VMCIContext_EnqueueDatagram(dstContext, newDG);
       if (retval < VMCI_SUCCESS) {
          VMCI_FreeKernelMem(newDG, dgSize);
-         VMCI_DEBUG_LOG(4, (LGPFX"Enqueue failed\n"));
          return retval;
       }
    }
@@ -742,6 +824,8 @@ VMCIDatagramDispatchAsGuest(VMCIDatagram *dg)
    int retval;
    VMCIResource *resource;
 
+   ASSERT(VMCI_GuestPersonalityActive());
+
    resource = VMCIResource_Get(dg->src, VMCI_RESOURCE_TYPE_DATAGRAM);
    if (NULL == resource) {
       return VMCI_ERROR_NO_HANDLE;
@@ -791,9 +875,6 @@ VMCIDatagram_Dispatch(VMCIId contextID,
 
    retval = VMCI_Route(&dg->src, &dg->dst, fromGuest, &route);
    if (retval < VMCI_SUCCESS) {
-      VMCI_DEBUG_LOG(4, (LGPFX"Failed to route datagram (src=0x%x, dst=0x%x, "
-                         "err=%d)\n.", dg->src.context, dg->dst.context,
-                         retval));
       return retval;
    }
 
@@ -843,12 +924,7 @@ VMCIDatagram_InvokeGuestHandler(VMCIDatagram *dg) // IN
    DatagramEntry *dstEntry;
 
    ASSERT(dg);
-
-   if (dg->payloadSize > VMCI_MAX_DG_PAYLOAD_SIZE) {
-      VMCI_DEBUG_LOG(4, (LGPFX"Payload (size=%"FMT64"u bytes) too large to "
-                         "deliver.\n", dg->payloadSize));
-      return VMCI_ERROR_PAYLOAD_TOO_LARGE;
-   }
+   ASSERT(VMCI_GuestPersonalityActive());
 
    resource = VMCIResource_Get(dg->dst, VMCI_RESOURCE_TYPE_DATAGRAM);
    if (NULL == resource) {
@@ -897,7 +973,7 @@ exit:
 /*
  *------------------------------------------------------------------------------
  *
- * vmci_datagram_send --
+ * VMCIDatagram_Send --
  *
  *      Sends the payload to the destination datagram handle.
  *
@@ -910,9 +986,9 @@ exit:
  *------------------------------------------------------------------------------
  */
 
-VMCI_EXPORT_SYMBOL(vmci_datagram_send)
+VMCI_EXPORT_SYMBOL(VMCIDatagram_Send)
 int
-vmci_datagram_send(VMCIDatagram *msg) // IN
+VMCIDatagram_Send(VMCIDatagram *msg) // IN
 {
    if (msg == NULL) {
       return VMCI_ERROR_INVALID_ARGS;
@@ -922,6 +998,158 @@ vmci_datagram_send(VMCIDatagram *msg) // IN
 }
 
 
+/*
+ *------------------------------------------------------------------------------
+ *
+ * DatagramGetWellKnownMap --
+ *
+ *      Gets a mapping between handle and wellknown resource.
+ *
+ * Results:
+ *      DatagramWKMapping * if found, NULL if not.
+ *
+ * Side effects:
+ *      None.
+ *
+ *------------------------------------------------------------------------------
+ */
+
+static DatagramWKMapping *
+DatagramGetWellKnownMap(VMCIId wellKnownID)  // IN:
+{
+   VMCIHashEntry *entry;
+   DatagramWKMapping *wkMap = NULL;
+   VMCIHandle wkHandle = VMCI_MAKE_HANDLE(VMCI_WELL_KNOWN_CONTEXT_ID,
+                                          wellKnownID);
+   entry = VMCIHashTable_GetEntry(wellKnownTable, wkHandle);
+   if (entry != NULL) {
+      wkMap = RESOURCE_CONTAINER(entry, DatagramWKMapping, entry);
+   }
+   return wkMap;
+}
+
+
+/*
+ *------------------------------------------------------------------------------
+ *
+ * DatagramReleaseWellKnownMap --
+ *
+ *      Releases a wellknown mapping.
+ *
+ * Results:
+ *      None.
+ *
+ * Side effects:
+ *      None.
+ *
+ *------------------------------------------------------------------------------
+ */
+
+static void
+DatagramReleaseWellKnownMap(DatagramWKMapping *wkMap)  // IN:
+{
+   if (VMCIHashTable_ReleaseEntry(wellKnownTable, &wkMap->entry) ==
+       VMCI_SUCCESS_ENTRY_DEAD) {
+      VMCI_FreeKernelMem(wkMap, sizeof *wkMap);
+   }
+}
+
+
+/*
+ *------------------------------------------------------------------------------
+ *
+ * VMCIDatagramRequestWellKnownMap --
+ *
+ *      Creates a mapping between handle and wellknown resource. If resource
+ *      is already used we fail the request.
+ *
+ * Results:
+ *      VMCI_SUCCESS if created, negative errno value otherwise.
+ *
+ * Side effects:
+ *      None.
+ *
+ *------------------------------------------------------------------------------
+ */
+
+int
+VMCIDatagramRequestWellKnownMap(VMCIId wellKnownID,           // IN:
+                                VMCIId contextID,             // IN:
+                                VMCIPrivilegeFlags privFlags) // IN:
+{
+   int result;
+   DatagramWKMapping *wkMap;
+   VMCIHandle wkHandle = VMCI_MAKE_HANDLE(VMCI_WELL_KNOWN_CONTEXT_ID,
+                                          wellKnownID);
+
+   if (privFlags & VMCI_PRIVILEGE_FLAG_RESTRICTED ||
+       !VMCIWellKnownID_AllowMap(wellKnownID, privFlags)) {
+      return VMCI_ERROR_NO_ACCESS;
+   }
+
+   wkMap = VMCI_AllocKernelMem(sizeof *wkMap, VMCI_MEMORY_NONPAGED);
+   if (wkMap == NULL) {
+      return VMCI_ERROR_NO_MEM;
+   }
+
+   VMCIHashTable_InitEntry(&wkMap->entry, wkHandle);
+   wkMap->contextID = contextID;
+
+   /* Fails if wkHandle (wellKnownID) already exists. */
+   result = VMCIHashTable_AddEntry(wellKnownTable, &wkMap->entry);
+   if (result != VMCI_SUCCESS) {
+      VMCI_FreeKernelMem(wkMap, sizeof *wkMap);
+      return result;
+   }
+   result = VMCIContext_AddWellKnown(contextID, wellKnownID);
+   if (UNLIKELY(result < VMCI_SUCCESS)) {
+      VMCIHashTable_RemoveEntry(wellKnownTable, &wkMap->entry);
+      VMCI_FreeKernelMem(wkMap, sizeof *wkMap);
+   }
+   return result;
+}
+
+
+/*
+ *------------------------------------------------------------------------------
+ *
+ * VMCIDatagramRemoveWellKnownMap --
+ *
+ *      Removes a mapping between handle and wellknown resource. Checks if
+ *      mapping belongs to calling context.
+ *
+ * Results:
+ *      VMCI_SUCCESS if removed, negative errno value otherwise.
+ *
+ * Side effects:
+ *      None.
+ *
+ *------------------------------------------------------------------------------
+ */
+
+int
+VMCIDatagramRemoveWellKnownMap(VMCIId wellKnownID,  // IN:
+                               VMCIId contextID)    // IN:
+{
+   int result = VMCI_ERROR_NO_ACCESS;
+   DatagramWKMapping *wkMap = DatagramGetWellKnownMap(wellKnownID);
+   if (wkMap == NULL) {
+      VMCI_DEBUG_LOG(4, (LGPFX"Failed to remove well-known mapping between "
+                         "resource (ID=0x%x) and context (ID=0x%x).\n",
+                         wellKnownID, contextID));
+      return VMCI_ERROR_NOT_FOUND;
+   }
+
+   if (contextID == wkMap->contextID) {
+      VMCIHashTable_RemoveEntry(wellKnownTable, &wkMap->entry);
+      VMCIContext_RemoveWellKnown(contextID, wellKnownID);
+      result = VMCI_SUCCESS;
+   }
+   DatagramReleaseWellKnownMap(wkMap);
+   return result;
+}
+
+
 /*
  *-----------------------------------------------------------------------------
  *
