@@ -41,7 +41,20 @@
 
 #define LGPFX "VMCIDoorbell: "
 
-#if !defined(__APPLE__)
+#if !defined(SOLARIS) && !defined(__APPLE__)
+
+#if defined(VMKERNEL)
+   /* VMK doesn't need BH locks, so use lower ranks. */
+#  define VMCIDoorbellInitLock(_l, _n) \
+   VMCI_InitLock(_l, _n, VMCI_LOCK_RANK_HIGHER)
+#  define VMCIDoorbellGrabLock(_l, _f)       VMCI_GrabLock(_l, _f)
+#  define VMCIDoorbellReleaseLock(_l, _f)    VMCI_ReleaseLock(_l, _f)
+#else // VMKERNEL
+#  define VMCIDoorbellInitLock(_l, _n) \
+   VMCI_InitLock(_l, _n, VMCI_LOCK_RANK_MIDDLE_BH)
+#  define VMCIDoorbellGrabLock(_l, _f)       VMCI_GrabLock_BH(_l, _f)
+#  define VMCIDoorbellReleaseLock(_l, _f)    VMCI_ReleaseLock_BH(_l, _f)
+#endif // VMKERNEL
 
 #define VMCI_DOORBELL_INDEX_TABLE_SIZE 64
 #define VMCI_DOORBELL_HASH(_idx) \
@@ -63,7 +76,6 @@ typedef struct VMCIDoorbellEntry {
    VMCICallback        notifyCB;
    void                *clientData;
    VMCIEvent           destroyEvent;
-   Atomic_uint32       active;       // Only used by guest personality
 } VMCIDoorbellEntry;
 
 typedef struct VMCIDoorbellIndexTable {
@@ -115,7 +127,7 @@ static void VMCIDoorbellDelayedDispatchCB(void *data);
  *    General init code.
  *
  * Result:
- *    VMCI_SUCCESS on success, lock allocation error otherwise.
+ *    None.
  *
  * Side effects:
  *    None.
@@ -123,7 +135,7 @@ static void VMCIDoorbellDelayedDispatchCB(void *data);
  *------------------------------------------------------------------------------
  */
 
-int
+void
 VMCIDoorbell_Init(void)
 {
    uint32 bucket;
@@ -132,8 +144,7 @@ VMCIDoorbell_Init(void)
       VMCIList_Init(&vmciDoorbellIT.entries[bucket]);
    }
 
-   return VMCI_InitLock(&vmciDoorbellIT.lock, "VMCIDoorbellIndexTableLock",
-                        VMCI_LOCK_RANK_DOORBELL);
+   VMCIDoorbellInitLock(&vmciDoorbellIT.lock, "VMCIDoorbellIndexTableLock");
 }
 
 
@@ -227,9 +238,7 @@ VMCIDoorbellReleaseCB(void *clientData) // IN: doorbell entry
  *    handle. Hypervisor endpoints are not yet supported.
  *
  * Result:
- *    VMCI_SUCCESS on success,
- *    VMCI_ERROR_NOT_FOUND if handle isn't found,
- *    VMCI_ERROR_INVALID_ARGS if handle is invalid.
+ *    VMCI_SUCCESS on success, VMCI_ERROR_INVALID_ARGS if handle is invalid.
  *
  * Side effects:
  *    None.
@@ -251,7 +260,7 @@ VMCIDoorbellGetPrivFlags(VMCIHandle handle,             // IN
 
       resource = VMCIResource_Get(handle, VMCI_RESOURCE_TYPE_DOORBELL);
       if (resource == NULL) {
-         return VMCI_ERROR_NOT_FOUND;
+         return VMCI_ERROR_INVALID_ARGS;
       }
       entry = RESOURCE_CONTAINER(resource, VMCIDoorbellEntry, resource);
       *privFlags = entry->privFlags;
@@ -260,7 +269,7 @@ VMCIDoorbellGetPrivFlags(VMCIHandle handle,             // IN
        /* Hypervisor endpoints for notifications are not supported (yet). */
       return VMCI_ERROR_INVALID_ARGS;
    } else {
-      *privFlags = vmci_context_get_priv_flags(handle.context);
+      *privFlags = VMCIContext_GetPrivFlags(handle.context);
    }
 
    return VMCI_SUCCESS;
@@ -336,7 +345,7 @@ VMCIDoorbellIndexTableAdd(VMCIDoorbellEntry *entry) // IN/OUT
 
    VMCIResource_Hold(&entry->resource);
 
-   VMCI_GrabLock_BH(&vmciDoorbellIT.lock, &flags);
+   VMCIDoorbellGrabLock(&vmciDoorbellIT.lock, &flags);
 
    /*
     * Below we try to allocate an index in the notification bitmap with "not
@@ -378,7 +387,7 @@ VMCIDoorbellIndexTableAdd(VMCIDoorbellEntry *entry) // IN/OUT
    bucket = VMCI_DOORBELL_HASH(entry->idx);
    VMCIList_Insert(&entry->idxListItem, &vmciDoorbellIT.entries[bucket]);
 
-   VMCI_ReleaseLock_BH(&vmciDoorbellIT.lock, flags);
+   VMCIDoorbellReleaseLock(&vmciDoorbellIT.lock, flags);
 }
 
 
@@ -402,14 +411,16 @@ VMCIDoorbellIndexTableAdd(VMCIDoorbellEntry *entry) // IN/OUT
 static void
 VMCIDoorbellIndexTableRemove(VMCIDoorbellEntry *entry) // IN/OUT
 {
+   uint32 bucket;
    VMCILockFlags flags;
 
    ASSERT(entry);
    ASSERT(VMCI_GuestPersonalityActive());
 
-   VMCI_GrabLock_BH(&vmciDoorbellIT.lock, &flags);
+   VMCIDoorbellGrabLock(&vmciDoorbellIT.lock, &flags);
 
-   VMCIList_Remove(&entry->idxListItem);
+   bucket = VMCI_DOORBELL_HASH(entry->idx);
+   VMCIList_Remove(&entry->idxListItem, &vmciDoorbellIT.entries[bucket]);
 
    notifyIdxCount--;
    if (entry->idx == maxNotifyIdx - 1) {
@@ -427,7 +438,7 @@ VMCIDoorbellIndexTableRemove(VMCIDoorbellEntry *entry) // IN/OUT
    }
    lastNotifyIdxReleased = entry->idx;
 
-   VMCI_ReleaseLock_BH(&vmciDoorbellIT.lock, flags);
+   VMCIDoorbellReleaseLock(&vmciDoorbellIT.lock, flags);
 
    VMCIResource_Release(&entry->resource);
 }
@@ -534,7 +545,7 @@ VMCIDoorbellUnlink(VMCIHandle handle, // IN
 /*
  *------------------------------------------------------------------------------
  *
- * vmci_doorbell_create --
+ * VMCIDoorbell_Create --
  *
  *    Creates a doorbell with the given callback. If the handle is
  *    VMCI_INVALID_HANDLE, a free handle will be assigned, if
@@ -554,13 +565,13 @@ VMCIDoorbellUnlink(VMCIHandle handle, // IN
  *------------------------------------------------------------------------------
  */
 
-VMCI_EXPORT_SYMBOL(vmci_doorbell_create)
+VMCI_EXPORT_SYMBOL(VMCIDoorbell_Create)
 int
-vmci_doorbell_create(VMCIHandle *handle,            // IN/OUT
-                     uint32 flags,                  // IN
-                     VMCIPrivilegeFlags privFlags,  // IN
-                     VMCICallback notifyCB,         // IN
-                     void *clientData)              // IN
+VMCIDoorbell_Create(VMCIHandle *handle,            // IN/OUT
+                    uint32 flags,                  // IN
+                    VMCIPrivilegeFlags privFlags,  // IN
+                    VMCICallback notifyCB,         // IN
+                    void *clientData)              // IN
 {
    VMCIDoorbellEntry *entry;
    VMCIHandle newHandle;
@@ -583,7 +594,7 @@ vmci_doorbell_create(VMCIHandle *handle,            // IN/OUT
    }
 
    if (VMCI_HANDLE_INVALID(*handle)) {
-      VMCIId contextID = vmci_get_context_id();
+      VMCIId contextID = VMCI_GetContextID();
       VMCIId resourceID = VMCIResource_GetID(contextID);
       if (resourceID == VMCI_INVALID_ID) {
          result = VMCI_ERROR_NO_HANDLE;
@@ -605,8 +616,7 @@ vmci_doorbell_create(VMCIHandle *handle,            // IN/OUT
       if (VMCI_HOST_CONTEXT_ID == handle->context) {
          validContext = TRUE;
       }
-      if (VMCI_GuestPersonalityActive() &&
-          vmci_get_context_id() == handle->context) {
+      if (VMCI_GuestPersonalityActive() && VMCI_GetContextID() == handle->context) {
          validContext = TRUE;
       }
 
@@ -627,7 +637,6 @@ vmci_doorbell_create(VMCIHandle *handle,            // IN/OUT
    entry->runDelayed = (flags & VMCI_FLAG_DELAYED_CB) ? TRUE : FALSE;
    entry->notifyCB = notifyCB;
    entry->clientData = clientData;
-   Atomic_Write(&entry->active, 0);
    VMCI_CreateEvent(&entry->destroyEvent);
 
    result = VMCIResource_Add(&entry->resource, VMCI_RESOURCE_TYPE_DOORBELL,
@@ -635,19 +644,15 @@ vmci_doorbell_create(VMCIHandle *handle,            // IN/OUT
    if (result != VMCI_SUCCESS) {
       VMCI_WARNING((LGPFX"Failed to add new resource (handle=0x%x:0x%x).\n",
                     newHandle.context, newHandle.resource));
-      if (result == VMCI_ERROR_DUPLICATE_ENTRY) {
-         result = VMCI_ERROR_ALREADY_EXISTS;
-      }
       goto destroy;
    }
 
    if (VMCI_GuestPersonalityActive()) {
-      VMCIDoorbellIndexTableAdd(entry);
       result = VMCIDoorbellLink(newHandle, entry->isDoorbell, entry->idx);
       if (VMCI_SUCCESS != result) {
          goto destroyResource;
-     }
-      Atomic_Write(&entry->active, 1);
+      }
+      VMCIDoorbellIndexTableAdd(entry);
    }
 
    if (VMCI_HANDLE_INVALID(*handle)) {
@@ -657,7 +662,6 @@ vmci_doorbell_create(VMCIHandle *handle,            // IN/OUT
    return result;
 
 destroyResource:
-   VMCIDoorbellIndexTableRemove(entry);
    VMCIResource_Remove(newHandle, VMCI_RESOURCE_TYPE_DOORBELL);
 destroy:
    VMCI_DestroyEvent(&entry->destroyEvent);
@@ -670,10 +674,10 @@ freeMem:
 /*
  *------------------------------------------------------------------------------
  *
- * vmci_doorbell_destroy --
+ * VMCIDoorbell_Destroy --
  *
  *    Destroys a doorbell previously created with
- *    vmci_doorbell_create. This operation may block waiting for a
+ *    VMCIDoorbell_Create. This operation may block waiting for a
  *    callback to finish.
  *
  * Results:
@@ -685,9 +689,9 @@ freeMem:
  *------------------------------------------------------------------------------
  */
 
-VMCI_EXPORT_SYMBOL(vmci_doorbell_destroy)
+VMCI_EXPORT_SYMBOL(VMCIDoorbell_Destroy)
 int
-vmci_doorbell_destroy(VMCIHandle handle)  // IN
+VMCIDoorbell_Destroy(VMCIHandle handle)  // IN
 {
    VMCIDoorbellEntry *entry;
    VMCIResource *resource;
@@ -798,7 +802,7 @@ VMCIDoorbellNotifyAsGuest(VMCIHandle handle,            // IN
 /*
  *------------------------------------------------------------------------------
  *
- * vmci_doorbell_notify --
+ * VMCIDoorbell_Notify --
  *
  *    Generates a notification on the doorbell identified by the
  *    handle. For host side generation of notifications, the caller
@@ -813,10 +817,10 @@ VMCIDoorbellNotifyAsGuest(VMCIHandle handle,            // IN
  *------------------------------------------------------------------------------
  */
 
-VMCI_EXPORT_SYMBOL(vmci_doorbell_notify)
+VMCI_EXPORT_SYMBOL(VMCIDoorbell_Notify)
 int
-vmci_doorbell_notify(VMCIHandle dst,               // IN
-                     VMCIPrivilegeFlags privFlags) // IN
+VMCIDoorbell_Notify(VMCIHandle dst,               // IN
+                    VMCIPrivilegeFlags privFlags) // IN
 {
    int retval;
    VMCIRoute route;
@@ -900,20 +904,13 @@ VMCIDoorbellHostContextNotify(VMCIId srcCID,     // IN
 
    ASSERT(VMCI_HostPersonalityActive());
 
-   if (VMCI_HANDLE_INVALID(handle)) {
+   resource = VMCIResource_Get(handle, VMCI_RESOURCE_TYPE_DOORBELL);
+   if (resource == NULL) {
       VMCI_DEBUG_LOG(4,
                      (LGPFX"Notifying an invalid doorbell (handle=0x%x:0x%x).\n",
                       handle.context, handle.resource));
       return VMCI_ERROR_INVALID_ARGS;
    }
-
-   resource = VMCIResource_Get(handle, VMCI_RESOURCE_TYPE_DOORBELL);
-   if (resource == NULL) {
-      VMCI_DEBUG_LOG(4,
-                     (LGPFX"Notifying an unknown doorbell (handle=0x%x:0x%x).\n",
-                      handle.context, handle.resource));
-      return VMCI_ERROR_NOT_FOUND;
-   }
    entry = RESOURCE_CONTAINER(resource, VMCIDoorbellEntry, resource);
 
    if (entry->runDelayed) {
@@ -973,28 +970,28 @@ VMCIDoorbell_Hibernate(Bool enterHibernate)
       return;
    }
 
-   VMCI_GrabLock_BH(&vmciDoorbellIT.lock, &flags);
+   VMCIDoorbellGrabLock(&vmciDoorbellIT.lock, &flags);
 
    for (bucket = 0; bucket < ARRAYSIZE(vmciDoorbellIT.entries); bucket++) {
       VMCIList_Scan(iter, &vmciDoorbellIT.entries[bucket]) {
          int result;
-         VMCIHandle h;
          VMCIDoorbellEntry *cur;
 
          cur = VMCIList_Entry(iter, VMCIDoorbellEntry, idxListItem);
-         h = VMCIResource_Handle(&cur->resource);
-         result = VMCIDoorbellLink(h, cur->isDoorbell, cur->idx);
+         result = VMCIDoorbellLink(cur->resource.handle, cur->isDoorbell,
+                                   cur->idx);
          if (result != VMCI_SUCCESS && result != VMCI_ERROR_DUPLICATE_ENTRY) {
             VMCI_WARNING((LGPFX"Failed to reregister doorbell "
                           "(handle=0x%x:0x%x) of resource %s to index "
-                          "(error=%d).\n",
-                          h.context, h.resource,
+                          "(error: %d).\n",
+                          cur->resource.handle.context,
+                          cur->resource.handle.resource,
                           cur->isDoorbell ? "doorbell" : "queue pair", result));
          }
       }
    }
 
-   VMCI_ReleaseLock_BH(&vmciDoorbellIT.lock, flags);
+   VMCIDoorbellReleaseLock(&vmciDoorbellIT.lock, flags);
 }
 
 
@@ -1019,8 +1016,8 @@ void
 VMCIDoorbell_Sync(void)
 {
    VMCILockFlags flags;
-   VMCI_GrabLock_BH(&vmciDoorbellIT.lock, &flags);
-   VMCI_ReleaseLock_BH(&vmciDoorbellIT.lock, flags);
+   VMCIDoorbellGrabLock(&vmciDoorbellIT.lock, &flags);
+   VMCIDoorbellReleaseLock(&vmciDoorbellIT.lock, flags);
    VMCIResource_Sync();
 }
 
@@ -1096,7 +1093,7 @@ VMCIDoorbellFireEntries(uint32 notifyIdx) // IN
 
    ASSERT(VMCI_GuestPersonalityActive());
 
-   VMCI_GrabLock_BH(&vmciDoorbellIT.lock, &flags);
+   VMCIDoorbellGrabLock(&vmciDoorbellIT.lock, &flags);
 
    VMCIList_Scan(iter, &vmciDoorbellIT.entries[bucket]) {
       VMCIDoorbellEntry *cur =
@@ -1104,7 +1101,7 @@ VMCIDoorbellFireEntries(uint32 notifyIdx) // IN
 
       ASSERT(cur);
 
-      if (cur->idx == notifyIdx && Atomic_Read(&cur->active) == 1) {
+      if (cur->idx == notifyIdx) {
          ASSERT(cur->notifyCB);
          if (cur->runDelayed) {
             int err;
@@ -1122,7 +1119,7 @@ VMCIDoorbellFireEntries(uint32 notifyIdx) // IN
    }
 
 out:
-   VMCI_ReleaseLock_BH(&vmciDoorbellIT.lock, flags);
+   VMCIDoorbellReleaseLock(&vmciDoorbellIT.lock, flags);
 }
 
 
@@ -1160,7 +1157,7 @@ VMCI_ScanNotificationBitmap(uint8 *bitmap)
 }
 
 
-#else // __APPLE__
+#else // SOLARIS) || __APPLE__
 
 /*
  *-----------------------------------------------------------------------------
@@ -1226,10 +1223,9 @@ VMCIDoorbellGetPrivFlags(VMCIHandle handle,             // IN
 }
 
 
-int
+void
 VMCIDoorbell_Init(void)
 {
-   return VMCI_SUCCESS;
 }
 
 
@@ -1238,4 +1234,4 @@ VMCIDoorbell_Exit(void)
 {
 }
 
-#endif // __APPLE__
+#endif // SOLARIS) || __APPLE__
