@@ -80,7 +80,7 @@ VMCI_HostInit(void)
                                     VMCI_DEFAULT_PROC_PRIVILEGE_FLAGS,
                                     -1, VMCI_VERSION, NULL, &hostContext);
    if (result < VMCI_SUCCESS) {
-      VMCI_WARNING((LGPFX"Failed to initialize VMCIContext (result=%d).\n",
+      VMCI_WARNING((LGPFX"Failed to initialize VMCIContext (result=%d)",
                     result));
       goto errorExit;
    }
@@ -90,7 +90,7 @@ VMCI_HostInit(void)
       goto hostContextExit;
    }
 
-   VMCI_DEBUG_LOG(0, (LGPFX"host components initialized.\n"));
+   VMCI_LOG((LGPFX"host components initialized."));
    return VMCI_SUCCESS;
 
 hostContextExit:
@@ -124,12 +124,11 @@ VMCI_HostCleanup(void)
 }
 
 
-#if defined(__APPLE__) || defined(VMKERNEL)
-/* Windows has its own implementation of this, and Linux doesn't need one. */
+#if !defined(_WIN32)
 /*
  *----------------------------------------------------------------------
  *
- * vmci_device_get --
+ * VMCI_DeviceGet --
  *
  *      Verifies that a valid VMCI device is present, and indicates
  *      the callers intention to use the device until it calls
@@ -144,17 +143,10 @@ VMCI_HostCleanup(void)
  *----------------------------------------------------------------------
  */
 
-VMCI_EXPORT_SYMBOL(vmci_device_get)
+VMCI_EXPORT_SYMBOL(VMCI_DeviceGet)
 Bool
-vmci_device_get(uint32 *apiVersion,                      // IN/OUT
-                VMCI_DeviceShutdownFn *deviceShutdownCB, // UNUSED
-                void *userData,                          // UNUSED
-                void **deviceRegistration)               // OUT
+VMCI_DeviceGet(uint32 *apiVersion)
 {
-   if (NULL != deviceRegistration) {
-      *deviceRegistration = NULL;
-   }
-
    if (*apiVersion > VMCI_KERNEL_API_VERSION) {
       *apiVersion = VMCI_KERNEL_API_VERSION;
       return FALSE;
@@ -171,7 +163,7 @@ vmci_device_get(uint32 *apiVersion,                      // IN/OUT
 /*
  *----------------------------------------------------------------------
  *
- * vmci_device_release --
+ * VMCI_DeviceRelease --
  *
  *      Indicates that the caller is done using the VMCI device.
  *
@@ -184,12 +176,12 @@ vmci_device_get(uint32 *apiVersion,                      // IN/OUT
  *----------------------------------------------------------------------
  */
 
-VMCI_EXPORT_SYMBOL(vmci_device_release)
+VMCI_EXPORT_SYMBOL(VMCI_DeviceRelease)
 void
-vmci_device_release(void *deviceRegistration) // UNUSED
+VMCI_DeviceRelease(void)
 {
 }
-#endif // __APPLE__ || VMKERNEL
+#endif // !_WIN32
 
 
 /*
@@ -253,12 +245,9 @@ VMCIUtil_Init(void)
     * We subscribe to the VMCI_EVENT_CTX_ID_UPDATE here so we can update the
     * internal context id when needed.
     */
-   if (vmci_event_subscribe(VMCI_EVENT_CTX_ID_UPDATE,
-#if !defined(linux) || defined(VMKERNEL)
-                            VMCI_FLAG_EVENT_NONE,
-#endif // !linux || VMKERNEL
-                            VMCIUtilCidUpdate, NULL,
-                            &ctxUpdateSubID) < VMCI_SUCCESS) {
+   if (VMCIEvent_Subscribe(VMCI_EVENT_CTX_ID_UPDATE, VMCI_FLAG_EVENT_NONE,
+                           VMCIUtilCidUpdate, NULL,
+                           &ctxUpdateSubID) < VMCI_SUCCESS) {
       VMCI_WARNING((LGPFX"Failed to subscribe to event (type=%d).\n",
                     VMCI_EVENT_CTX_ID_UPDATE));
    }
@@ -284,7 +273,7 @@ VMCIUtil_Init(void)
 void
 VMCIUtil_Exit(void)
 {
-   if (vmci_event_unsubscribe(ctxUpdateSubID) < VMCI_SUCCESS) {
+   if (VMCIEvent_Unsubscribe(ctxUpdateSubID) < VMCI_SUCCESS) {
       VMCI_WARNING((LGPFX"Failed to unsubscribe to event (type=%d) with "
                     "subscriber (ID=0x%x).\n", VMCI_EVENT_CTX_ID_UPDATE,
                     ctxUpdateSubID));
@@ -369,12 +358,7 @@ VMCI_CheckHostCapabilities(void)
    result &= VMCIDatagram_CheckHostCapabilities();
    result &= VMCIUtilCheckHostCapabilities();
 
-   if (!result) {
-      /* If it failed, then make sure this goes to the system event log. */
-      VMCI_WARNING((LGPFX"Host capability checked failed.\n"));
-   } else {
-      VMCI_DEBUG_LOG(0, (LGPFX"Host capability check passed.\n"));
-   }
+   VMCI_LOG((LGPFX"Host capability check: %s\n", result ? "PASSED" : "FAILED"));
 
    return result;
 }
@@ -526,7 +510,7 @@ VMCI_ReadDatagramsFromPort(VMCIIoHandle ioHandle,  // IN
 /*
  *----------------------------------------------------------------------------
  *
- * vmci_get_context_id --
+ * VMCI_GetContextID --
  *
  *    Returns the current context ID.  Note that since this is accessed only
  *    from code running in the host, this always returns the host context ID.
@@ -540,9 +524,9 @@ VMCI_ReadDatagramsFromPort(VMCIIoHandle ioHandle,  // IN
  *----------------------------------------------------------------------------
  */
 
-VMCI_EXPORT_SYMBOL(vmci_get_context_id)
+VMCI_EXPORT_SYMBOL(VMCI_GetContextID)
 VMCIId
-vmci_get_context_id(void)
+VMCI_GetContextID(void)
 {
    if (VMCI_GuestPersonalityActive()) {
       if (Atomic_Read(&vmContextID) == VMCI_INVALID_ID) {
@@ -566,7 +550,7 @@ vmci_get_context_id(void)
 /*
  *----------------------------------------------------------------------
  *
- * vmci_version --
+ * VMCI_Version --
  *
  *     Returns the version of the VMCI driver.
  *
@@ -579,9 +563,9 @@ vmci_get_context_id(void)
  *----------------------------------------------------------------------
  */
 
-VMCI_EXPORT_SYMBOL(vmci_version)
+VMCI_EXPORT_SYMBOL(VMCI_Version)
 uint32
-vmci_version(void)
+VMCI_Version()
 {
    return VMCI_VERSION;
 }
@@ -611,46 +595,31 @@ VMCI_SharedInit(void)
 
    result = VMCIResource_Init();
    if (result < VMCI_SUCCESS) {
-      VMCI_WARNING((LGPFX"Failed to initialize VMCIResource (result=%d).\n",
+      VMCI_WARNING((LGPFX"Failed to initialize VMCIResource (result=%d)",
                     result));
       goto errorExit;
    }
 
    result = VMCIContext_Init();
    if (result < VMCI_SUCCESS) {
-      VMCI_WARNING((LGPFX"Failed to initialize VMCIContext (result=%d).\n",
+      VMCI_WARNING((LGPFX"Failed to initialize VMCIContext (result=%d)",
                     result));
       goto resourceExit;
    }
 
    result = VMCIDatagram_Init();
    if (result < VMCI_SUCCESS) {
-      VMCI_WARNING((LGPFX"Failed to initialize VMCIDatagram (result=%d).\n",
+      VMCI_WARNING((LGPFX"Failed to initialize VMCIDatagram (result=%d)",
                     result));
       goto contextExit;
    }
 
-   result = VMCIEvent_Init();
-   if (result < VMCI_SUCCESS) {
-      VMCI_WARNING((LGPFX"Failed to initialize VMCIEvent (result=%d).\n",
-                    result));
-      goto datagramExit;
-   }
+   VMCIEvent_Init();
+   VMCIDoorbell_Init();
 
-   result = VMCIDoorbell_Init();
-   if (result < VMCI_SUCCESS) {
-      VMCI_WARNING((LGPFX"Failed to initialize VMCIDoorbell (result=%d).\n",
-                    result));
-      goto eventExit;
-   }
-
-   VMCI_DEBUG_LOG(0, (LGPFX"shared components initialized.\n"));
+   VMCI_LOG((LGPFX"shared components initialized."));
    return VMCI_SUCCESS;
 
-eventExit:
-   VMCIEvent_Exit();
-datagramExit:
-   VMCIDatagram_Exit();
 contextExit:
    VMCIContext_Exit();
 resourceExit:
