@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2007-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2007 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -61,6 +61,21 @@ static int VMCIEventRegisterSubscription(VMCISubscription *sub,
                                          void *callbackData);
 static VMCISubscription *VMCIEventUnregisterSubscription(VMCIId subID);
 
+
+#if defined(VMKERNEL)
+   /* VMK doesn't need BH locks, so use lower ranks. */
+#  define VMCIEventInitLock(_lock, _name) \
+   VMCI_InitLock(_lock, _name, VMCI_LOCK_RANK_HIGHER)
+#  define VMCIEventGrabLock(_lock, _flags)    VMCI_GrabLock(_lock, _flags)
+#  define VMCIEventReleaseLock(_lock, _flags) VMCI_ReleaseLock(_lock, _flags)
+#else // VMKERNEL
+#  define VMCIEventInitLock(_lock, _name) \
+   VMCI_InitLock(_lock, _name, VMCI_LOCK_RANK_HIGHER_BH)
+#  define VMCIEventGrabLock(_lock, _flags)    VMCI_GrabLock_BH(_lock, _flags)
+#  define VMCIEventReleaseLock(_lock, _flags) VMCI_ReleaseLock_BH(_lock, _flags)
+#endif // VMKERNEL
+
+
 static VMCIList subscriberArray[VMCI_EVENT_MAX];
 static VMCILock subscriberLock;
 
@@ -69,10 +84,6 @@ typedef struct VMCIDelayedEventInfo {
    uint8 eventPayload[sizeof(VMCIEventData_Max)];
 } VMCIDelayedEventInfo;
 
-typedef struct VMCIEventRef {
-   VMCISubscription *sub;
-   VMCIListItem   listItem;
-} VMCIEventRef;
 
 /*
  *----------------------------------------------------------------------
@@ -82,7 +93,7 @@ typedef struct VMCIEventRef {
  *      General init code.
  *
  * Results:
- *      VMCI_SUCCESS on success, appropriate error code otherwise.
+ *      None.
  *
  * Side effects:
  *      None.
@@ -90,7 +101,7 @@ typedef struct VMCIEventRef {
  *----------------------------------------------------------------------
  */
 
-int
+void
 VMCIEvent_Init(void)
 {
    int i;
@@ -98,9 +109,7 @@ VMCIEvent_Init(void)
    for (i = 0; i < VMCI_EVENT_MAX; i++) {
       VMCIList_Init(&subscriberArray[i]);
    }
-
-   return VMCI_InitLock(&subscriberLock, "VMCIEventSubscriberLock",
-                        VMCI_LOCK_RANK_EVENT);
+   VMCIEventInitLock(&subscriberLock, "VMCIEventSubscriberLock");
 }
 
 
@@ -171,8 +180,8 @@ void
 VMCIEvent_Sync(void)
 {
    VMCILockFlags lockFlags;
-   VMCI_GrabLock_BH(&subscriberLock, &lockFlags);
-   VMCI_ReleaseLock_BH(&subscriberLock, lockFlags);
+   VMCIEventGrabLock(&subscriberLock, &lockFlags);
+   VMCIEventReleaseLock(&subscriberLock, lockFlags);
 }
 
 
@@ -281,9 +290,9 @@ EventReleaseCB(void *clientData) // IN
 
    ASSERT(sub);
 
-   VMCI_GrabLock_BH(&subscriberLock, &flags);
+   VMCIEventGrabLock(&subscriberLock, &flags);
    VMCIEventRelease(sub);
-   VMCI_ReleaseLock_BH(&subscriberLock, flags);
+   VMCIEventReleaseLock(&subscriberLock, flags);
 
    return 0;
 }
@@ -359,9 +368,9 @@ VMCIEventDelayedDispatchCB(void *data) // IN
 
    sub->callback(sub->id, ed, sub->callbackData);
 
-   VMCI_GrabLock_BH(&subscriberLock, &flags);
+   VMCIEventGrabLock(&subscriberLock, &flags);
    VMCIEventRelease(sub);
-   VMCI_ReleaseLock_BH(&subscriberLock, flags);
+   VMCIEventReleaseLock(&subscriberLock, flags);
 
    VMCI_FreeKernelMem(eventInfo, sizeof *eventInfo);
 }
@@ -390,13 +399,11 @@ VMCIEventDeliver(VMCIEventMsg *eventMsg)  // IN
    VMCIListItem *iter;
    VMCILockFlags flags;
 
-   VMCIList noDelayList;
-   VMCIList_Init(&noDelayList);
-
    ASSERT(eventMsg);
 
-   VMCI_GrabLock_BH(&subscriberLock, &flags);
+   VMCIEventGrabLock(&subscriberLock, &flags);
    VMCIList_Scan(iter, &subscriberArray[eventMsg->eventData.event]) {
+      VMCI_EventData *ed;
       VMCISubscription *cur = VMCIList_Entry(iter, VMCISubscription,
                                              subscriberListItem);
       ASSERT(cur && cur->event == eventMsg->eventData.event);
@@ -425,40 +432,6 @@ VMCIEventDeliver(VMCIEventMsg *eventMsg)  // IN
          }
 
       } else {
-         VMCIEventRef *eventRef;
-
-         /*
-          * To avoid possible lock rank voilation when holding
-          * subscriberLock, we construct a local list of
-          * subscribers and release subscriberLock before
-          * invokes the callbacks. This is similar to delayed
-          * callbacks, but callbacks is invoked right away here.
-          */
-         if ((eventRef = VMCI_AllocKernelMem(sizeof *eventRef,
-                                             (VMCI_MEMORY_ATOMIC |
-                                              VMCI_MEMORY_NONPAGED))) == NULL) {
-            err = VMCI_ERROR_NO_MEM;
-            goto out;
-         }
-
-         VMCIEventGet(cur);
-         eventRef->sub = cur;
-         VMCIList_InitEntry(&eventRef->listItem);
-         VMCIList_Insert(&eventRef->listItem, &noDelayList);
-      }
-   }
-
-out:
-   VMCI_ReleaseLock_BH(&subscriberLock, flags);
-
-   if (!VMCIList_Empty(&noDelayList)) {
-      VMCI_EventData *ed;
-      VMCIListItem *iter2;
-
-      VMCIList_ScanSafe(iter, iter2, &noDelayList) {
-         VMCIEventRef *eventRef = VMCIList_Entry(iter, VMCIEventRef,
-                                                 listItem);
-         VMCISubscription *cur = eventRef->sub;
          uint8 eventPayload[sizeof(VMCIEventData_Max)];
 
          /* We set event data before each callback to ensure isolation. */
@@ -467,14 +440,12 @@ out:
                 (size_t)eventMsg->hdr.payloadSize);
          ed = (VMCI_EventData *)eventPayload;
          cur->callback(cur->id, ed, cur->callbackData);
-
-         VMCI_GrabLock_BH(&subscriberLock, &flags);
-         VMCIEventRelease(cur);
-         VMCI_ReleaseLock_BH(&subscriberLock, flags);
-         VMCI_FreeKernelMem(eventRef, sizeof *eventRef);
       }
    }
 
+out:
+   VMCIEventReleaseLock(&subscriberLock, flags);
+
    return err;
 }
 
@@ -563,27 +534,9 @@ VMCIEventRegisterSubscription(VMCISubscription *sub,   // IN
       /*
        * In the vmkernel we defer delivery of events to a helper world.  This
        * makes the event delivery more consistent across hosts and guests with
-       * regard to which locks are held.  Memory access and guest paused events
-       * are an exception to this, since clients need to know immediately that
-       * the device memory is disabled (if we delay such events, then clients
-       * may be notified too late).
+       * regard to which locks are held.
        */
-      if (VMCI_EVENT_MEM_ACCESS_ON == event ||
-          VMCI_EVENT_MEM_ACCESS_OFF == event ||
-          VMCI_EVENT_GUEST_PAUSED == event ||
-          VMCI_EVENT_GUEST_UNPAUSED == event) {
-         /*
-          * Client must expect to get such events synchronously, and should
-          * perform its locking accordingly.  If it can't handle this, then
-          * fail.
-          */
-         if (flags & VMCI_FLAG_EVENT_DELAYED_CB) {
-            return VMCI_ERROR_INVALID_ARGS;
-         }
-         sub->runDelayed = FALSE;
-      } else {
-         sub->runDelayed = TRUE;
-      }
+      sub->runDelayed = TRUE;
    } else if (!VMCI_CanScheduleDelayedWork()) {
       /*
        * If the platform doesn't support delayed work callbacks then don't
@@ -607,7 +560,7 @@ VMCIEventRegisterSubscription(VMCISubscription *sub,   // IN
    sub->callbackData = callbackData;
    VMCIList_InitEntry(&sub->subscriberListItem);
 
-   VMCI_GrabLock_BH(&subscriberLock, &lockFlags);
+   VMCIEventGrabLock(&subscriberLock, &lockFlags);
 
    /* Check if creation of a new event is allowed. */
    if (!VMCI_CanCreate()) {
@@ -645,7 +598,7 @@ VMCIEventRegisterSubscription(VMCISubscription *sub,   // IN
    }
 
 exit:
-   VMCI_ReleaseLock_BH(&subscriberLock, lockFlags);
+   VMCIEventReleaseLock(&subscriberLock, lockFlags);
    return result;
 #  undef VMCI_EVENT_MAX_ATTEMPTS
 }
@@ -674,13 +627,13 @@ VMCIEventUnregisterSubscription(VMCIId subID)    // IN
    VMCILockFlags flags;
    VMCISubscription *s;
 
-   VMCI_GrabLock_BH(&subscriberLock, &flags);
+   VMCIEventGrabLock(&subscriberLock, &flags);
    s = VMCIEventFind(subID);
    if (s != NULL) {
       VMCIEventRelease(s);
-      VMCIList_Remove(&s->subscriberListItem);
+      VMCIList_Remove(&s->subscriberListItem, &subscriberArray[s->event]);
    }
-   VMCI_ReleaseLock_BH(&subscriberLock, flags);
+   VMCIEventReleaseLock(&subscriberLock, flags);
 
    if (s != NULL) {
       VMCI_WaitOnEvent(&s->destroyEvent, EventReleaseCB, s);
@@ -694,7 +647,7 @@ VMCIEventUnregisterSubscription(VMCIId subID)    // IN
 /*
  *----------------------------------------------------------------------
  *
- * vmci_event_subscribe --
+ * VMCIEvent_Subscribe --
  *
  *      Subscribe to given event. The callback specified can be fired
  *      in different contexts depending on what flag is specified while
@@ -714,20 +667,15 @@ VMCIEventUnregisterSubscription(VMCIId subID)    // IN
  *----------------------------------------------------------------------
  */
 
-VMCI_EXPORT_SYMBOL(vmci_event_subscribe)
+VMCI_EXPORT_SYMBOL(VMCIEvent_Subscribe)
 int
-vmci_event_subscribe(VMCI_Event event,        // IN
-#if !defined(linux) || defined(VMKERNEL)
-                     uint32 flags,            // IN
-#endif // !linux || VMKERNEL
-                     VMCI_EventCB callback,   // IN
-                     void *callbackData,      // IN
-                     VMCIId *subscriptionID)  // OUT
+VMCIEvent_Subscribe(VMCI_Event event,        // IN
+                    uint32 flags,            // IN
+                    VMCI_EventCB callback,   // IN
+                    void *callbackData,      // IN
+                    VMCIId *subscriptionID)  // OUT
 {
    int retval;
-#if defined(linux) && !defined(VMKERNEL)
-   uint32 flags = VMCI_FLAG_EVENT_NONE;
-#endif // linux && !VMKERNEL
    VMCISubscription *s = NULL;
 
    if (subscriptionID == NULL) {
@@ -755,7 +703,7 @@ vmci_event_subscribe(VMCI_Event event,        // IN
 /*
  *----------------------------------------------------------------------
  *
- * vmci_event_unsubscribe --
+ * VMCIEvent_Unsubscribe --
  *
  *      Unsubscribe to given event. Removes it from list and frees it.
  *      Will return callbackData if requested by caller.
@@ -769,9 +717,9 @@ vmci_event_subscribe(VMCI_Event event,        // IN
  *----------------------------------------------------------------------
  */
 
-VMCI_EXPORT_SYMBOL(vmci_event_unsubscribe)
+VMCI_EXPORT_SYMBOL(VMCIEvent_Unsubscribe)
 int
-vmci_event_unsubscribe(VMCIId subID)   // IN
+VMCIEvent_Unsubscribe(VMCIId subID)   // IN
 {
    VMCISubscription *s;
 
