@@ -72,19 +72,14 @@ static VMCIHashTable *resourceTable = NULL;
 int
 VMCIResource_Init(void)
 {
-   int err = VMCI_InitLock(&resourceIdLock, "VMCIRIDLock",
-                           VMCI_LOCK_RANK_RESOURCE);
-   if (err < VMCI_SUCCESS) {
-      return err;
-   }
-
    resourceTable = VMCIHashTable_Create(128);
    if (resourceTable == NULL) {
       VMCI_WARNING((LGPFX"Failed creating a resource hash table for VMCI.\n"));
-      VMCI_CleanupLock(&resourceIdLock);
       return VMCI_ERROR_NO_MEM;
    }
 
+   VMCI_InitLock(&resourceIdLock, "VMCIRIDLock", VMCI_LOCK_RANK_HIGHEST);
+
    return VMCI_SUCCESS;
 }
 
@@ -210,6 +205,7 @@ VMCIResource_Add(VMCIResource *resource,                // IN
    resource->type = resourceType;
    resource->containerFreeCB = containerFreeCB;
    resource->containerObject = containerObject;
+   resource->handle = resourceHandle;
 
    /* Add resource to hashtable. */
    result = VMCIHashTable_AddEntry(resourceTable, &resource->hashEntry);
@@ -376,30 +372,6 @@ VMCIResource_Release(VMCIResource *resource)
 }
 
 
-/*
- *------------------------------------------------------------------------------
- *
- * VMCIResource_Handle --
- *
- *      Get the handle for the given resource.
- *
- * Results:
- *      The resource's associated handle.
- *
- * Side effects:
- *      None.
- *
- *------------------------------------------------------------------------------
- */
-
-VMCIHandle
-VMCIResource_Handle(VMCIResource *resource)
-{
-   ASSERT(resource);
-   return resource->hashEntry.handle;
-}
-
-
 /*
  *------------------------------------------------------------------------------
  *
