@@ -67,20 +67,7 @@
  */
 
 struct VMCIQueueKernelIf {
-   VMCIMutex __mutex;               /* Protects the queue. */
-   VMCIMutex *mutex;                /* Shared by producer/consumer queues. */
-   size_t numPages;                 /* Number of pages incl. header. */
-   Bool host;                       /* Host or guest? */
-   union {
-      struct {
-         dma_addr_t *pas;           /* Physical addresses. */
-         void **vas;                /* Virtual addresses. */
-      } g;                          /* Guest. */
-      struct {
-         struct page **headerPage;  /* Guest queue header pages. */
-         struct page **page;        /* Guest queue pages. */
-      } h;                          /* Host. */
-   } u;
+   struct page **page;
 };
 
 typedef struct VMCIDelayedWorkInfo {
@@ -89,8 +76,6 @@ typedef struct VMCIDelayedWorkInfo {
    void *data;
 } VMCIDelayedWorkInfo;
 
-extern struct pci_dev *vmci_pdev;
-
 
 /*
  *-----------------------------------------------------------------------------
@@ -100,7 +85,7 @@ extern struct pci_dev *vmci_pdev;
  *      Initializes the lock. Must be called before use.
  *
  * Results:
- *      Always VMCI_SUCCESS.
+ *      None
  *
  * Side effects:
  *      Thread can block.
@@ -108,13 +93,12 @@ extern struct pci_dev *vmci_pdev;
  *-----------------------------------------------------------------------------
  */
 
-int
+void
 VMCI_InitLock(VMCILock *lock,    // IN:
               char *name,        // IN: Unused on Linux
               VMCILockRank rank) // IN: Unused on Linux
 {
    spin_lock_init(lock);
-   return VMCI_SUCCESS;
 }
 
 
@@ -428,13 +412,11 @@ int VMCIHost_CompareUser(VMCIHostUser *user1,
       return VMCI_ERROR_INVALID_ARGS;
    }
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 5, 0)
-#  define vmw_uid_eq(a, b) uid_eq(a, b)
-#else
-#  define vmw_uid_eq(a, b) ((a) == (b))
-#endif
-
-   return vmw_uid_eq(*user1, *user2) ? VMCI_SUCCESS : VMCI_ERROR_GENERIC;
+   if (*user1 == *user2) {
+      return VMCI_SUCCESS;
+   } else {
+      return VMCI_ERROR_GENERIC;
+   }
 }
 
 
@@ -723,8 +705,8 @@ VMCI_SignalEvent(VMCIEvent *event)  // IN:
 
 void
 VMCI_WaitOnEvent(VMCIEvent *event,              // IN:
-                 VMCIEventReleaseCB releaseCB,  // IN:
-                 void *clientData)              // IN:
+		 VMCIEventReleaseCB releaseCB,  // IN:
+		 void *clientData)              // IN:
 {
    /*
     * XXX Should this be a TASK_UNINTERRUPTIBLE wait? I'm leaving it
@@ -796,9 +778,7 @@ VMCI_WaitOnEventInterruptible(VMCIEvent *event,              // IN:
  */
 
 int
-VMCIMutex_Init(VMCIMutex *mutex,  // IN
-               char *name,        // IN: Unused
-               VMCILockRank rank) // IN: Unused
+VMCIMutex_Init(VMCIMutex *mutex) // IN:
 {
    sema_init(mutex, 1);
    return VMCI_SUCCESS;
@@ -878,9 +858,14 @@ VMCIMutex_Release(VMCIMutex *mutex) // IN:
  *
  * VMCI_AllocQueue --
  *
- *      Allocates kernel queue pages of specified size with IOMMU mappings,
- *      plus space for the queue structure/kernel interface and the queue
- *      header.
+ *      Allocates kernel VA space of specified size, plus space for the
+ *      queue structure/kernel interface and the queue header.  Allocates
+ *      physical pages for the queue data pages.
+ *
+ *      PAGE m:      VMCIQueueHeader (VMCIQueue->qHeader)
+ *      PAGE m+1:    VMCIQueue
+ *      PAGE m+1+q:  VMCIQueueKernelIf (VMCIQueue->kernelIf)
+ *      PAGE n-size: Data pages (VMCIQueue->kernelIf->page[])
  *
  * Results:
  *      Pointer to the queue on success, NULL otherwise.
@@ -892,57 +877,39 @@ VMCIMutex_Release(VMCIMutex *mutex) // IN:
  */
 
 void *
-VMCI_AllocQueue(uint64 size,  // IN: size of queue (not including header)
-                uint32 flags) // IN: queuepair flags
+VMCI_AllocQueue(uint64 size) // IN: size of queue (not including header)
 {
-   size_t i;
+   uint64 i;
    VMCIQueue *queue;
-   const size_t numPages = CEILING(size, PAGE_SIZE) + 1;
-   const size_t pasSize = numPages * sizeof *queue->kernelIf->u.g.pas;
-   const size_t vasSize = numPages * sizeof *queue->kernelIf->u.g.vas;
-   const size_t queueSize =
-      sizeof *queue + sizeof *(queue->kernelIf) + pasSize + vasSize;
+   VMCIQueueHeader *qHeader;
+   const uint64 numDataPages = CEILING(size, PAGE_SIZE);
+   const uint queueSize =
+      PAGE_SIZE +
+      sizeof *queue + sizeof *(queue->kernelIf) +
+      numDataPages * sizeof *(queue->kernelIf->page);
 
-   /*
-    * Size should be enforced by VMCIQPair_Alloc(), double-check here.
-    * Allocating too much on Linux can cause the system to become
-    * unresponsive, because we allocate page-by-page, and we allow the
-    * system to wait for pages rather than fail.
-    */
-
-   if (size > VMCI_MAX_GUEST_QP_MEMORY) {
-      ASSERT(FALSE);
+   qHeader = (VMCIQueueHeader *)vmalloc(queueSize);
+   if (!qHeader) {
       return NULL;
    }
 
-   queue = vmalloc(queueSize);
-   if (!queue) {
-      return NULL;
-   }
+   queue = (VMCIQueue *)((uint8 *)qHeader + PAGE_SIZE);
+   queue->qHeader = qHeader;
+   queue->kernelIf = (VMCIQueueKernelIf *)((uint8 *)queue + sizeof *queue);
+   queue->kernelIf->page = (struct page **)((uint8 *)queue->kernelIf +
+                                            sizeof *(queue->kernelIf));
 
-   queue->qHeader = NULL;
-   queue->savedHeader = NULL;
-   queue->kernelIf = (VMCIQueueKernelIf *)(queue + 1);
-   queue->kernelIf->mutex = NULL;
-   queue->kernelIf->numPages = numPages;
-   queue->kernelIf->u.g.pas = (dma_addr_t *)(queue->kernelIf + 1);
-   queue->kernelIf->u.g.vas =
-      (void **)((uint8 *)queue->kernelIf->u.g.pas + pasSize);
-   queue->kernelIf->host = FALSE;
-
-   for (i = 0; i < numPages; i++) {
-      queue->kernelIf->u.g.vas[i] =
-         dma_alloc_coherent(&vmci_pdev->dev, PAGE_SIZE,
-                            &queue->kernelIf->u.g.pas[i], GFP_KERNEL);
-      if (!queue->kernelIf->u.g.vas[i]) {
-         VMCI_FreeQueue(queue, i * PAGE_SIZE); /* Size excl. the header. */
+   for (i = 0; i < numDataPages; i++) {
+      queue->kernelIf->page[i] = alloc_pages(GFP_KERNEL, 0);
+      if (!queue->kernelIf->page[i]) {
+         while (i) {
+            __free_page(queue->kernelIf->page[--i]);
+         }
+         vfree(qHeader);
          return NULL;
       }
    }
 
-   /* Queue header is the first page. */
-   queue->qHeader = queue->kernelIf->u.g.vas[0];
-
    return (void *)queue;
 }
 
@@ -972,14 +939,10 @@ VMCI_FreeQueue(void *q,     // IN:
 
    if (queue) {
       uint64 i;
-
-      /* Given size does not include header, so add in a page here. */
-      for (i = 0; i < CEILING(size, PAGE_SIZE) + 1; i++) {
-         dma_free_coherent(&vmci_pdev->dev, PAGE_SIZE,
-                           queue->kernelIf->u.g.vas[i],
-                           queue->kernelIf->u.g.pas[i]);
+      for (i = 0; i < CEILING(size, PAGE_SIZE); i++) {
+         __free_page(queue->kernelIf->page[i]);
       }
-      vfree(queue);
+      vfree(queue->qHeader);
    }
 }
 
@@ -1040,29 +1003,33 @@ VMCI_AllocPPNSet(void *prodQ,            // IN:
       return VMCI_ERROR_NO_MEM;
    }
 
-   for (i = 0; i < numProducePages; i++) {
+   producePPNs[0] = page_to_pfn(vmalloc_to_page(produceQ->qHeader));
+   for (i = 1; i < numProducePages; i++) {
       unsigned long pfn;
 
-      producePPNs[i] = pfn = produceQ->kernelIf->u.g.pas[i] >> PAGE_SHIFT;
+      producePPNs[i] = pfn = page_to_pfn(produceQ->kernelIf->page[i - 1]);
 
       /*
        * Fail allocation if PFN isn't supported by hypervisor.
        */
 
-      if (sizeof pfn > sizeof *producePPNs && pfn != producePPNs[i]) {
+      if (sizeof pfn > sizeof *producePPNs &&
+          pfn != producePPNs[i]) {
          goto ppnError;
       }
    }
-   for (i = 0; i < numConsumePages; i++) {
+   consumePPNs[0] = page_to_pfn(vmalloc_to_page(consumeQ->qHeader));
+   for (i = 1; i < numConsumePages; i++) {
       unsigned long pfn;
 
-      consumePPNs[i] = pfn = consumeQ->kernelIf->u.g.pas[i] >> PAGE_SHIFT;
+      consumePPNs[i] = pfn = page_to_pfn(consumeQ->kernelIf->page[i - 1]);
 
       /*
        * Fail allocation if PFN isn't supported by hypervisor.
        */
 
-      if (sizeof pfn > sizeof *consumePPNs && pfn != consumePPNs[i]) {
+      if (sizeof pfn > sizeof *consumePPNs &&
+          pfn != consumePPNs[i]) {
          goto ppnError;
       }
    }
@@ -1178,17 +1145,11 @@ __VMCIMemcpyToQueue(VMCIQueue *queue,   // OUT:
    size_t bytesCopied = 0;
 
    while (bytesCopied < size) {
-      const uint64 pageIndex = (queueOffset + bytesCopied) / PAGE_SIZE;
-      const size_t pageOffset = (queueOffset + bytesCopied) & (PAGE_SIZE - 1);
-      void *va;
+      uint64 pageIndex = (queueOffset + bytesCopied) / PAGE_SIZE;
+      size_t pageOffset = (queueOffset + bytesCopied) & (PAGE_SIZE - 1);
+      void *va = kmap(kernelIf->page[pageIndex]);
       size_t toCopy;
 
-      if (kernelIf->host) {
-         va = kmap(kernelIf->u.h.page[pageIndex]);
-      } else {
-         va = kernelIf->u.g.vas[pageIndex + 1]; /* Skip header. */
-      }
-
       ASSERT(va);
       if (size - bytesCopied > PAGE_SIZE - pageOffset) {
          /* Enough payload to fill up from this page. */
@@ -1204,9 +1165,7 @@ __VMCIMemcpyToQueue(VMCIQueue *queue,   // OUT:
          /* The iovec will track bytesCopied internally. */
          err = memcpy_fromiovec((uint8 *)va + pageOffset, iov, toCopy);
          if (err != 0) {
-            if (kernelIf->host) {
-               kunmap(kernelIf->u.h.page[pageIndex]);
-            }
+            kunmap(kernelIf->page[pageIndex]);
             return VMCI_ERROR_INVALID_ARGS;
          }
       } else {
@@ -1214,9 +1173,7 @@ __VMCIMemcpyToQueue(VMCIQueue *queue,   // OUT:
       }
 
       bytesCopied += toCopy;
-      if (kernelIf->host) {
-         kunmap(kernelIf->u.h.page[pageIndex]);
-      }
+      kunmap(kernelIf->page[pageIndex]);
    }
 
    return VMCI_SUCCESS;
@@ -1253,17 +1210,11 @@ __VMCIMemcpyFromQueue(void *dest,             // OUT:
    size_t bytesCopied = 0;
 
    while (bytesCopied < size) {
-      const uint64 pageIndex = (queueOffset + bytesCopied) / PAGE_SIZE;
-      const size_t pageOffset = (queueOffset + bytesCopied) & (PAGE_SIZE - 1);
-      void *va;
+      uint64 pageIndex = (queueOffset + bytesCopied) / PAGE_SIZE;
+      size_t pageOffset = (queueOffset + bytesCopied) & (PAGE_SIZE - 1);
+      void *va = kmap(kernelIf->page[pageIndex]);
       size_t toCopy;
 
-      if (kernelIf->host) {
-         va = kmap(kernelIf->u.h.page[pageIndex]);
-      } else {
-         va = kernelIf->u.g.vas[pageIndex + 1]; /* Skip header. */
-      }
-
       ASSERT(va);
       if (size - bytesCopied > PAGE_SIZE - pageOffset) {
          /* Enough payload to fill up this page. */
@@ -1279,9 +1230,7 @@ __VMCIMemcpyFromQueue(void *dest,             // OUT:
          /* The iovec will track bytesCopied internally. */
          err = memcpy_toiovec(iov, (uint8 *)va + pageOffset, toCopy);
          if (err != 0) {
-            if (kernelIf->host) {
-               kunmap(kernelIf->u.h.page[pageIndex]);
-            }
+            kunmap(kernelIf->page[pageIndex]);
             return VMCI_ERROR_INVALID_ARGS;
          }
       } else {
@@ -1289,9 +1238,7 @@ __VMCIMemcpyFromQueue(void *dest,             // OUT:
       }
 
       bytesCopied += toCopy;
-      if (kernelIf->host) {
-         kunmap(kernelIf->u.h.page[pageIndex]);
-      }
+      kunmap(kernelIf->page[pageIndex]);
    }
 
    return VMCI_SUCCESS;
@@ -1320,11 +1267,8 @@ VMCIMemcpyToQueue(VMCIQueue *queue,   // OUT:
                   const void *src,    // IN:
                   size_t srcOffset,   // IN:
                   size_t size,        // IN:
-                  int bufType,        // IN: Unused
-                  Bool canBlock)      // IN: Unused
+                  int bufType)        // IN: Unused
 {
-   ASSERT(canBlock || !queue->kernelIf->host);
-
    return __VMCIMemcpyToQueue(queue, queueOffset,
                               (uint8 *)src + srcOffset, size, FALSE);
 }
@@ -1352,76 +1296,8 @@ VMCIMemcpyFromQueue(void *dest,             // OUT:
                     const VMCIQueue *queue, // IN:
                     uint64 queueOffset,     // IN:
                     size_t size,            // IN:
-                    int bufType,            // IN: Unused
-                    Bool canBlock)          // IN: Unused
+                    int bufType)            // IN: Unused
 {
-   ASSERT(canBlock || !queue->kernelIf->host);
-
-   return __VMCIMemcpyFromQueue((uint8 *)dest + destOffset,
-                                queue, queueOffset, size, FALSE);
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * VMCIMemcpyToQueueLocal --
- *
- *      Copies from a given buffer to a local VMCI queue. On Linux, this is the
- *      same as a regular copy.
- *
- * Results:
- *      Zero on success, negative error code on failure.
- *
- * Side effects:
- *      None.
- *
- *-----------------------------------------------------------------------------
- */
-
-int
-VMCIMemcpyToQueueLocal(VMCIQueue *queue,     // OUT
-                       uint64 queueOffset,   // IN
-                       const void *src,      // IN
-                       size_t srcOffset,     // IN
-                       size_t size,          // IN
-                       int bufType,          // IN: Unused
-                       Bool canBlock)        // IN: Unused
-{
-   ASSERT(canBlock || !queue->kernelIf->host);
-
-   return __VMCIMemcpyToQueue(queue, queueOffset,
-                              (uint8 *)src + srcOffset, size, FALSE);;
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * VMCIMemcpyFromQueueLocal --
- *
- *      Copies to a given buffer from a VMCI Queue.
- *
- * Results:
- *      Zero on success, negative error code on failure.
- *
- * Side effects:
- *      None.
- *
- *-----------------------------------------------------------------------------
- */
-
-int
-VMCIMemcpyFromQueueLocal(void *dest,             // OUT:
-                         size_t destOffset,      // IN:
-                         const VMCIQueue *queue, // IN:
-                         uint64 queueOffset,     // IN:
-                         size_t size,            // IN:
-                         int bufType,            // IN: Unused
-                         Bool canBlock)          // IN: Unused
-{
-   ASSERT(canBlock || !queue->kernelIf->host);
-
    return __VMCIMemcpyFromQueue((uint8 *)dest + destOffset,
                                 queue, queueOffset, size, FALSE);
 }
@@ -1449,10 +1325,8 @@ VMCIMemcpyToQueueV(VMCIQueue *queue,      // OUT:
                    const void *src,       // IN: iovec
                    size_t srcOffset,      // IN: ignored
                    size_t size,           // IN:
-                   int bufType,           // IN: Unused
-                   Bool canBlock)         // IN: Unused
+                   int bufType)           // IN: ignored
 {
-   ASSERT(canBlock || !queue->kernelIf->host);
 
    /*
     * We ignore srcOffset because src is really a struct iovec * and will
@@ -1484,11 +1358,8 @@ VMCIMemcpyFromQueueV(void *dest,              // OUT: iovec
                      const VMCIQueue *queue,  // IN:
                      uint64 queueOffset,      // IN:
                      size_t size,             // IN:
-                     int bufType,             // IN: Unused
-                     Bool canBlock)           // IN: Unused
+                     int bufType)             // IN: ignored
 {
-   ASSERT(canBlock || !queue->kernelIf->host);
-
    /*
     * We ignore destOffset because dest is really a struct iovec * and will
     * maintain offset internally.
@@ -1555,24 +1426,12 @@ VMCIQueue *
 VMCIHost_AllocQueue(uint64 size) // IN:
 {
    VMCIQueue *queue;
-   const size_t numPages = CEILING(size, PAGE_SIZE) + 1;
-   const size_t queueSize = sizeof *queue + sizeof *(queue->kernelIf);
-   const size_t queuePageSize = numPages * sizeof *queue->kernelIf->u.h.page;
+   const uint queueSize = sizeof *queue + sizeof *(queue->kernelIf);
 
-   queue = VMCI_AllocKernelMem(queueSize + queuePageSize, VMCI_MEMORY_NORMAL);
+   queue = VMCI_AllocKernelMem(queueSize, VMCI_MEMORY_NORMAL);
    if (queue) {
       queue->qHeader = NULL;
-      queue->savedHeader = NULL;
-      queue->kernelIf = (VMCIQueueKernelIf *)(queue + 1);
-      queue->kernelIf->host = TRUE;
-      queue->kernelIf->mutex = NULL;
-      queue->kernelIf->numPages = numPages;
-      queue->kernelIf->u.h.headerPage =
-         (struct page **)((uint8*)queue + queueSize);
-      queue->kernelIf->u.h.page = &queue->kernelIf->u.h.headerPage[1];
-      memset(queue->kernelIf->u.h.headerPage, 0,
-             (sizeof *queue->kernelIf->u.h.headerPage *
-              queue->kernelIf->numPages));
+      queue->kernelIf = (VMCIQueueKernelIf *)((uint8 *)queue + sizeof *queue);
    }
 
    return queue;
@@ -1607,424 +1466,10 @@ VMCIHost_FreeQueue(VMCIQueue *queue,   // IN:
 }
 
 
-/*
- *-----------------------------------------------------------------------------
- *
- * VMCI_InitQueueMutex()
- *
- *       Initialize the mutex for the pair of queues.  This mutex is used to
- *       protect the qHeader and the buffer from changing out from under any
- *       users of either queue.  Of course, it's only any good if the mutexes
- *       are actually acquired.  Queue structure must lie on non-paged memory
- *       or we cannot guarantee access to the mutex.
- *
- * Results:
- *       None.
- *
- * Side Effects:
- *       None.
- *
- *----------------------------------------------------------------------------
- */
-
-void
-VMCI_InitQueueMutex(VMCIQueue *produceQ, // IN/OUT
-                    VMCIQueue *consumeQ) // IN/OUT
-{
-   ASSERT(produceQ);
-   ASSERT(consumeQ);
-   ASSERT(produceQ->kernelIf);
-   ASSERT(consumeQ->kernelIf);
-
-   /*
-    * Only the host queue has shared state - the guest queues do not
-    * need to synchronize access using a queue mutex.
-    */
-
-   if (produceQ->kernelIf->host) {
-      produceQ->kernelIf->mutex = &produceQ->kernelIf->__mutex;
-      consumeQ->kernelIf->mutex = &produceQ->kernelIf->__mutex;
-      sema_init(produceQ->kernelIf->mutex, 1);
-   }
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * VMCI_CleanupQueueMutex()
- *
- *       Cleans up the mutex for the pair of queues.
- *
- * Results:
- *       None.
- *
- * Side Effects:
- *       None.
- *
- *----------------------------------------------------------------------------
- */
-
-void
-VMCI_CleanupQueueMutex(VMCIQueue *produceQ, // IN/OUT
-                       VMCIQueue *consumeQ) // IN/OUT
-{
-   ASSERT(produceQ);
-   ASSERT(consumeQ);
-   ASSERT(produceQ->kernelIf);
-   ASSERT(consumeQ->kernelIf);
-
-   if (produceQ->kernelIf->host) {
-      produceQ->kernelIf->mutex = NULL;
-      consumeQ->kernelIf->mutex = NULL;
-   }
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * VMCI_AcquireQueueMutex()
- *
- *       Acquire the mutex for the queue.  Note that the produceQ and
- *       the consumeQ share a mutex.  So, only one of the two need to
- *       be passed in to this routine.  Either will work just fine.
- *
- * Results:
- *       VMCI_SUCCESS always.
- *
- * Side Effects:
- *       May block the caller.
- *
- *----------------------------------------------------------------------------
- */
-
-int
-VMCI_AcquireQueueMutex(VMCIQueue *queue, // IN
-                       Bool canBlock)    // IN: Unused
-{
-   ASSERT(queue);
-   ASSERT(queue->kernelIf);
-
-   if (queue->kernelIf->host) {
-      ASSERT(canBlock);
-      ASSERT(queue->kernelIf->mutex);
-      down(queue->kernelIf->mutex);
-   }
-
-   return VMCI_SUCCESS;
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * VMCI_ReleaseQueueMutex()
- *
- *       Release the mutex for the queue.  Note that the produceQ and
- *       the consumeQ share a mutex.  So, only one of the two need to
- *       be passed in to this routine.  Either will work just fine.
- *
- * Results:
- *       None.
- *
- * Side Effects:
- *       May block the caller.
- *
- *----------------------------------------------------------------------------
- */
-
-void
-VMCI_ReleaseQueueMutex(VMCIQueue *queue) // IN
-{
-   ASSERT(queue);
-   ASSERT(queue->kernelIf);
-
-   if (queue->kernelIf->host) {
-      ASSERT(queue->kernelIf->mutex);
-      up(queue->kernelIf->mutex);
-   }
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * VMCI_LockQueueHeader()
- *
- *       Acquire a spinlock guarding the queue header.  Note that the produceQ
- *       and the consumeQ share the lock mutex.  So, only one of the two need to
- *       be passed in to this routine.  Either will work just fine.
- *
- * Results:
- *       None.
- *
- * Side Effects:
- *       None.
- *
- *----------------------------------------------------------------------------
- */
-
-void
-VMCI_LockQueueHeader(VMCIQueue *queue) // IN
-{
-   ASSERT(queue);
-   ASSERT(queue->kernelIf);
-   ASSERT(!queue->kernelIf->host);
-
-   /*
-    * We don't support non-blocking on the host right now, so we won't get
-    * here for a host queue.  And there's no lock required on the guest.  So
-    * this is a NOP.
-    */
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * VMCI_UnlockQueueHeader()
- *
- *       Release the spinlock guarding the queue header.
- *
- * Results:
- *       None.
- *
- * Side Effects:
- *       None.
- *
- *----------------------------------------------------------------------------
- */
-
-void
-VMCI_UnlockQueueHeader(VMCIQueue *queue) // IN
-{
-   ASSERT(queue);
-   ASSERT(queue->kernelIf);
-   ASSERT(!queue->kernelIf->host);
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * VMCIReleasePageStorePages --
- *
- *       Helper function to release pages in the PageStoreAttachInfo
- *       previously obtained using get_user_pages.
- *
- * Results:
- *       None.
- *
- * Side Effects:
- *       None.
- *
- *-----------------------------------------------------------------------------
- */
-
-static void
-VMCIReleasePages(struct page **pages,  // IN
-                 uint64 numPages,      // IN
-                 Bool dirty)           // IN
-{
-   int i;
-
-   for (i = 0; i < numPages; i++) {
-      ASSERT(pages[i]);
-      if (dirty) {
-         set_page_dirty(pages[i]);
-      }
-      page_cache_release(pages[i]);
-      pages[i] = NULL;
-   }
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * VMCIHost_RegisterUserMemory --
- *
- *       Registers the specification of the user pages used for backing a queue
- *       pair. Enough information to map in pages is stored in the OS specific
- *       part of the VMCIQueue structure.
- *
- * Results:
- *       VMCI_SUCCESS on sucess, negative error code on failure.
- *
- * Side Effects:
- *       None.
- *
- *-----------------------------------------------------------------------------
- */
-
-int
-VMCIHost_RegisterUserMemory(QueuePairPageStore *pageStore,  // IN
-                            VMCIQueue *produceQ,            // OUT
-                            VMCIQueue *consumeQ)            // OUT
-{
-   VA64 produceUVA;
-   VA64 consumeUVA;
-
-   ASSERT(produceQ->kernelIf->u.h.headerPage &&
-          consumeQ->kernelIf->u.h.headerPage);
-
-   /*
-    * The new style and the old style mapping only differs in that we either
-    * get a single or two UVAs, so we split the single UVA range at the
-    * appropriate spot.
-    */
-
-   produceUVA = pageStore->pages;
-   consumeUVA = pageStore->pages + produceQ->kernelIf->numPages * PAGE_SIZE;
-   return VMCIHost_GetUserMemory(produceUVA, consumeUVA, produceQ, consumeQ);
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * VMCIHost_UnregisterUserMemory --
- *
- *       Releases and removes the references to user pages stored in the attach
- *       struct.
- *
- * Results:
- *       None
- *
- * Side Effects:
- *       Pages are released from the page cache and may become
- *       swappable again.
- *
- *-----------------------------------------------------------------------------
- */
-
-void
-VMCIHost_UnregisterUserMemory(VMCIQueue *produceQ,         // IN/OUT
-                              VMCIQueue *consumeQ)         // IN/OUT
-{
-   ASSERT(produceQ->kernelIf);
-   ASSERT(consumeQ->kernelIf);
-   ASSERT(!produceQ->qHeader && !consumeQ->qHeader);
-
-   VMCIReleasePages(produceQ->kernelIf->u.h.headerPage,
-                    produceQ->kernelIf->numPages, TRUE);
-   memset(produceQ->kernelIf->u.h.headerPage, 0,
-          (sizeof *produceQ->kernelIf->u.h.headerPage *
-           produceQ->kernelIf->numPages));
-   VMCIReleasePages(consumeQ->kernelIf->u.h.headerPage,
-                    consumeQ->kernelIf->numPages, TRUE);
-   memset(consumeQ->kernelIf->u.h.headerPage, 0,
-          (sizeof *consumeQ->kernelIf->u.h.headerPage *
-           consumeQ->kernelIf->numPages));
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * VMCIHost_MapQueues --
- *
- *       Once VMCIHost_RegisterUserMemory has been performed on a
- *       queue, the queue pair headers can be mapped into the
- *       kernel. Once mapped, they must be unmapped with
- *       VMCIHost_UnmapQueues prior to calling
- *       VMCIHost_UnregisterUserMemory.
- *
- * Results:
- *       VMCI_SUCCESS if pages are mapped, appropriate error code otherwise.
- *
- * Side Effects:
- *       Pages are pinned.
- *
- *-----------------------------------------------------------------------------
- */
-
-int
-VMCIHost_MapQueues(VMCIQueue *produceQ,  // IN/OUT
-                   VMCIQueue *consumeQ,  // IN/OUT
-                   uint32 flags)         // UNUSED
-{
-   int result;
-
-   if (!produceQ->qHeader || !consumeQ->qHeader) {
-      struct page *headers[2];
-
-      if (produceQ->qHeader != consumeQ->qHeader) {
-         return VMCI_ERROR_QUEUEPAIR_MISMATCH;
-      }
-
-      if (produceQ->kernelIf->u.h.headerPage == NULL ||
-          *produceQ->kernelIf->u.h.headerPage == NULL) {
-         return VMCI_ERROR_UNAVAILABLE;
-      }
-
-      ASSERT(*produceQ->kernelIf->u.h.headerPage &&
-             *consumeQ->kernelIf->u.h.headerPage);
-
-      headers[0] = *produceQ->kernelIf->u.h.headerPage;
-      headers[1] = *consumeQ->kernelIf->u.h.headerPage;
-
-      produceQ->qHeader = vmap(headers, 2, VM_MAP, PAGE_KERNEL);
-      if (produceQ->qHeader != NULL) {
-         consumeQ->qHeader =
-            (VMCIQueueHeader *)((uint8 *)produceQ->qHeader + PAGE_SIZE);
-         result = VMCI_SUCCESS;
-      } else {
-         Log("vmap failed\n");
-         result = VMCI_ERROR_NO_MEM;
-     }
-   } else {
-      result = VMCI_SUCCESS;
-   }
-
-   return result;
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * VMCIHost_UnmapQueues --
- *
- *       Unmaps previously mapped queue pair headers from the kernel.
- *
- * Results:
- *       VMCI_SUCCESS always.
- *
- * Side Effects:
- *       Pages are unpinned.
- *
- *-----------------------------------------------------------------------------
- */
-
-int
-VMCIHost_UnmapQueues(VMCIGuestMemID gid,   // IN
-                     VMCIQueue *produceQ,  // IN/OUT
-                     VMCIQueue *consumeQ)  // IN/OUT
-{
-   if (produceQ->qHeader) {
-      ASSERT(consumeQ->qHeader);
-
-      if (produceQ->qHeader < consumeQ->qHeader) {
-         vunmap(produceQ->qHeader);
-      } else {
-         vunmap(consumeQ->qHeader);
-      }
-      produceQ->qHeader = NULL;
-      consumeQ->qHeader = NULL;
-   }
-
-   return VMCI_SUCCESS;
-}
-
-
 /*
  *-----------------------------------------------------------------------------
  *
  * VMCIHost_GetUserMemory --
- *
- *
  *       Lock the user pages referenced by the {produce,consume}Buffer
  *       struct into memory and populate the {produce,consume}Pages
  *       arrays in the attach structure with them.
@@ -2039,47 +1484,93 @@ VMCIHost_UnmapQueues(VMCIGuestMemID gid,   // IN
  */
 
 int
-VMCIHost_GetUserMemory(VA64 produceUVA,       // IN
-                       VA64 consumeUVA,       // IN
-                       VMCIQueue *produceQ,   // OUT
-                       VMCIQueue *consumeQ)   // OUT
+VMCIHost_GetUserMemory(PageStoreAttachInfo *attach,      // IN/OUT
+                       VMCIQueue *produceQ,              // OUT
+                       VMCIQueue *consumeQ)              // OUT
 {
    int retval;
    int err = VMCI_SUCCESS;
 
+
+   attach->producePages =
+      VMCI_AllocKernelMem(attach->numProducePages * sizeof attach->producePages[0],
+                          VMCI_MEMORY_NORMAL);
+   if (attach->producePages == NULL) {
+      return VMCI_ERROR_NO_MEM;
+   }
+   attach->consumePages =
+      VMCI_AllocKernelMem(attach->numConsumePages * sizeof attach->consumePages[0],
+                          VMCI_MEMORY_NORMAL);
+   if (attach->consumePages == NULL) {
+      err = VMCI_ERROR_NO_MEM;
+      goto errorDealloc;
+   }
+
    down_write(&current->mm->mmap_sem);
    retval = get_user_pages(current,
                            current->mm,
-                           (VA)produceUVA,
-                           produceQ->kernelIf->numPages,
+                           (VA)attach->produceBuffer,
+                           attach->numProducePages,
                            1, 0,
-                           produceQ->kernelIf->u.h.headerPage,
+                           attach->producePages,
                            NULL);
-   if (retval < produceQ->kernelIf->numPages) {
-      Log("get_user_pages(produce) failed (retval=%d)\n", retval);
-      VMCIReleasePages(produceQ->kernelIf->u.h.headerPage, retval, FALSE);
+   if (retval < attach->numProducePages) {
+      Log("get_user_pages(produce) failed: %d\n", retval);
+      if (retval > 0) {
+         int i;
+         for (i = 0; i < retval; i++) {
+            page_cache_release(attach->producePages[i]);
+         }
+      }
       err = VMCI_ERROR_NO_MEM;
       goto out;
    }
 
    retval = get_user_pages(current,
                            current->mm,
-                           (VA)consumeUVA,
-                           consumeQ->kernelIf->numPages,
+                           (VA)attach->consumeBuffer,
+                           attach->numConsumePages,
                            1, 0,
-                           consumeQ->kernelIf->u.h.headerPage,
+                           attach->consumePages,
                            NULL);
-   if (retval < consumeQ->kernelIf->numPages) {
-      Log("get_user_pages(consume) failed (retval=%d)\n", retval);
-      VMCIReleasePages(consumeQ->kernelIf->u.h.headerPage, retval, FALSE);
-      VMCIReleasePages(produceQ->kernelIf->u.h.headerPage,
-                       produceQ->kernelIf->numPages, FALSE);
+   if (retval < attach->numConsumePages) {
+      int i;
+      Log("get_user_pages(consume) failed: %d\n", retval);
+      if (retval > 0) {
+         for (i = 0; i < retval; i++) {
+            page_cache_release(attach->consumePages[i]);
+         }
+      }
+      for (i = 0; i < attach->numProducePages; i++) {
+         page_cache_release(attach->producePages[i]);
+      }
       err = VMCI_ERROR_NO_MEM;
    }
 
 out:
    up_write(&current->mm->mmap_sem);
 
+   if (err == VMCI_SUCCESS) {
+      produceQ->qHeader = kmap(attach->producePages[0]);
+      produceQ->kernelIf->page = &attach->producePages[1];
+      consumeQ->qHeader = kmap(attach->consumePages[0]);
+      consumeQ->kernelIf->page = &attach->consumePages[1];
+   }
+
+errorDealloc:
+   if (err < VMCI_SUCCESS) {
+      if (attach->producePages != NULL) {
+         VMCI_FreeKernelMem(attach->producePages,
+                            attach->numProducePages *
+                            sizeof attach->producePages[0]);
+      }
+      if (attach->consumePages != NULL) {
+         VMCI_FreeKernelMem(attach->consumePages,
+                            attach->numConsumePages *
+                            sizeof attach->consumePages[0]);
+      }
+   }
+
    return err;
 }
 
@@ -2102,12 +1593,38 @@ out:
  */
 
 void
-VMCIHost_ReleaseUserMemory(VMCIQueue *produceQ,  // IN/OUT
-                           VMCIQueue *consumeQ)  // IN/OUT
+VMCIHost_ReleaseUserMemory(PageStoreAttachInfo *attach,      // IN/OUT
+                           VMCIQueue *produceQ,              // OUT
+                           VMCIQueue *consumeQ)              // OUT
 {
-   ASSERT(produceQ->kernelIf->u.h.headerPage);
+   int i;
 
-   VMCIHost_UnregisterUserMemory(produceQ, consumeQ);
+   ASSERT(attach->producePages);
+   ASSERT(attach->consumePages);
+
+   kunmap(attach->producePages[0]);
+   kunmap(attach->consumePages[0]);
+
+   for (i = 0; i < attach->numProducePages; i++) {
+      ASSERT(attach->producePages[i]);
+
+      set_page_dirty(attach->producePages[i]);
+      page_cache_release(attach->producePages[i]);
+   }
+
+   for (i = 0; i < attach->numConsumePages; i++) {
+      ASSERT(attach->consumePages[i]);
+
+      set_page_dirty(attach->consumePages[i]);
+      page_cache_release(attach->consumePages[i]);
+   }
+
+   VMCI_FreeKernelMem(attach->producePages,
+                      attach->numProducePages *
+                      sizeof attach->producePages[0]);
+   VMCI_FreeKernelMem(attach->consumePages,
+                      attach->numConsumePages *
+                      sizeof attach->consumePages[0]);
 }
 
 
@@ -2129,9 +1646,9 @@ VMCIHost_ReleaseUserMemory(VMCIQueue *produceQ,  // IN/OUT
 
 void
 VMCI_ReadPortBytes(VMCIIoHandle handle,  // IN: Unused
-                   VMCIIoPort port,      // IN
-                   uint8 *buffer,        // OUT
-                   size_t bufferLength)  // IN
+		   VMCIIoPort port,      // IN
+		   uint8 *buffer,        // OUT
+		   size_t bufferLength)  // IN
 {
    insb(port, buffer, bufferLength);
 }
