@@ -75,6 +75,11 @@ static ssize_t HgfsWrite(struct file *file,
                          size_t count,
                          loff_t *offset);
 #endif
+
+static loff_t HgfsSeek(struct file *file,
+                       loff_t  offset,
+                       int origin);
+
 static int HgfsFsync(struct file *file,
 #if defined VMW_FSYNC_OLD
                      struct dentry *dentry,
@@ -112,6 +117,7 @@ static ssize_t HgfsSpliceRead(struct file *file,
 struct file_operations HgfsFileFileOperations = {
    .owner      = THIS_MODULE,
    .open       = HgfsOpen,
+   .llseek     = HgfsSeek,
 #if defined VMW_USE_AIO
    .aio_read   = HgfsAioRead,
    .aio_write  = HgfsAioWrite,
@@ -897,6 +903,51 @@ HgfsWrite(struct file *file,      // IN: File to write to
 }
 #endif
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * HgfsSeek --
+ *
+ *    Called whenever a process moves the file pointer for a file in our
+ *    filesystem. Our function is just a thin wrapper around
+ *    generic_file_llseek() that tries to validate the dentry first.
+ *
+ * Results:
+ *    Returns the new position of the file pointer on success,
+ *    or a negative error on failure.
+ *
+ * Side effects:
+ *    None
+ *
+ *----------------------------------------------------------------------
+ */
+
+static loff_t
+HgfsSeek(struct file *file,  // IN:  File to seek
+         loff_t offset,      // IN:  Number of bytes to seek
+         int origin)         // IN:  Position to seek from
+
+{
+   loff_t result = -1;
+
+   ASSERT(file);
+   ASSERT(file->f_dentry);
+
+   LOG(6, (KERN_DEBUG "VMware hgfs: HgfsSeek: seek to %Lu bytes from fh %u "
+           "from position %d\n", offset, FILE_GET_FI_P(file)->handle, origin));
+
+   result = (loff_t) HgfsRevalidate(file->f_dentry);
+   if (result) {
+      LOG(6, (KERN_DEBUG "VMware hgfs: HgfsSeek: invalid dentry\n"));
+      goto out;
+   }
+
+   result = generic_file_llseek(file, offset, origin);
+
+  out:
+   return result;
+}
+
 
 /*
  *----------------------------------------------------------------------
