@@ -43,9 +43,6 @@
 #include "compat_uaccess.h"
 #include "compat_version.h"
 
-/* Must be included after sched.h. */
-#include <linux/smp_lock.h>
-
 #include "filesystem.h"
 #include "transport.h"
 #include "hgfsDevLinux.h"
@@ -60,10 +57,10 @@
 #include "hgfs.h"
 
 /* Synchronization primitives. */
-spinlock_t hgfsBigLock = SPIN_LOCK_UNLOCKED;
+DEFINE_SPINLOCK(hgfsBigLock);
 
 /* Other variables. */
-compat_kmem_cache *hgfsInodeCache = NULL;
+compat_kmem_cache *hgfsInodeCache;
 
 /* Global protocol version switch. */
 HgfsOp hgfsVersionOpen;
@@ -93,7 +90,12 @@ static void HgfsResetOps(void);
 
 
 /* HGFS filesystem high-level operations. */
-#if defined VMW_GETSB_2618
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 38)
+static struct dentry *HgfsMount(struct file_system_type *fs_type,
+                                int flags,
+                                const char *dev_name,
+                                void *rawData);
+#elif defined(VMW_GETSB_2618)
 static int HgfsGetSb(struct file_system_type *fs_type,
                      int flags,
                      const char *dev_name,
@@ -112,7 +114,11 @@ static struct file_system_type hgfsType = {
    .name         = HGFS_NAME,
 
    .fs_flags     = FS_BINARY_MOUNTDATA,
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 38)
+   .mount        = HgfsMount,
+#else
    .get_sb       = HgfsGetSb,
+#endif
    .kill_sb      = kill_anon_super,
 };
 
@@ -435,7 +441,33 @@ HgfsReadSuper(struct super_block *sb, // OUT: Superblock object
  * HGFS filesystem high-level operations.
  */
 
-#if defined VMW_GETSB_2618
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 38)
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * HgfsMount --
+ *
+ *    Invokes generic kernel code to mount a deviceless filesystem.
+ *
+ * Results:
+ *    Mount's root dentry structure on success
+ *    ERR_PTR()-encoded negative error code on failure
+ *
+ * Side effects:
+ *    None
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+struct dentry *
+HgfsMount(struct file_system_type *fs_type, // IN: file system type of mount
+          int flags,                        // IN: mount flags
+          const char *dev_name,             // IN: device mounting on
+          void *rawData)                    // IN: mount arguments
+{
+   return mount_nodev(fs_type, flags, rawData, HgfsReadSuper);
+}
+#elif defined VMW_GETSB_2618
 /*
  *-----------------------------------------------------------------------------
  *
