@@ -33,9 +33,9 @@
 #include <linux/net.h>
 #include <linux/inet.h>
 #include <linux/errno.h>
+#include <linux/kthread.h>
 
 #include "compat_kernel.h"
-#include "compat_kthread.h"
 #include "compat_mutex.h"
 #include "compat_version.h"
 #include "compat_sched.h"
@@ -406,11 +406,11 @@ HgfsSocketReceiveHandler(void *data)
 
       /* Wait for data to become available */
       wait_event_interruptible(hgfsRecvThreadWait,
-                  (HgfsSocketIsReceiverIdle() && compat_kthread_should_stop()) ||
+                  (HgfsSocketIsReceiverIdle() && kthread_should_stop()) ||
                   test_bit(HGFS_REQ_THREAD_RECV, &hgfsRecvThreadFlags));
 
       /* Kill yourself if told so. */
-      if (compat_kthread_should_stop()) {
+      if (kthread_should_stop()) {
          LOG(6, (KERN_DEBUG "VMware hgfs: %s: told to exit\n", __func__));
          break;
       }
@@ -476,7 +476,7 @@ HgfsCreateTcpSocket(void)
    addr.sin_addr.s_addr = in_aton(HOST_IP);
    addr.sin_port = htons(HOST_PORT);
 
-   error = compat_sock_create_kern(AF_INET, SOCK_STREAM, IPPROTO_TCP, &socket);
+   error = sock_create_kern(AF_INET, SOCK_STREAM, IPPROTO_TCP, &socket);
    if (error < 0) {
       LOG(8, ("%s: sock_create_kern failed: %d.\n", __func__, error));
       return NULL;
@@ -525,7 +525,7 @@ HgfsCreateVsockSocket(void)
    addr.svm_cid = VMCI_HOST_CONTEXT_ID;
    addr.svm_port = HOST_VSOCKET_PORT;
 
-   error = compat_sock_create_kern(family, SOCK_STREAM, IPPROTO_TCP, &socket);
+   error = sock_create_kern(family, SOCK_STREAM, IPPROTO_TCP, &socket);
    if (error < 0) {
       LOG(8, ("%s: sock_create_kern failed: %d.\n", __func__, error));
       return NULL;
@@ -580,7 +580,7 @@ HgfsSocketChannelOpen(HgfsTransportChannel *channel,
     * Install the new "data ready" handler that will wake up the
     * receiving thread.
     */
-   oldSocketDataReady = xchg(&socket->sk->compat_sk_data_ready,
+   oldSocketDataReady = xchg(&socket->sk->sk_data_ready,
                              HgfsSocketDataReady);
 
    /* Reset receive buffer when a new connection is connected. */
@@ -591,8 +591,7 @@ HgfsSocketChannelOpen(HgfsTransportChannel *channel,
    LOG(8, ("%s: socket channel connected.\n", __func__));
 
    /* Create the recv thread. */
-   recvThread = compat_kthread_run(HgfsSocketReceiveHandler,
-                                   channel, "vmhgfs-rep");
+   recvThread = kthread_run(HgfsSocketReceiveHandler, channel, "vmhgfs-rep");
    if (IS_ERR(recvThread)) {
       LOG(4, (KERN_ERR "VMware hgfs: %s: "
               "failed to create recv thread, err %ld\n",
@@ -682,7 +681,7 @@ HgfsSocketChannelClose(HgfsTransportChannel *channel)
 {
    /* Stop the recv thread before we change the channel status. */
    ASSERT(recvThread != NULL);
-   compat_kthread_stop(recvThread);
+   kthread_stop(recvThread);
 
    sock_release(channel->priv);
    channel->priv = NULL;
