@@ -77,11 +77,16 @@ VSockVmciNotifyWaitingWrite(VSockVmciSock *vsk)    // IN
 
    if (!PKT_FIELD(vsk, peerWaitingWriteDetected)) {
       PKT_FIELD(vsk, peerWaitingWriteDetected) = TRUE;
-      PKT_FIELD(vsk, writeNotifyWindow) -= PAGE_SIZE;
-      if (PKT_FIELD(vsk, writeNotifyWindow) <
-                    PKT_FIELD(vsk, writeNotifyMinWindow)) {
+      if (PKT_FIELD(vsk, writeNotifyWindow) < PAGE_SIZE) {
          PKT_FIELD(vsk, writeNotifyWindow) =
             PKT_FIELD(vsk, writeNotifyMinWindow);
+      } else {
+         PKT_FIELD(vsk, writeNotifyWindow) -= PAGE_SIZE;
+         if (PKT_FIELD(vsk, writeNotifyWindow) <
+             PKT_FIELD(vsk, writeNotifyMinWindow)) {
+            PKT_FIELD(vsk, writeNotifyWindow) =
+               PKT_FIELD(vsk, writeNotifyMinWindow);
+         }
       }
    }
    notifyLimit = vsk->consumeSize - PKT_FIELD(vsk, writeNotifyWindow);
@@ -538,7 +543,7 @@ VSockVmciNotifyPktSocketInit(struct sock *sk) // IN
    VSockVmciSock *vsk;
    vsk = vsock_sk(sk);
 
-   PKT_FIELD(vsk, writeNotifyWindow) = 0;
+   PKT_FIELD(vsk, writeNotifyWindow) = PAGE_SIZE;
    PKT_FIELD(vsk, writeNotifyMinWindow) = PAGE_SIZE;
    PKT_FIELD(vsk, peerWaitingRead) = FALSE;
    PKT_FIELD(vsk, peerWaitingWrite) = FALSE;
@@ -1151,6 +1156,9 @@ VSockVmciNotifyPktProcessRequest(struct sock *sk) // IN
    vsk = vsock_sk(sk);
 
    PKT_FIELD(vsk, writeNotifyWindow) = vsk->consumeSize;
+   if (vsk->consumeSize < PKT_FIELD(vsk, writeNotifyMinWindow)) {
+      PKT_FIELD(vsk, writeNotifyMinWindow) = vsk->consumeSize;
+   }
 }
 
 
@@ -1180,6 +1188,9 @@ VSockVmciNotifyPktProcessNegotiate(struct sock *sk) // IN
    vsk = vsock_sk(sk);
 
    PKT_FIELD(vsk, writeNotifyWindow) = vsk->consumeSize;
+   if (vsk->consumeSize < PKT_FIELD(vsk, writeNotifyMinWindow)) {
+      PKT_FIELD(vsk, writeNotifyMinWindow) = vsk->consumeSize;
+   }
 }
 
 
