@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2010-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2010 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -26,8 +26,6 @@
 #if !defined _COPYPASTE_COMPAT_H
 #define _COPYPASTE_COMPAT_H
 
-#include "vmware.h"
-
 /* Functions to be compatible with old text copy/paste directly based on backdoor cmd. */
 Bool CopyPaste_RequestSelection(void);
 Bool CopyPaste_GetBackdoorSelections(void);
@@ -35,17 +33,4 @@ Bool CopyPaste_IsRpcCPSupported(void);
 void CopyPaste_Init(void);
 void CopyPaste_SetVersion(int version);
 
-int32
-CopyPaste_GetHostSelectionLen(void);
-
-void
-CopyPaste_GetHostSelection(unsigned int size, // IN
-                           char *data);       // OUT
-
-void
-CopyPaste_SetSelLength(uint32 length); // IN
-
-void
-CopyPaste_SetNextPiece(uint32 data); // IN
-
 #endif
