@@ -87,6 +87,7 @@ GdkAtom GDK_SELECTION_TYPE_UTF8_STRING;
 CopyPasteUIX11::CopyPasteUIX11()
  : mClipboardEmpty(true),
    mHGStagingDir(""),
+   mIsClipboardOwner(false),
    mHGGetFilesInitiated(false),
    mFileTransferDone(false),
    mBlockAdded(false),
@@ -129,13 +130,11 @@ CopyPasteUIX11::Init()
 
    Gtk::TargetEntry gnome(FCP_TARGET_NAME_GNOME_COPIED_FILES);
    Gtk::TargetEntry kde(FCP_TARGET_NAME_URI_LIST);
-   Gtk::TargetEntry vmware(VMWARE_TARGET);
    gnome.set_info(FCP_TARGET_INFO_GNOME_COPIED_FILES);
    kde.set_info(FCP_TARGET_INFO_URI_LIST);
 
    mListTargets.push_back(gnome);
    mListTargets.push_back(kde);
-   mListTargets.push_back(vmware);
 
    mCP->srcRecvClipChanged.connect(
       sigc::mem_fun(this, &CopyPasteUIX11::GetRemoteClipboardCB));
@@ -219,15 +218,9 @@ CopyPasteUIX11::VmxCopyPasteVersionChanged(RpcChannel *chan,    // IN
 void
 CopyPasteUIX11::GetLocalClipboard(void)
 {
-   Glib::RefPtr<Gtk::Clipboard> refClipboard =
-      Gtk::Clipboard::get(GDK_SELECTION_CLIPBOARD);
-   Glib::RefPtr<Gtk::Clipboard> refPrimary =
-      Gtk::Clipboard::get(GDK_SELECTION_PRIMARY);
-
    g_debug("%s: enter.\n", __FUNCTION__);
 
-   if (refClipboard->wait_is_target_available(VMWARE_TARGET) &&
-       refPrimary->wait_is_target_available(VMWARE_TARGET)) {
+   if (mIsClipboardOwner) {
       /* If we are clipboard owner, send a not-changed clip to host. */
       g_debug("%s: we are owner, send unchanged clip back.\n", __FUNCTION__);
       CPClipboard clip;
@@ -243,6 +236,9 @@ CopyPasteUIX11::GetLocalClipboard(void)
       return;
    }
 
+   Glib::RefPtr<Gtk::Clipboard> refClipboard =
+      Gtk::Clipboard::get(GDK_SELECTION_CLIPBOARD);
+
    mClipTime = 0;
    mPrimTime = 0;
    mGHSelection = GDK_SELECTION_CLIPBOARD;
@@ -305,10 +301,8 @@ CopyPasteUIX11::LocalGetFileRequestCB(Gtk::SelectionData& sd,        // IN:
                                       guint info)                    // IN:
 {
    g_debug("%s: enter.\n", __FUNCTION__);
+   mHGCopiedUriList = "";
    VmTimeType curTime;
-   mBlockAdded = false;
-
-   sd.set(sd.get_target().c_str(), "");
 
    curTime = GetCurrentTime();
 
@@ -323,15 +317,28 @@ CopyPasteUIX11::LocalGetFileRequestCB(Gtk::SelectionData& sd,        // IN:
       return;
    }
 
-   if (!mCP->IsCopyPasteAllowed()) {
-      g_debug("%s: copy paste is not allowed, returning.\n",
+   if (!mIsClipboardOwner || !mCP->IsCopyPasteAllowed()) {
+      g_debug("%s: not clipboard ownder, or copy paste not allowed, returning.\n",
             __FUNCTION__);
+      sd.set(sd.get_target().c_str(), "");
       return;
    }
 
    g_debug("%s: Got paste request, target is %s\n", __FUNCTION__,
          sd.get_target().c_str());
 
+   if (mHGGetFilesInitiated) {
+      /*
+       * On KDE (at least), we can see this multiple times, so ignore if
+       * we are already getting files.
+       */
+      g_debug("%s: already processing a GetFile request, returning.\n",
+              __FUNCTION__);
+      sd.set(sd.get_target().c_str(), mHGCopiedUriList.c_str());
+      return;
+   }
+
+
    /* Copy the files. */
    if (!mHGGetFilesInitiated) {
       utf::string str;
@@ -348,10 +355,12 @@ CopyPasteUIX11::LocalGetFileRequestCB(Gtk::SelectionData& sd,        // IN:
 
       if (0 == hgStagingDir.bytes()) {
          g_debug("%s: Can not create staging directory\n", __FUNCTION__);
+         sd.set(sd.get_target().c_str(), "");
          return;
       }
       mHGGetFilesInitiated = true;
 
+      mBlockAdded = false;
       if (DnD_BlockIsReady(mBlockCtrl) && mBlockCtrl->AddBlock(mBlockCtrl->fd, hgStagingDir.c_str())) {
          g_debug("%s: add block for %s.\n",
                __FUNCTION__, hgStagingDir.c_str());
@@ -374,6 +383,7 @@ CopyPasteUIX11::LocalGetFileRequestCB(Gtk::SelectionData& sd,        // IN:
       } else {
          g_debug("%s: Unknown request target: %s\n", __FUNCTION__,
                sd.get_target().c_str());
+         sd.set(sd.get_target().c_str(), "");
          return;
       }
 
@@ -381,6 +391,7 @@ CopyPasteUIX11::LocalGetFileRequestCB(Gtk::SelectionData& sd,        // IN:
       stagingDirName = GetLastDirName(hgStagingDir);
       if (0 == stagingDirName.bytes()) {
          g_debug("%s: Can not get staging directory name\n", __FUNCTION__);
+         sd.set(sd.get_target().c_str(), "");
          return;
       }
 
@@ -403,6 +414,7 @@ CopyPasteUIX11::LocalGetFileRequestCB(Gtk::SelectionData& sd,        // IN:
 
    if (0 == mHGCopiedUriList.bytes()) {
       g_debug("%s: Can not get uri list\n", __FUNCTION__);
+      sd.set(sd.get_target().c_str(), "");
       return;
    }
 
@@ -436,6 +448,7 @@ CopyPasteUIX11::LocalGetFileRequestCB(Gtk::SelectionData& sd,        // IN:
          if (select(0, NULL, NULL, NULL, &tv) == -1) {
             g_debug("%s: error in select (%s).\n", __FUNCTION__,
                   strerror(errno));
+            sd.set(sd.get_target().c_str(), "");
             return;
          }
       }
@@ -483,13 +496,13 @@ CopyPasteUIX11::LocalGetTextOrRTFRequestCB(Gtk::SelectionData& sd, // IN/OUT
 
    if (target == TARGET_NAME_APPLICATION_RTF ||
        target == TARGET_NAME_TEXT_RICHTEXT) {
-      if (0 == mHGRTFData.bytes()) {
+      if (0 == mHGRTFData.size()) {
          g_debug("%s: Can not get valid RTF data\n", __FUNCTION__);
          return;
       }
 
       g_debug("%s: providing RTF data, size %"FMTSZ"u\n",
-            __FUNCTION__, mHGRTFData.bytes());
+            __FUNCTION__, mHGRTFData.size());
 
       sd.set(target.c_str(), mHGRTFData.c_str());
    }
@@ -530,6 +543,7 @@ void
 CopyPasteUIX11::LocalClearClipboardCB(void)
 {
    g_debug("%s: got clear callback\n", __FUNCTION__);
+   mIsClipboardOwner = FALSE;
 }
 
 
@@ -621,20 +635,13 @@ again:
 
    CPClipboard_Clear(&mClipboard);
 
-   if (refClipboard->wait_is_target_available(VMWARE_TARGET)) {
-      g_debug("%s: we are owner, send unchanged clip back.\n", __FUNCTION__);
-      CPClipboard_SetChanged(&mClipboard, FALSE);
-      mCP->DestUISendClip(&mClipboard);
-      return;
-   }
-
    /* First check for URIs. This must always be done first */
    bool haveURIs = false;
    std::string format;
-   if (refClipboard->wait_is_target_available(FCP_TARGET_NAME_GNOME_COPIED_FILES)) {
+   if (mCP->CheckCapability(DND_CP_CAP_FILE_CP) && refClipboard->wait_is_target_available(FCP_TARGET_NAME_GNOME_COPIED_FILES)) {
       format = FCP_TARGET_NAME_GNOME_COPIED_FILES;
       haveURIs = true;
-   } else if (refClipboard->wait_is_target_available(FCP_TARGET_NAME_URI_LIST)) {
+   } else if (mCP->CheckCapability(DND_CP_CAP_FILE_CP) && refClipboard->wait_is_target_available(FCP_TARGET_NAME_URI_LIST)) {
       format = FCP_TARGET_NAME_URI_LIST;
       haveURIs = true;
    }
@@ -649,7 +656,7 @@ again:
    /* Try to get image data from clipboard. */
    Glib::RefPtr<Gdk::Pixbuf> img = refClipboard->wait_for_image();
    gsize bufSize;
-   if (img) {
+   if (mCP->CheckCapability(DND_CP_CAP_IMAGE_CP) && img) {
       gchar *buf = NULL;
 
       img->save_to_buffer(buf, bufSize, Glib::ustring("png"));
@@ -678,7 +685,7 @@ again:
       haveRTF = true;
    }
 
-   if (haveRTF) {
+   if (mCP->CheckCapability(DND_CP_CAP_RTF_CP) && haveRTF) {
       /*
        * There is a function for waiting for rtf data, but that was leading
        * to crashes. It's use required we instantiate a class that implements
@@ -701,7 +708,8 @@ again:
    }
 
    /* Try to get Text data from clipboard. */
-   if (refClipboard->wait_is_text_available()) {
+   if (mCP->CheckCapability(DND_CP_CAP_PLAIN_TEXT_CP) &&
+       refClipboard->wait_is_text_available()) {
       g_debug("%s: ask for text\n", __FUNCTION__);
       Glib::ustring str = refClipboard->wait_for_text();
       bufSize = str.bytes();
@@ -763,6 +771,12 @@ CopyPasteUIX11::LocalReceivedFileListCB(const Gtk::SelectionData& sd)        //
    g_debug("%s: enter", __FUNCTION__);
    const utf::string target = sd.get_target().c_str();
 
+   if (!mCP->CheckCapability(DND_CP_CAP_FILE_CP)) {
+      /*
+       * Disallowed based on caps settings, return.
+       */
+      return;
+   }
    if (target == FCP_TARGET_NAME_GNOME_COPIED_FILES ||
        target == FCP_TARGET_NAME_URI_LIST) {
       LocalGetSelectionFileList(sd);
@@ -799,6 +813,13 @@ CopyPasteUIX11::LocalGetFileContentsRequestCB(Gtk::SelectionData& sd, // IN
    utf::string pre;
    utf::string post;
 
+   if (!mCP->CheckCapability(DND_CP_CAP_FILE_CONTENT_CP)) {
+      /*
+       * Disallowed based on caps settings, return.
+       */
+      return;
+   }
+
    sd.set(sd.get_target().c_str(), "");
 
    /* Provide URIs for each path in the guest's file list. */
@@ -917,11 +938,8 @@ CopyPasteUIX11::LocalGetSelectionFileList(const Gtk::SelectionData& sd)      //
        */
       newRelPath = Str_Strrchr(newPath, DIRSEPC) + 1; // Point to char after '/'
 
-      /*
-       * XXX For directory, value is -1, so if there is any directory,
-       * total size is not accurate.
-       */
-      if ((size = File_GetSize(newPath)) >= 0) {
+      /* Keep track of how big the fcp files are. */
+      if ((size = File_GetSizeEx(newPath)) >= 0) {
          totalSize += size;
       } else {
          g_debug("%s: Unable to get file size for %s\n", __FUNCTION__, newPath);
@@ -1082,13 +1100,11 @@ CopyPasteUIX11::GetRemoteClipboardCB(const CPClipboard *clip) // IN
    }
 
    /* Clear the clipboard contents if we are the owner. */
-   if (refClipboard->wait_is_target_available(VMWARE_TARGET)) {
+   if (mIsClipboardOwner) {
       refClipboard->clear();
-      g_debug("%s: Cleared local clipboard", __FUNCTION__);
-   }
-   if (refPrimary->wait_is_target_available(VMWARE_TARGET)) {
       refPrimary->clear();
-      g_debug("%s: Cleared local primary selection", __FUNCTION__);
+      mIsClipboardOwner = false;
+      g_debug("%s: Cleared local clipboard", __FUNCTION__);
    }
 
    mHGTextData.clear();
@@ -1099,33 +1115,35 @@ CopyPasteUIX11::GetRemoteClipboardCB(const CPClipboard *clip) // IN
        CPClipboard_ItemExists(clip, CPFORMAT_RTF)) {
       std::list<Gtk::TargetEntry> targets;
 
+      /*
+       * rtf should be first in the target list otherwise OpenOffice may not
+       * accept paste.
+       */
+      if (CPClipboard_GetItem(clip, CPFORMAT_RTF, &buf, &sz)) {
+         g_debug("%s: RTF data, size %"FMTSZ"u.\n", __FUNCTION__, sz);
+         Gtk::TargetEntry appRtf(TARGET_NAME_APPLICATION_RTF);
+         Gtk::TargetEntry textRtf(TARGET_NAME_TEXT_RICHTEXT);
+
+         targets.push_back(appRtf);
+         targets.push_back(textRtf);
+         mHGRTFData = std::string((const char *)buf);
+         mIsClipboardOwner = true;
+      }
+
       if (CPClipboard_GetItem(clip, CPFORMAT_TEXT, &buf, &sz)) {
          Gtk::TargetEntry stringText(TARGET_NAME_STRING);
          Gtk::TargetEntry plainText(TARGET_NAME_TEXT_PLAIN);
          Gtk::TargetEntry utf8Text(TARGET_NAME_UTF8_STRING);
          Gtk::TargetEntry compountText(TARGET_NAME_COMPOUND_TEXT);
-         Gtk::TargetEntry vmware(VMWARE_TARGET);
 
          g_debug("%s: Text data, size %"FMTSZ"u.\n", __FUNCTION__, sz);
          targets.push_back(stringText);
          targets.push_back(plainText);
          targets.push_back(utf8Text);
          targets.push_back(compountText);
-         targets.push_back(vmware);
          mHGTextData = utf::string(reinterpret_cast<char *>(buf),
                                    STRING_ENCODING_UTF8);
-      }
-
-      if (CPClipboard_GetItem(clip, CPFORMAT_RTF, &buf, &sz)) {
-         g_debug("%s: RTF data, size %"FMTSZ"u.\n", __FUNCTION__, sz);
-         Gtk::TargetEntry appRtf(TARGET_NAME_APPLICATION_RTF);
-         Gtk::TargetEntry textRtf(TARGET_NAME_TEXT_RICHTEXT);
-         Gtk::TargetEntry vmware(VMWARE_TARGET);
-
-         targets.push_back(appRtf);
-         targets.push_back(textRtf);
-         targets.push_back(vmware);
-         mHGRTFData = utf::string((const char *)buf);
+         mIsClipboardOwner = true;
       }
 
       refClipboard->set(targets,
@@ -1164,6 +1182,7 @@ CopyPasteUIX11::GetRemoteClipboardCB(const CPClipboard *clip) // IN
                       sigc::mem_fun(this, &CopyPasteUIX11::LocalGetFileRequestCB),
                       sigc::mem_fun(this, &CopyPasteUIX11::LocalClearClipboardCB));
 
+      mIsClipboardOwner = true;
       mHGGetListTime = GetCurrentTime();
       mHGGetFilesInitiated = false;
       mHGCopiedUriList = "";
@@ -1178,6 +1197,7 @@ CopyPasteUIX11::GetRemoteClipboardCB(const CPClipboard *clip) // IN
          refPrimary->set(mListTargets,
                          sigc::mem_fun(this, &CopyPasteUIX11::LocalGetFileContentsRequestCB),
                          sigc::mem_fun(this, &CopyPasteUIX11::LocalClearClipboardCB));
+         mIsClipboardOwner = true;
       }
    }
 }
@@ -1423,6 +1443,7 @@ CopyPasteUIX11::GetLocalFilesDone(bool success)
       /* Copied files are already removed in common layer. */
       mHGStagingDir.clear();
    }
+   mHGGetFilesInitiated = false;
 }
 
 
