@@ -100,6 +100,7 @@ private:
    bool mClipboardEmpty;
    utf::string mHGStagingDir;
    std::list<Gtk::TargetEntry> mListTargets;
+   bool mIsClipboardOwner;
    uint64 mClipTime;
    uint64 mPrimTime;
    GdkAtom mGHSelection;
@@ -111,7 +112,7 @@ private:
    utf::string mHGCopiedUriList;
    utf::utf8string mHGFCPData;
    utf::string mHGTextData;
-   utf::string mHGRTFData;
+   std::string mHGRTFData;
    std::vector<utf::string> mHGFileContentsList;
    bool mFileTransferDone;
    bool mBlockAdded;
