@@ -75,6 +75,7 @@ DnDUIX11::DnDUIX11(ToolsAppCtx *ctx)
       m_mousePosX(0),
       m_mousePosY(0),
       m_dc(NULL),
+      m_numPendingRequest(0),
       m_destDropTime(0)
 {
    g_debug("%s: enter\n", __FUNCTION__);
@@ -758,7 +759,10 @@ DnDUIX11::GtkDestDragMotionCB(const Glib::RefPtr<Gdk::DragContext> &dc,
          m_GHDnDInProgress = true;
          /* only begin drag enter after we get the data */
          /* Need to grab all of the data. */
-         m_detWnd->drag_get_data(dc, target, timeValue);
+         if (!RequestData(dc, timeValue)) {
+            g_debug("%s: RequestData failed.\n", __FUNCTION__);
+            return false;
+         }
       } else {
          g_debug("%s: Multiple drag motions before gh data has been received.\n",
                __FUNCTION__);
@@ -1039,8 +1043,6 @@ DnDUIX11::GtkDestDragDataReceivedCB(const Glib::RefPtr<Gdk::DragContext> &dc,
       return;
    }
 
-   CPClipboard_Clear(&m_clipboard);
-
    /*
     * Try to get data provided from the source.  If we cannot get any data,
     * there is no need to inform the guest of anything. If there is no data,
@@ -1052,6 +1054,12 @@ DnDUIX11::GtkDestDragDataReceivedCB(const Glib::RefPtr<Gdk::DragContext> &dc,
       CommonResetCB();
       return;
    }
+
+   m_numPendingRequest--;
+   if (m_numPendingRequest > 0) {
+      return;
+   }
+
    if (CPClipboard_IsEmpty(&m_clipboard)) {
       g_debug("%s: Failed getting item.\n", __FUNCTION__);
       CommonResetCB();
@@ -1147,7 +1155,7 @@ DnDUIX11::SetCPClipboardFromGtk(const Gtk::SelectionData& sd) // IN
    const utf::string target = sd.get_target().c_str();
 
    /* Try to get file list. */
-   if (target == DRAG_TARGET_NAME_URI_LIST) {
+   if (m_DnD->CheckCapability(DND_CP_CAP_FILE_DND) && target == DRAG_TARGET_NAME_URI_LIST) {
       /*
        * Turn the uri list into two \0  delimited lists. One for full paths and
        * one for just the last path component.
@@ -1202,7 +1210,8 @@ DnDUIX11::SetCPClipboardFromGtk(const Gtk::SelectionData& sd) // IN
           */
          newRelPath = Str_Strrchr(newPath, DIRSEPC) + 1; // Point to char after '/'
 
-         if ((size = File_GetSize(newPath)) >= 0) {
+         /* Keep track of how big the dnd files are. */
+         if ((size = File_GetSizeEx(newPath)) >= 0) {
             totalSize += size;
          } else {
             g_debug("%s: unable to get file size for %s\n", __FUNCTION__, newPath);
@@ -1224,16 +1233,17 @@ DnDUIX11::SetCPClipboardFromGtk(const Gtk::SelectionData& sd) // IN
    }
 
    /* Try to get plain text. */
-   if (target == TARGET_NAME_STRING ||
+   if (m_DnD->CheckCapability(DND_CP_CAP_PLAIN_TEXT_DND) && (
+       target == TARGET_NAME_STRING ||
        target == TARGET_NAME_TEXT_PLAIN ||
        target == TARGET_NAME_UTF8_STRING ||
-       target == TARGET_NAME_COMPOUND_TEXT) {
-      utf::string source = sd.get_data_as_string().c_str();
-      if (source.bytes() > 0 &&
-          source.bytes() < DNDMSG_MAX_ARGSZ &&
+       target == TARGET_NAME_COMPOUND_TEXT)) {
+      std::string source = sd.get_data_as_string();
+      if (source.size() > 0 &&
+          source.size() < DNDMSG_MAX_ARGSZ &&
           CPClipboard_SetItem(&m_clipboard, CPFORMAT_TEXT, source.c_str(),
-                              source.bytes() + 1)) {
-         g_debug("%s: Got text, size %"FMTSZ"u\n", __FUNCTION__, source.bytes());
+                              source.size() + 1)) {
+         g_debug("%s: Got text, size %"FMTSZ"u\n", __FUNCTION__, source.size());
       } else {
          g_debug("%s: Failed to get text\n", __FUNCTION__);
          return false;
@@ -1242,14 +1252,15 @@ DnDUIX11::SetCPClipboardFromGtk(const Gtk::SelectionData& sd) // IN
    }
 
    /* Try to get RTF string. */
-   if (target == TARGET_NAME_APPLICATION_RTF ||
-       target == TARGET_NAME_TEXT_RICHTEXT) {
-      utf::string source = sd.get_data_as_string().c_str();
-      if (source.bytes() > 0 &&
-          source.bytes() < DNDMSG_MAX_ARGSZ &&
+   if (m_DnD->CheckCapability(DND_CP_CAP_RTF_DND) && (
+       target == TARGET_NAME_APPLICATION_RTF ||
+       target == TARGET_NAME_TEXT_RICHTEXT)) {
+      std::string source = sd.get_data_as_string();
+      if (source.size() > 0 &&
+          source.size() < DNDMSG_MAX_ARGSZ &&
           CPClipboard_SetItem(&m_clipboard, CPFORMAT_RTF, source.c_str(),
-                              source.bytes() + 1)) {
-         g_debug("%s: Got RTF, size %"FMTSZ"u\n", __FUNCTION__, source.bytes());
+                              source.size() + 1)) {
+         g_debug("%s: Got RTF, size %"FMTSZ"u\n", __FUNCTION__, source.size());
          return true;
       } else {
          g_debug("%s: Failed to get text\n", __FUNCTION__ );
@@ -1260,6 +1271,68 @@ DnDUIX11::SetCPClipboardFromGtk(const Gtk::SelectionData& sd) // IN
 }
 
 
+/**
+ *
+ * Ask for clipboard data from drag source.
+ *
+ * @param[in] dc   Associated drag context
+ * @param[in] time Time of the request
+ *
+ * @return true if there is any data request, false otherwise.
+ */
+
+bool
+DnDUIX11::RequestData(const Glib::RefPtr<Gdk::DragContext> &dc,
+                      guint time)
+{
+   Glib::RefPtr<Gtk::TargetList> targets;
+   targets = Gtk::TargetList::create(std::list<Gtk::TargetEntry>());
+
+   CPClipboard_Clear(&m_clipboard);
+   m_numPendingRequest = 0;
+
+   /*
+    * First check file list. If file list is available, all other formats will
+    * be ignored.
+    */
+   targets->add(Glib::ustring(DRAG_TARGET_NAME_URI_LIST));
+   Glib::ustring target = m_detWnd->drag_dest_find_target(dc, targets);
+   targets->remove(Glib::ustring(DRAG_TARGET_NAME_URI_LIST));
+   if (target != "") {
+      m_detWnd->drag_get_data(dc, target, time);
+      m_numPendingRequest++;
+      return true;
+   }
+
+   /* Then check plain text. */
+   targets->add(Glib::ustring(TARGET_NAME_UTF8_STRING));
+   targets->add(Glib::ustring(TARGET_NAME_STRING));
+   targets->add(Glib::ustring(TARGET_NAME_TEXT_PLAIN));
+   targets->add(Glib::ustring(TARGET_NAME_COMPOUND_TEXT));
+   target = m_detWnd->drag_dest_find_target(dc, targets);
+   targets->remove(Glib::ustring(TARGET_NAME_STRING));
+   targets->remove(Glib::ustring(TARGET_NAME_TEXT_PLAIN));
+   targets->remove(Glib::ustring(TARGET_NAME_UTF8_STRING));
+   targets->remove(Glib::ustring(TARGET_NAME_COMPOUND_TEXT));
+   if (target != "") {
+      m_detWnd->drag_get_data(dc, target, time);
+      m_numPendingRequest++;
+   }
+
+   /* Then check RTF. */
+   targets->add(Glib::ustring(TARGET_NAME_APPLICATION_RTF));
+   targets->add(Glib::ustring(TARGET_NAME_TEXT_RICHTEXT));
+   target = m_detWnd->drag_dest_find_target(dc, targets);
+   targets->remove(Glib::ustring(TARGET_NAME_APPLICATION_RTF));
+   targets->remove(Glib::ustring(TARGET_NAME_TEXT_RICHTEXT));
+   if (target != "") {
+      m_detWnd->drag_get_data(dc, target, time);
+      m_numPendingRequest++;
+   }
+   return (m_numPendingRequest > 0);
+}
+
+
 /**
  *
  * Try to get last directory name from a full path name.
@@ -2029,7 +2102,7 @@ DnDUIX11::GetTimeInMillis(void)
 
 
 /**
- * Update version information in mDnD.
+ * Update version information in m_DnD.
  *
  * @param[ignored] chan RpcChannel pointer
  * @param[in] version the version negotiated with host.
