@@ -37,6 +37,7 @@ extern "C" {
 #include "copyPasteDnDWrapper.h"
 
 #include "pointer.h"
+#include "vmware/tools/utils.h"
 
 extern "C" {
    #include "vm_assert.h"
@@ -57,7 +58,7 @@ static void PointerGrabbed(void);
 static void PointerUngrabbed(void);
 static gboolean PointerUpdatePointerLoop(gpointer clientData);
 
-#define POINTER_UPDATE_TIMEOUT 10
+#define POINTER_UPDATE_TIMEOUT 100
 
 #if !defined(WIN32)
 /*
@@ -277,11 +278,9 @@ PointerUpdatePointerLoop(gpointer clientData) // IN: unused
       CopyPasteDnDWrapper *wrapper = CopyPasteDnDWrapper::GetInstance();
       ToolsAppCtx *ctx = wrapper->GetToolsAppCtx();
       if (ctx) {
-         src = g_timeout_source_new(POINTER_UPDATE_TIMEOUT);
-         if (src) {
-            VMTOOLSAPP_ATTACH_SOURCE(ctx, src, PointerUpdatePointerLoop, NULL, NULL);
-            g_source_unref(src);
-         }
+         src = VMTools_CreateTimer(POINTER_UPDATE_TIMEOUT);
+         VMTOOLSAPP_ATTACH_SOURCE(ctx, src, PointerUpdatePointerLoop, NULL, NULL);
+         g_source_unref(src);
       }
    }
 
@@ -311,7 +310,6 @@ PointerUpdatePointerLoop(gpointer clientData) // IN: unused
 void
 Pointer_Init(ToolsAppCtx *ctx)
 {
-   g_debug("%s: enter\n", __FUNCTION__);
    absoluteMouseState = GuestApp_GetAbsoluteMouseState();
    PointerUpdatePointerLoop(NULL);
    mouseIsGrabbed = FALSE;
