@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2010-2017,2019-2022 VMware, Inc. All rights reserved.
+ * Copyright (C) 2010 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -62,17 +62,24 @@
 
 #ifndef NO_MULTIMON
 
-#include "resolutionInt.h"
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+
+#include "resolutionInt.h"
+#include "resolutionRandR12.h"
+#include <X11/extensions/Xrandr.h>
 #include <X11/X.h>
 #include <X11/Xmd.h>
 #include <X11/extensions/panoramiXproto.h>
-#include <X11/extensions/Xrandr.h>
 
-#include "resolutionRandR12.h"
+#include <X11/Xlib.h>
+#undef Bool
+
+#include "vmware.h"
+#include "resolution.h"
+#include "debug.h"
+
 #include "str.h"
 #include "strutil.h"
 #include "util.h"
@@ -139,7 +146,7 @@ static FILE *_ofile;
 #define LOG_STOP fclose(_ofile)
 #else
 #define LOG_START
-#include <glib.h>
+#include <gtk/gtk.h>
 #define LOG_STOP
 #endif
 
@@ -304,7 +311,7 @@ RandR12GetInfo(Display *display,     // IN: Pointer to our display connection
    }
 
    /*
-    * Confidence checks. This should never really happen with current drivers.
+    * Sanity checks. This should never really happen with current drivers.
     */
 
    if (nVMWOutput != info->nOutput) {
@@ -341,7 +348,7 @@ RandR12GetInfo(Display *display,     // IN: Pointer to our display connection
    }
 
    /*
-    * Confidence check. This should never really happen with our drivers.
+    * Sanity check. This should never really happen with our drivers.
     */
 
    if (numVMWCrtc != nVMWOutput) {
@@ -361,17 +368,17 @@ RandR12GetInfo(Display *display,     // IN: Pointer to our display connection
  *
  * RandR12CrtcDisable --
  *
- *      Deactivate crtcs and associated outputs before an fb size change.
- *      The function deactivates crtcs and associated outputs
+ *      Disable crtcs and associated outputs before an fb size change.
+ *      The function disables crtcs and associated outputs
  *      1) whose scanout area is too big for the new fb size.
- *      2) that are going to be deactivated with the new topology.
+ *      2) that are going to be disabled with the new topology.
  *
  * Results:
  *      TRUE on success, FALSE on failure.
  *
  * Side effects:
  *      The RandR12info context is modified.
- *      The current mode of deactivated outputs is set to "None".
+ *      The current mode of disabled outputs is set to "None".
  *
  *-----------------------------------------------------------------------------
  */
@@ -779,7 +786,6 @@ RandR12SetupOutput(Display *display,        // IN: The display connection
                    int height)              // IN: Height of scanout area
 {
    RRCrtc crtcID = info->xrrRes->crtcs[rrOutput->crtc];
-   XRRCrtcInfo *crtcInfo = info->crtcs[rrOutput->crtc];
    XRRModeInfo *mode;
    Status ret;
 
@@ -792,14 +798,9 @@ RandR12SetupOutput(Display *display,        // IN: The display connection
    if (!mode) {
       return FALSE;
    }
-   if (!crtcInfo) {
-       g_warning("%s: Wasn't able to find crtc info for crtc id %d.\n", __func__,
-                   (int)crtcID);
-       return FALSE;
-   }
 
    ret = XRRSetCrtcConfig(display, info->xrrRes, crtcID, CurrentTime, x, y,
-                          mode->id, crtcInfo->rotation, &rrOutput->id, 1);
+                          mode->id, RR_Rotate_0, &rrOutput->id, 1);
    if (ret == Success) {
       rrOutput->mode = mode->id;
       return TRUE;
@@ -833,6 +834,8 @@ RandR12DeleteModes(Display *display,  // IN: The display connection
                    RandR12Info *info) // IN: RandR12Info context
 {
    XRRScreenResources *xrrRes = info->xrrRes;
+   XRRModeInfo *modeInfo;
+   RandR12Output *rrOutput;
    unsigned int i, j;
    unsigned int w, h;
    Bool used;
@@ -843,8 +846,7 @@ RandR12DeleteModes(Display *display,  // IN: The display connection
     */
 
    for (i = 0; i < xrrRes->nmode; ++i) {
-      XRRModeInfo *modeInfo = &xrrRes->modes[i];
-
+      modeInfo = &xrrRes->modes[i];
       if (sscanf(modeInfo->name, RR12_MODE_FORMAT, &w, &h) != 2) {
          continue;
       }
@@ -859,7 +861,7 @@ RandR12DeleteModes(Display *display,  // IN: The display connection
        */
 
       for (j = 0; j < info->nOutput; ++j) {
-         RandR12Output *rrOutput = &info->outputs[j];
+         rrOutput = &info->outputs[j];
 
          if (rrOutput->mode != modeInfo->id) {
             if (RandR12OutputHasMode(rrOutput->output, modeInfo)) {
@@ -914,6 +916,7 @@ RandR12Revert(Display *display,    // IN: The display connection
    unsigned int i;
    RandR12Info *info = *pInfo;
    XRRScreenResources *xrrRes = info->xrrRes;
+   RandR12Output *rrOutput;
    XRRCrtcInfo *crtc;
    RRCrtc crtcID;
 
@@ -921,7 +924,7 @@ RandR12Revert(Display *display,    // IN: The display connection
 
    for (i = 0; i < info->nOutput; ++i) {
 
-      RandR12Output *rrOutput = &info->outputs[i];
+      rrOutput = &info->outputs[i];
       crtc = info->crtcs[rrOutput->crtc];
       crtcID = xrrRes->crtcs[rrOutput->crtc];
 
@@ -1001,10 +1004,11 @@ RandR12_SetTopology(Display *dpy,           // IN/OUT: The display connection
       return FALSE;
    }
 
+   XGrabServer(dpy);
    info = RandR12GetInfo(dpy, rootWin);
    if (!info) {
       g_warning("%s: Setup info struct failed.\n", __func__);
-      return FALSE;
+      goto out_ungrab;
    }
 
    RandR12GetDpi(dpy, screen, info);
@@ -1050,6 +1054,7 @@ RandR12_SetTopology(Display *dpy,           // IN/OUT: The display connection
    RandR12DeleteModes(dpy, info);
 
    XSync(dpy, FALSE);
+   XUngrabServer(dpy);
    RandR12FreeInfo(info);
 
    LOG_STOP;
