@@ -240,7 +240,13 @@ Bool
 GHIPlatformIsSupported(void)
 {
    const char *desktopEnv = Xdg_DetectDesktopEnv();
-   return (strcmp(desktopEnv, "GNOME") == 0) || (strcmp(desktopEnv, "KDE") == 0);
+   Bool supported = (g_strcmp0(desktopEnv, "GNOME") == 0) ||
+                    (g_strcmp0(desktopEnv, "KDE") == 0);
+   if (!supported) {
+      g_message("GHI not available under unsupported desktop environment %s\n",
+                desktopEnv ? desktopEnv : "(nil)");
+   }
+   return supported;
 }
 
 
@@ -270,6 +276,14 @@ GHIPlatformInit(GMainLoop *mainLoop,            // IN
 
    Gtk::Main::init_gtkmm_internals();
 
+   if (!GHIPlatformIsSupported()) {
+      /*
+       * Don't bother allocating resources if running under an unsupported
+       * desktop environment.
+       */
+      return NULL;
+   }
+
    ghip = (GHIPlatform *) Util_SafeCalloc(1, sizeof *ghip);
    ghip->appsByWindowExecutable =
       g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
@@ -282,6 +296,7 @@ GHIPlatformInit(GMainLoop *mainLoop,            // IN
    }
 
    desktopEnv = Xdg_DetectDesktopEnv();
+   ASSERT(desktopEnv); // Asserting based on GHIPlatformIsSupported check above.
    g_desktop_app_info_set_desktop_env(desktopEnv);
 
 #ifdef REDIST_GMENU
@@ -376,6 +391,9 @@ GHIPlatformCleanup(GHIPlatform *ghip) // IN
       return;
    }
 
+#ifdef REDIST_GMENU
+   delete ghip->menuItemManager;
+#endif
    g_hash_table_destroy(ghip->appsByWindowExecutable);
    free(ghip);
 }
@@ -449,12 +467,19 @@ GHIPlatformGetBinaryInfo(GHIPlatform *ghip,         // IN: platform-specific sta
    memset(&uri, 0, sizeof uri);
    state.uri = &uri;
 
-   if (pathURIUtf8[0] == '/') {
-      realCmd = pathURIUtf8;
-   } else if (uriParseUriA(&state, pathURIUtf8) == URI_SUCCESS) {
+   /* Strip query component. */
+   size_t uriSize = strlen(pathURIUtf8) + 1;
+   gchar *uriSansQuery = (gchar*)g_alloca(uriSize);
+   memcpy(uriSansQuery, pathURIUtf8, uriSize);
+   gchar *tmp = strchr(uriSansQuery, '?');
+   if (tmp) { *tmp = '\0'; }
+
+   if (uriSansQuery[0] == '/') {
+      realCmd = uriSansQuery;
+   } else if (uriParseUriA(&state, uriSansQuery) == URI_SUCCESS) {
       if (URI_TEXTRANGE_EQUAL(uri.scheme, "file")) {
-         gchar* tmp = (gchar*)g_alloca(strlen(pathURIUtf8) + 1);
-         uriUriStringToUnixFilenameA(pathURIUtf8, tmp);
+         gchar* tmp = (gchar*)g_alloca(strlen(uriSansQuery) + 1);
+         uriUriStringToUnixFilenameA(uriSansQuery, tmp);
 
          Glib::ustring unixFile;
          unixFile.assign(tmp);
@@ -480,7 +505,7 @@ GHIPlatformGetBinaryInfo(GHIPlatform *ghip,         // IN: platform-specific sta
           *     content type.
           */
 
-         if (contentType == "application/x-desktop") {
+         if (g_str_has_suffix(unixFile.c_str(), ".desktop")) {
             Glib::RefPtr<Gio::DesktopAppInfo> desktopFileInfo;
 
             desktopFileInfo = Gio::DesktopAppInfo::create_from_filename(unixFile);
@@ -489,7 +514,7 @@ GHIPlatformGetBinaryInfo(GHIPlatform *ghip,         // IN: platform-specific sta
                GHIX11IconGetIconsForDesktopFile(unixFile.c_str(), iconList);
                success = TRUE;
             }
-         } else if (appMgr.GetAppByUri(pathURIUtf8, app)) {
+         } else if (appMgr.GetAppByUri(uriSansQuery, app)) {
             friendlyName = app.symbolicName;
             GHIX11IconGetIconsByName(app.iconName.c_str(), iconList);
             success = TRUE;
