@@ -50,7 +50,6 @@ extern "C" {
 #include "guestrpc/ghiShellAction.h"
 #include "guestrpc/ghiStartMenu.h"
 #include "guestrpc/ghiTrayIcon.h"
-#include "guestrpc/ghiTrashFolder.h"
 #include "vmware/guestrpc/capabilities.h"
 
 static DynBuf gTcloUpdate;
@@ -1196,216 +1195,6 @@ GHITcloRestoreOutlookTempFolder(RpcInData *data) // IN/OUT: RPC data
 }
 
 
-/*
- *-----------------------------------------------------------------------------
- *
- * GHITcloTrashFolderAction --
- *
- *      Wrapper function for the ghi.guest.trashFolder.action RPC.
- *
- * Results:
- *      TRUE on success, FALSE on failure.
- *
- * Side effects:
- *      XXX.
- *
- *-----------------------------------------------------------------------------
- */
-
-gboolean
-GHITcloTrashFolderAction(RpcInData *data)       // IN/OUT:
-{
-   Bool ret = FALSE;
-
-   Debug("%s: Enter.\n", __FUNCTION__);
-
-   /* Check our arguments. */
-   ASSERT(data);
-   if (!data) {
-      return FALSE;
-   }
-
-   ASSERT(data->name);
-   ASSERT(data->args);
-
-   if (!data->name || !data->args) {
-      Debug("%s: Invalid arguments.\n", __FUNCTION__);
-      return RPCIN_SETRETVALS(data, "Invalid arguments.", FALSE);
-   }
-
-   Debug("%s: Got RPC, name: \"%s\", argument length: %"FMTSZ"u.\n",
-         __FUNCTION__, data->name, data->argsSize);
-
-   /*
-    * XXX: Trash/Recycle Folder support is not used by the host at this time
-    * so we can ignore this RPC.
-    */
-
-   /*
-    * We don't have any out parameters, so we write empty values into the
-    * result fields of the RpcInData structure.
-    */
-   RPCIN_SETRETVALS(data, "", FALSE);
-
-   /* Set our return value and return to the caller. */
-   ret = TRUE;
-
-   Debug("%s: Exit.\n", __FUNCTION__);
-   return ret;
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * GHISendTrashFolderStateRPC --
- *
- *      Send the ghi.guest.trashFolder.state RPC to the host. This function is
- *      used by the platform-specific GHI backends to notify the host when the
- *      state of the Trash (aka Recycle Bin) folder changes. Currently, the only
- *      information sent using this RPC is the empty/non-empty state of the
- *      Trash folder.
- *
- * Results:
- *      TRUE if RPC was sent successfully, FALSE otherwise.
- *
- * Side effects:
- *      XXX.
- *
- *-----------------------------------------------------------------------------
- */
-
-Bool
-GHISendTrashFolderStateRPC(Bool empty)  // IN: TRUE if the trash folder is empty.
-{
-   Bool ret = FALSE;
-   DynBuf outBuf;
-
-   GHITrashFolderStateV1 *v1ptr = NULL;
-   GHITrashFolderState msg;
-   XDR xdrs;
-
-   Debug("%s: Enter.\n", __FUNCTION__);
-
-   memset(&msg, 0, sizeof msg);
-   DynBuf_Init(&outBuf);
-
-   /* Create the GHITrashFolderStateV1 structure to send. */
-   v1ptr = (GHITrashFolderStateV1*) Util_SafeMalloc(sizeof *v1ptr);
-   if (NULL == v1ptr) {
-      Debug("%s: Failed to allocate trash folder state buffer.\n", __FUNCTION__);
-      goto exit;
-   }
-
-   v1ptr->empty = empty;
-
-   /* Create the GHITrashFolderState structure to send. */
-   msg.ver = GHI_TRASH_FOLDER_STATE_V1;
-   msg.GHITrashFolderState_u.trashFolderStateV1 = v1ptr;
-   v1ptr = NULL; /* This will be freed by VMX_XDR_FREE(). */
-
-   /* Create a new DynXdr structure for this message. */
-   ASSERT_MEM_ALLOC(DynXdr_Create(&xdrs));
-
-   /* Encode the structure into our DynXdr. */
-   if (!xdr_GHITrashFolderState(&xdrs, &msg)) {
-      Debug("%s: Failed to encode XDR data.\n", __FUNCTION__);
-      goto exit;
-   }
-
-   /* Append our RPC name and a space to the DynBuf. */
-   if (!DynBuf_Append(&outBuf,
-                      GHI_RPC_TRASH_FOLDER_STATE,
-                      strlen(GHI_RPC_TRASH_FOLDER_STATE))) {
-      Debug("%s: Failed to append RPC name to DynBuf.\n", __FUNCTION__);
-      goto exit;
-   }
-
-   if (!DynBuf_Append(&outBuf, " ", 1)) {
-      Debug("%s: Failed to append space to DynBuf.\n", __FUNCTION__);
-      goto exit;
-   }
-
-   /* Append the XDR serialized data to the DynBuf. */
-   if (!DynBuf_Append(&outBuf, DynXdr_Get(&xdrs), xdr_getpos(&xdrs)) )
-   {
-      Debug("%s: Failed to append XDR serialized data to DynBuf.\n",
-            __FUNCTION__);
-      goto exit;
-   }
-
-   if (!RpcOut_SendOneRaw(DynBuf_Get(&outBuf),
-                          DynBuf_GetSize(&outBuf),
-                          NULL,
-                          NULL)) {
-      Debug("%s: Failed to send RPC to host!\n", __FUNCTION__);
-      goto exit;
-   }
-   ret = TRUE;
-
-exit:
-   DynBuf_Destroy(&outBuf);
-   DynXdr_Destroy(&xdrs, TRUE);
-   VMX_XDR_FREE(xdr_GHITrashFolderState, &msg);
-   Debug("%s: Exit.\n", __FUNCTION__);
-   return ret;
-}
-
-
-/*
- *-----------------------------------------------------------------------------
- *
- * GHITcloTrashFolderGetIcon --
- *
- *      Return the icon for the Trash folder (aka Recycle Bin) to the host.
- *
- * Results:
- *      TRUE on success, FALSE on failure.
- *
- * Side effects:
- *      XXX.
- *
- *-----------------------------------------------------------------------------
- */
-
-gboolean
-GHITcloTrashFolderGetIcon(RpcInData *data) // IN/OUT:
-{
-   Bool ret = FALSE;
-
-   Debug("%s: Enter.\n", __FUNCTION__);
-
-   /* Check our arguments. */
-   ASSERT(data);
-   ASSERT(data->name);
-
-   if (!(data && data->name)) {
-      Debug("%s: Invalid arguments.\n", __FUNCTION__);
-      goto exit;
-   }
-
-   Debug("%s: Got RPC, name: \"%s\", argument length: %"FMTSZ"u.\n",
-         __FUNCTION__, data->name, data->argsSize);
-
-
-   /*
-    * XXX: Trash/Recycle Folder support is not used by the host at this time
-    * so we can ignore this RPC.
-    */
-
-   /*
-    * Write the result into the RPC out parameters.
-    */
-   RPCIN_SETRETVALS(data, "", FALSE);
-
-   ret = TRUE;
-
-exit:
-   Debug("%s: Exit.\n", __FUNCTION__);
-   return ret;
-}
-
-
 /*
  *-----------------------------------------------------------------------------
  *
