@@ -288,6 +288,8 @@ UnityPlatformInit(UnityWindowTracker *tracker,                            // IN
    INIT_ATOM(WM_STATE);
    INIT_ATOM(WM_TRANSIENT_FOR);
    INIT_ATOM(WM_WINDOW_ROLE);
+   INIT_ATOM(COMPOUND_TEXT);
+   INIT_ATOM(UTF8_STRING);
 
 #  undef INIT_ATOM
 
@@ -1244,7 +1246,6 @@ UnityX11HandleEvents(gpointer data) // IN
    ASSERT(up);
    ASSERT(up->isRunning);
 
-   Debug("Starting unity event handling\n");
    while (XEventsQueued(up->display, QueuedAfterFlush)) {
       /*
        * This outer loop is here to make sure we really process all available events
@@ -2291,6 +2292,13 @@ UnityPlatformSetDesktopWorkAreas(UnityPlatform *up,     // IN
       screenInfo->height = rootHeight;
    }
 
+   if ((uint32)numScreens != numWorkAreas) {
+      Warning("Mismatch between host-specified work areas and available "
+              "screens.  Request dropped.\n");
+      free(screenInfo);
+      return FALSE;
+   }
+
    /*
     * New and improved wild'n'crazy scheme to map the host's work area
     * coordinates to a collection of struts.
