@@ -65,13 +65,14 @@
 #include "err.h"
 #include "hostinfo.h"
 #include "guest_os.h"
+#include "guest_msg_def.h"
 #include "conf.h"
 #include "vixCommands.h"
 #include "foundryToolsDaemon.h"
 #include "printer.h"
 #include "base64.h"
 #include "syncDriver.h"
-#include "hgfsServer.h"
+#include "hgfsServerManager.h"
 #include "hgfs.h"
 #include "system.h"
 #include "codeset.h"
@@ -88,7 +89,6 @@
 #include "vixTools.h"
 #include "vixOpenSource.h"
 
-#define GUESTMSG_MAX_IN_SIZE (64 * 1024) /* vmx/main/guest_msg.c */
 #define MAX64_DECIMAL_DIGITS 20          /* 2^64 = 18,446,744,073,709,551,616 */
 
 #if defined(linux) || defined(_WIN32)
@@ -109,20 +109,22 @@ static Bool ToolsDaemonSyncDriverThawCallback(void *clientData);
 static char *ToolsDaemonTcloGetQuotedString(const char *args,
                                             const char **endOfArg);
   
-static char * ToolsDaemonTcloGetEncodedQuotedString(const char *args,
-                                                    const char **endOfArg);
+static VixError ToolsDaemonTcloGetEncodedQuotedString(const char *args,
+                                                      const char **endOfArg,
+                                                      char **result);
 
-RpcInRet ToolsDaemonTcloReceiveVixCommand(RpcInData *data);
+gboolean ToolsDaemonTcloReceiveVixCommand(RpcInData *data);
 
-RpcInRet ToolsDaemonHgfsImpersonated(RpcInData *data);
+static HgfsServerMgrData gFoundryHgfsBkdrConn;
+gboolean ToolsDaemonHgfsImpersonated(RpcInData *data);
 
 #if defined(linux) || defined(_WIN32)
-RpcInRet ToolsDaemonTcloSyncDriverFreeze(RpcInData *data);
+gboolean ToolsDaemonTcloSyncDriverFreeze(RpcInData *data);
 
-RpcInRet ToolsDaemonTcloSyncDriverThaw(RpcInData *data);
+gboolean ToolsDaemonTcloSyncDriverThaw(RpcInData *data);
 #endif
 
-RpcInRet ToolsDaemonTcloMountHGFS(RpcInData *data);
+gboolean ToolsDaemonTcloMountHGFS(RpcInData *data);
 
 void ToolsDaemonTcloReportProgramCompleted(const char *requestName,
                                            VixError err,
@@ -140,7 +142,6 @@ void ToolsDaemonTcloReportProgramCompleted(const char *requestName,
 static Bool thisProcessRunsAsRoot = FALSE;
 
 
-
 /*
  *-----------------------------------------------------------------------------
  *
@@ -158,7 +159,7 @@ static Bool thisProcessRunsAsRoot = FALSE;
  *-----------------------------------------------------------------------------
  */
 
-RpcInRet
+gboolean
 FoundryToolsDaemonRunProgram(RpcInData *data) // IN
 {
    VixError err = VIX_OK;
@@ -180,8 +181,19 @@ FoundryToolsDaemonRunProgram(RpcInData *data) // IN
     * may be NULL.
     */
    requestName = ToolsDaemonTcloGetQuotedString(data->args, &data->args);
-   commandLine = ToolsDaemonTcloGetEncodedQuotedString(data->args, &data->args);
-   commandLineArgs = ToolsDaemonTcloGetEncodedQuotedString(data->args, &data->args);
+
+   err = ToolsDaemonTcloGetEncodedQuotedString(data->args, &data->args,
+                                               &commandLine);
+   if (err != VIX_OK) {
+      goto abort;
+   }
+
+   err = ToolsDaemonTcloGetEncodedQuotedString(data->args, &data->args,
+                                               &commandLineArgs);
+   if (err != VIX_OK) {
+      goto abort;
+   }
+
    credentialTypeStr = ToolsDaemonTcloGetQuotedString(data->args, &data->args);
    obfuscatedNamePassword = ToolsDaemonTcloGetQuotedString(data->args, &data->args);
    directoryPath = ToolsDaemonTcloGetQuotedString(data->args, &data->args);
@@ -268,7 +280,7 @@ abort:
  *-----------------------------------------------------------------------------
  */
 
-RpcInRet
+gboolean
 FoundryToolsDaemonGetToolsProperties(RpcInData *data) // IN
 {
    VixError err = VIX_OK;
@@ -351,7 +363,7 @@ abort:
  *-----------------------------------------------------------------------------
  */
 
-RpcInRet
+gboolean
 ToolsDaemonTcloCheckUserAccount(RpcInData *data) // IN
 {
    VixError err = VIX_OK;
@@ -441,6 +453,27 @@ FoundryToolsDaemon_Initialize(ToolsAppCtx *ctx)
    }
 #endif
 
+   /* Register a straight through connection with the Hgfs server. */
+   HgfsServerManager_DataInit(&gFoundryHgfsBkdrConn,
+                              VIX_BACKDOORCOMMAND_SEND_HGFS_PACKET,
+                              NULL,    // rpc - no rpc registered
+                              NULL);   // rpc callback
+   HgfsServerManager_Register(&gFoundryHgfsBkdrConn);
+
+}
+
+
+/**
+ * Uninitializes internal state of the Foundry daemon.
+ *
+ * @param[in]  ctx      Application context.
+ */
+
+void
+FoundryToolsDaemon_Uninitialize(ToolsAppCtx *ctx)
+{
+   HgfsServerManager_Unregister(&gFoundryHgfsBkdrConn);
+   VixTools_Uninitialize();
 }
 
 
@@ -527,22 +560,27 @@ ToolsDaemonTcloGetQuotedString(const char *args,      // IN
  *-----------------------------------------------------------------------------
  */
 
-static char *
+static VixError
 ToolsDaemonTcloGetEncodedQuotedString(const char *args,      // IN
-                                      const char **endOfArg) // OUT
+                                      const char **endOfArg, // OUT
+                                      char **result)         // OUT
 {
+   VixError err = VIX_OK;
    char *rawResultStr = NULL;
    char *resultStr = NULL;
 
    rawResultStr = ToolsDaemonTcloGetQuotedString(args, endOfArg);
    if (NULL == rawResultStr) {
-      return(NULL);
+      goto abort;
    }
 
-   resultStr = VixMsg_DecodeString(rawResultStr);
-   free(rawResultStr);
+   err = VixMsg_DecodeString(rawResultStr, &resultStr);
 
-   return resultStr;
+abort:
+   free(rawResultStr);
+   *result = resultStr;
+
+   return err;
 }
 
 
@@ -564,7 +602,7 @@ ToolsDaemonTcloGetEncodedQuotedString(const char *args,      // IN
  */
 
 #if defined(linux) || defined(_WIN32)
-RpcInRet
+gboolean
 ToolsDaemonTcloSyncDriverFreeze(RpcInData *data)
 {
    static char resultBuffer[DEFAULT_RESULT_MSG_MAX_LENGTH];
@@ -710,7 +748,7 @@ exit:
  */
 
 #if defined(linux) || defined(_WIN32)
-RpcInRet
+gboolean
 ToolsDaemonTcloSyncDriverThaw(RpcInData *data) // IN
 {
    static char resultBuffer[DEFAULT_RESULT_MSG_MAX_LENGTH];
@@ -763,7 +801,7 @@ ToolsDaemonTcloSyncDriverThaw(RpcInData *data) // IN
  *-----------------------------------------------------------------------------
  */
 
-RpcInRet
+gboolean
 ToolsDaemonTcloMountHGFS(RpcInData *data) // IN
 {
    VixError err = VIX_OK;
@@ -862,11 +900,12 @@ ToolsDaemonTcloMountHGFS(RpcInData *data) // IN
  *-----------------------------------------------------------------------------
  */
 
-RpcInRet
+gboolean
 ToolsDaemonHgfsImpersonated(RpcInData *data) // IN
 {
    VixError err;
    size_t hgfsPacketSize = 0;
+   size_t hgfsReplySize = 0;
    const char *origArgs = data->args;
    Bool impersonatingVMWareUser = FALSE;
    char *credentialTypeStr = NULL;
@@ -944,7 +983,6 @@ ToolsDaemonHgfsImpersonated(RpcInData *data) // IN
                                                             &userToken);
       if (!impersonatingVMWareUser) {
          err = VIX_E_GUEST_USER_PERMISSIONS;
-         hgfsPacketSize = 0;
          goto abort;
       }
    }
@@ -953,9 +991,12 @@ ToolsDaemonHgfsImpersonated(RpcInData *data) // IN
     * Impersonation was okay, so let's give our packet to
     * the HGFS server and forward the reply packet back.
     */
-   HgfsServer_ProcessPacket(data->args,        // packet in buf
-                            hgfsReplyPacket,   // packet out buf
-                            &hgfsPacketSize);   // in/out size
+   hgfsReplySize = sizeof resultPacket - (hgfsReplyPacket - resultPacket);
+   HgfsServerManager_ProcessPacket(&gFoundryHgfsBkdrConn, // hgfs server connection
+                                   data->args,            // packet in buf
+                                   hgfsPacketSize,        // packet in size
+                                   hgfsReplyPacket,       // packet out buf
+                                   &hgfsReplySize);       // reply buf/data size
 
 abort:
    if (impersonatingVMWareUser) {
@@ -972,7 +1013,7 @@ abort:
    data->result = resultPacket;
    data->resultLen = STRLEN_OF_MAX_64_BIT_NUMBER_AS_STRING
                         + OTHER_TEXT_SIZE
-                        + hgfsPacketSize;
+                        + hgfsReplySize;
    
    /*
     * Render the foundry error codes into the buffer.
@@ -1086,7 +1127,7 @@ ToolsDaemonTcloReportProgramCompleted(const char *requestName,    // IN
  *-----------------------------------------------------------------------------
  */
 
-RpcInRet
+gboolean
 ToolsDaemonTcloReceiveVixCommand(RpcInData *data) // IN
 {
    VixError err = VIX_OK;
@@ -1100,7 +1141,8 @@ ToolsDaemonTcloReceiveVixCommand(RpcInData *data) // IN
    char *destPtr = NULL;
    int vixPrefixDataSize = (MAX64_DECIMAL_DIGITS * 2)
                              + (sizeof(' ') * 2)
-                             + sizeof('\0');
+                             + sizeof('\0')
+                             + sizeof(' ') * 10;   // for RPC header
 
    /*
     * Our temporary buffer will be the same size as what the
@@ -1153,18 +1195,6 @@ abort:
       err = VIX_E_OUT_OF_MEMORY;
    }
 
-   /*
-    * This should never happen since tcloBuffer is pretty huge.
-    * But, there is no harm in being paranoid and I don't want to
-    * find out the buffer is too small when we are halfway through
-    * formatting the response.
-    */
-   if ((32 + resultValueLength + 32) >= sizeof(tcloBuffer)) {
-      err = VIX_E_OUT_OF_MEMORY;
-      resultValueLength = 0;
-   }
-
-
    /*
     * All Foundry tools commands return results that start with a foundry error
     * and a guest-OS-specific error.
