@@ -37,6 +37,7 @@
 #include "conf.h"
 #include "rpcout.h"
 #include "str.h"
+#include "vmcheck.h"
 #include "vmtoolsd_version.h"
 #include "vmware/tools/utils.h"
 #include "vmware/tools/i18n.h"
@@ -60,19 +61,27 @@ ToolsCoreRunCommand(const gchar *option,
                     gpointer data,
                     GError **error)
 {
-   char *result = NULL;
-   Bool status = FALSE;
+#if defined(_WIN32)
+   VMTools_AttachConsole();
+#endif
+   if (VmCheck_IsVirtualWorld()) {
+      char *result = NULL;
+      Bool status = FALSE;
 
-   status = RpcOut_sendOne(&result, NULL, "%s", value);
+      status = RpcOut_sendOne(&result, NULL, "%s", value);
 
-   if (!status) {
-      g_printerr("%s\n", result ? result : "NULL");
-   } else {
-      g_print("%s\n", result);
+      if (!status) {
+         g_printerr("%s\n", result ? result : "NULL");
+      } else {
+         g_print("%s\n", result);
+      }
+
+      vm_free(result);
+      exit(status ? 0 : 1);
    }
-
-   vm_free(result);
-   exit(status ? 0 : 1);
+   g_printerr("%s\n",
+              SU_(cmdline.rpcerror, "Unable to send command to VMware hypervisor."));
+   exit(1);
 }
 
 
@@ -100,7 +109,9 @@ ToolsCoreIgnoreArg(const gchar *option,
 
 
 /**
- * Signals a specific event in a running service instance.
+ * Signals a specific event in a running service instance. Since this function
+ * doesn't know whether the service is running through the SCM, it first tries
+ * to open a local event, and if that fails, tries a global event.
  *
  * @param[in]  svcname     Name of the service to be signaled.
  * @param[in]  evtFmt      Format string for the event name. It should expect
@@ -120,7 +131,19 @@ ToolsCoreSignalEvent(const gchar *svcname,
 
    ASSERT(svcname != NULL);
 
-   evt = Str_Aswprintf(NULL, evtFmt, svcname);
+   evt = Str_Aswprintf(NULL, evtFmt, L"Local", svcname);
+   if (evt == NULL) {
+      g_printerr("Out of memory!\n");
+      goto exit;
+   }
+
+   h = OpenEvent(EVENT_MODIFY_STATE, FALSE, evt);
+   if (h != NULL) {
+      goto dispatch;
+   }
+
+   vm_free(evt);
+   evt = Str_Aswprintf(NULL, evtFmt, L"Global", svcname);
    if (evt == NULL) {
       g_printerr("Out of memory!\n");
       goto exit;
@@ -131,6 +154,7 @@ ToolsCoreSignalEvent(const gchar *svcname,
       goto error;
    }
 
+dispatch:
    if (!SetEvent(h)) {
       goto error;
    }
@@ -255,7 +279,7 @@ ToolsCore_ParseCommandLine(ToolsServiceState *state,
 #endif
 
    context = g_option_context_new(NULL);
-#if GLIB_MAJOR_VERSION >= 2 && GLIB_MINOR_VERSION >= 12
+#if GLIB_CHECK_VERSION(2, 12, 0)
    g_option_context_set_summary(context, N_("Runs the VMware Tools daemon."));
 #endif
    g_option_context_add_main_entries(context, clOptions, NULL);
