@@ -23,9 +23,11 @@
  */
 
 #include <string.h>
+#include "toolsCoreInt.h"
+
 #include "vm_assert.h"
 #include "guestApp.h"
-#include "toolsCoreInt.h"
+#include "serviceObj.h"
 #include "util.h"
 #include "vmware/tools/i18n.h"
 #include "vmware/tools/utils.h"
@@ -68,10 +70,14 @@ ToolsCoreDumpAppInfo(ToolsServiceState *state,
       if (preg->prov->dumpState != NULL) {
          preg->prov->dumpState(&state->ctx, preg->prov, reg);
       } else {
-         g_message("      App type %u (no provider info).\n", type);
+         ToolsCore_LogState(TOOLS_STATE_LOG_PLUGIN,
+                            "App type %u (no provider info).\n",
+                            type);
       }
     } else {
-      g_message("      App type %u (no provider).\n", type);
+      ToolsCore_LogState(TOOLS_STATE_LOG_PLUGIN,
+                         "App type %u (no provider).\n",
+                         type);
    }
    return TRUE;
 }
@@ -88,10 +94,30 @@ static void
 ToolsCoreDumpPluginInfo(ToolsServiceState *state,
                         ToolsPluginData *plugin)
 {
-   g_message("   Plugin: %s\n", plugin->name);
+   ToolsCore_LogState(TOOLS_STATE_LOG_CONTAINER, "Plugin: %s\n", plugin->name);
 
    if (plugin->regs == NULL) {
-      g_message("      No registrations.\n");
+      ToolsCore_LogState(TOOLS_STATE_LOG_PLUGIN, "No registrations.\n");
+   }
+}
+
+
+/**
+ * State dump callback for service properties.
+ *
+ * @param[in]  ctx   The application context.
+ * @param[in]  prov  Unused.
+ * @param[in]  reg   The application registration data.
+ */
+
+static void
+ToolsCoreDumpProperty(ToolsAppCtx *ctx,
+                      ToolsAppProvider *prov,
+                      gpointer reg)
+{
+   if (reg != NULL) {
+      ToolsCore_LogState(TOOLS_STATE_LOG_PLUGIN, "Service property: %s.\n",
+                         ((ToolsServiceProperty *)reg)->name);
    }
 }
 
@@ -111,7 +137,7 @@ ToolsCoreDumpRPC(ToolsAppCtx *ctx,
 {
    if (reg != NULL) {
       RpcChannelCallback *cb = reg;
-      g_message("      RPC callback: %s\n", cb->name);
+      ToolsCore_LogState(TOOLS_STATE_LOG_PLUGIN, "RPC callback: %s\n", cb->name);
    }
 }
 
@@ -131,7 +157,7 @@ ToolsCoreDumpSignal(ToolsAppCtx *ctx,
 {
    if (reg != NULL) {
       ToolsPluginSignalCb *sig = reg;
-      g_message("      Signal callback: %s\n", sig->signame);
+      ToolsCore_LogState(TOOLS_STATE_LOG_PLUGIN, "Signal callback: %s\n", sig->signame);
    }
 }
 
@@ -189,7 +215,7 @@ ToolsCoreRegisterApp(ToolsServiceState *state,
       preg->state = TOOLS_PROVIDER_ACTIVE;
    }
 
-   if (!preg->prov->registerApp(&state->ctx, preg->prov, reg)) {
+   if (!preg->prov->registerApp(&state->ctx, preg->prov, plugin, reg)) {
       g_warning("Failed registration of app type %d (%s) from plugin %s.",
                 type, preg->prov->name, plugin->name);
       goto exit;
@@ -283,6 +309,7 @@ ToolsCoreForEachPlugin(ToolsServiceState *state,
 
       for (j = 0; j < regs->len; j++) {
          guint k;
+         guint pregIdx;
          ToolsAppReg *reg = &g_array_index(regs, ToolsAppReg, j);
          ToolsAppProviderReg *preg = NULL;
 
@@ -293,6 +320,7 @@ ToolsCoreForEachPlugin(ToolsServiceState *state,
                                                       k);
             if (tmp->prov->regType == reg->type) {
                preg = tmp;
+               pregIdx = k;
                break;
             }
          }
@@ -314,18 +342,47 @@ ToolsCoreForEachPlugin(ToolsServiceState *state,
                j = regs->len;
                break;
             }
+
+            /*
+             * The registration callback may have modified the provider array,
+             * so we need to re-read the provider pointer.
+             */
+            preg = &g_array_index(state->providers, ToolsAppProviderReg, pregIdx);
          }
       }
    }
 }
 
 
+/**
+ * Callback to register service properties.
+ *
+ * @param[in]  ctx      The application context.
+ * @param[in]  prov     Unused.
+ * @param[in]  plugin   Unused.
+ * @param[in]  reg      The property registration data.
+ *
+ * @return TRUE
+ */
+
+static gboolean
+ToolsCoreRegisterProperty(ToolsAppCtx *ctx,
+                          ToolsAppProvider *prov,
+                          ToolsPluginData *plugin,
+                          gpointer reg)
+{
+   ToolsCoreService_RegisterProperty(ctx->serviceObj, reg);
+   return TRUE;
+}
+
+
 /**
  * Registration callback for GuestRPC applications.
  *
- * @param[in]  ctx   The application context.
- * @param[in]  prov  Unused.
- * @param[in]  reg   The application registration data.
+ * @param[in]  ctx      The application context.
+ * @param[in]  prov     Unused.
+ * @param[in]  plugin   Unused.
+ * @param[in]  reg      The application registration data.
  *
  * @return TRUE.
  */
@@ -333,6 +390,7 @@ ToolsCoreForEachPlugin(ToolsServiceState *state,
 static gboolean
 ToolsCoreRegisterRPC(ToolsAppCtx *ctx,
                      ToolsAppProvider *prov,
+                     ToolsPluginData *plugin,
                      gpointer reg)
 {
    RpcChannel_RegisterCallback(ctx->rpc, reg);
@@ -343,9 +401,10 @@ ToolsCoreRegisterRPC(ToolsAppCtx *ctx,
 /**
  * Registration callback for signal connections.
  *
- * @param[in]  ctx   The application context.
- * @param[in]  prov  Unused.
- * @param[in]  reg   The application registration data.
+ * @param[in]  ctx      The application context.
+ * @param[in]  prov     Unused.
+ * @param[in]  plugin   Unused.
+ * @param[in]  reg      The application registration data.
  *
  * @return TRUE if the signal exists.
  */
@@ -353,16 +412,27 @@ ToolsCoreRegisterRPC(ToolsAppCtx *ctx,
 static gboolean
 ToolsCoreRegisterSignal(ToolsAppCtx *ctx,
                         ToolsAppProvider *prov,
+                        ToolsPluginData *plugin,
                         gpointer reg)
 {
+   gboolean valid;
+   guint sigId;
+   GQuark sigDetail;
    ToolsPluginSignalCb *sig = reg;
-   if (g_signal_lookup(sig->signame, G_OBJECT_TYPE(ctx->serviceObj)) != 0) {
+
+   valid = g_signal_parse_name(sig->signame,
+                               G_OBJECT_TYPE(ctx->serviceObj),
+                               &sigId,
+                               &sigDetail,
+                               FALSE);
+   if (valid) {
       g_signal_connect(ctx->serviceObj,
                        sig->signame,
                        sig->callback,
                        sig->clientData);
       return TRUE;
    }
+
    return FALSE;
 }
 
@@ -537,6 +607,7 @@ ToolsCore_DumpPluginInfo(ToolsServiceState *state)
 gboolean
 ToolsCore_LoadPlugins(ToolsServiceState *state)
 {
+   gboolean pluginDirExists;
    gboolean ret = FALSE;
    gchar *pluginRoot;
 
@@ -595,7 +666,10 @@ ToolsCore_LoadPlugins(ToolsServiceState *state)
       goto exit;
    }
 
-   /* Load the container-specific plugins. */
+   /*
+    * Load the container-specific plugins. Ignore if the plugin directory
+    * doesn't exist when running in debug mode.
+    */
 
    if (state->pluginPath == NULL) {
       state->pluginPath = g_strdup_printf("%s%s%c%s",
@@ -605,15 +679,30 @@ ToolsCore_LoadPlugins(ToolsServiceState *state)
                                           state->name);
    }
 
-   if (!g_file_test(state->pluginPath, G_FILE_TEST_IS_DIR)) {
+   pluginDirExists = g_file_test(state->pluginPath, G_FILE_TEST_IS_DIR);
+   if (state->debugPlugin == NULL && !pluginDirExists) {
       g_warning("Plugin path is not a directory: %s\n", state->pluginPath);
       goto exit;
    }
 
-   if (!ToolsCoreLoadDirectory(&state->ctx, state->pluginPath, state->plugins)) {
+   if (pluginDirExists &&
+       !ToolsCoreLoadDirectory(&state->ctx, state->pluginPath, state->plugins)) {
       goto exit;
    }
 
+   /*
+    * If there is a debug plugin, see if it exports standard plugin registration
+    * data too.
+    */
+   if (state->debugData != NULL && state->debugData->debugPlugin->plugin != NULL) {
+      ToolsPluginData *data = state->debugData->debugPlugin->plugin;
+      ToolsPlugin *plugin = g_malloc(sizeof *plugin);
+      plugin->module = NULL;
+      plugin->data = data;
+      VMTools_BindTextDomain(data->name, NULL, NULL);
+      g_ptr_array_add(state->plugins, plugin);
+   }
+
    ret = TRUE;
 
 exit:
@@ -679,6 +768,17 @@ ToolsCore_RegisterPlugins(ToolsServiceState *state)
    fakeReg.state = TOOLS_PROVIDER_ACTIVE;
    g_array_append_val(state->providers, fakeReg);
 
+   fakeProv = g_malloc0(sizeof *fakeProv);
+   fakeProv->regType = TOOLS_SVC_PROPERTY;
+   fakeProv->regSize = sizeof (ToolsServiceProperty);
+   fakeProv->name = "Service Properties";
+   fakeProv->registerApp = ToolsCoreRegisterProperty;
+   fakeProv->dumpState = ToolsCoreDumpProperty;
+
+   fakeReg.prov = fakeProv;
+   fakeReg.state = TOOLS_PROVIDER_ACTIVE;
+   g_array_append_val(state->providers, fakeReg);
+
 
    /*
     * First app providers need to be identified, so that we know that they're
@@ -711,47 +811,50 @@ void
 ToolsCore_UnloadPlugins(ToolsServiceState *state)
 {
    guint i;
-   GArray *pcaps = NULL;
 
    if (state->plugins == NULL) {
       return;
    }
 
-   g_signal_emit_by_name(state->ctx.serviceObj,
-                         TOOLS_CORE_SIG_CAPABILITIES,
-                         &state->ctx,
-                         FALSE,
-                         &pcaps);
+   if (state->capsRegistered) {
+      GArray *pcaps = NULL;
+      g_signal_emit_by_name(state->ctx.serviceObj,
+                            TOOLS_CORE_SIG_CAPABILITIES,
+                            &state->ctx,
+                            FALSE,
+                            &pcaps);
 
-   if (pcaps != NULL) {
-      if (state->ctx.rpc) {
-         ToolsCore_SetCapabilities(state->ctx.rpc, pcaps, FALSE);
+      if (pcaps != NULL) {
+         if (state->ctx.rpc) {
+            ToolsCore_SetCapabilities(state->ctx.rpc, pcaps, FALSE);
+         }
+         g_array_free(pcaps, TRUE);
       }
-      g_array_free(pcaps, TRUE);
    }
 
-   g_signal_emit_by_name(state->ctx.serviceObj, TOOLS_CORE_SIG_SHUTDOWN, &state->ctx);
-
    /*
-    * Stop all app providers, and free the memory we allocated for the two
+    * Stop all app providers, and free the memory we allocated for the
     * internal app providers.
     */
-   for (i = 0; i < state->providers->len; i++) {
+   for (i = 0; state->providers != NULL && i < state->providers->len; i++) {
        ToolsAppProviderReg *preg = &g_array_index(state->providers,
                                                   ToolsAppProviderReg,
                                                   i);
 
-      if (preg->prov->shutdown != NULL) {
+      if (preg->prov->shutdown != NULL && preg->state == TOOLS_PROVIDER_ACTIVE) {
          preg->prov->shutdown(&state->ctx, preg->prov);
       }
 
       if (preg->prov->regType == TOOLS_APP_GUESTRPC ||
           preg->prov->regType == TOOLS_APP_SIGNALS ||
-          preg->prov->regType == TOOLS_APP_PROVIDER) {
+          preg->prov->regType == TOOLS_APP_PROVIDER ||
+          preg->prov->regType == TOOLS_SVC_PROPERTY) {
          g_free(preg->prov);
       }
    }
 
+   g_signal_emit_by_name(state->ctx.serviceObj, TOOLS_CORE_SIG_SHUTDOWN, &state->ctx);
+
    while (state->plugins->len > 0) {
       ToolsPlugin *plugin = g_ptr_array_index(state->plugins, state->plugins->len - 1);
       GArray *regs = (plugin->data != NULL) ? plugin->data->regs : NULL;
@@ -770,12 +873,16 @@ ToolsCore_UnloadPlugins(ToolsServiceState *state)
       }
 
       g_ptr_array_remove_index(state->plugins, state->plugins->len - 1);
-      g_module_close(plugin->module);
+      if (plugin->module != NULL) {
+         g_module_close(plugin->module);
+      }
       g_free(plugin);
    }
 
-   g_array_free(state->providers, TRUE);
-   state->providers = NULL;
+   if (state->providers != NULL) {
+      g_array_free(state->providers, TRUE);
+      state->providers = NULL;
+   }
 
    g_ptr_array_free(state->plugins, TRUE);
    state->plugins = NULL;
