@@ -32,6 +32,7 @@
 #include "toolboxCmdInt.h"
 #include "toolboxcmd_version.h"
 #include "system.h"
+#include "unicode.h"
 #include "vmware/tools/guestrpc.h"
 #include "vmware/tools/i18n.h"
 #include "vmware/tools/utils.h"
@@ -61,11 +62,6 @@ const typedef struct CmdTable {
 } CmdTable;
 
 
-/*
- * Sadly, our home-brewed implementation of getopt() doesn't come with an
- * implementation of getopt_long().
- */
-#ifndef _WIN32
 static struct option long_options[] = {
    /* quiet sets a flag */
    { "quiet", no_argument, 0, 'q' },
@@ -74,7 +70,6 @@ static struct option long_options[] = {
    { "help", no_argument, 0, 'h' },
    { "version", no_argument, 0, 'v' },
    { 0, 0, 0, 0 } };
-#endif
 
 static gboolean gQuiet = FALSE;
 static const char *options = "hqv";
@@ -104,7 +99,7 @@ static CmdTable commands[] = {
    { "stat",      Stat_Command,     TRUE,    FALSE,   Stat_Help},
    { "device",    Device_Command,   TRUE,    FALSE,   Device_Help},
 #if (defined(_WIN32) || defined(linux)) && !defined(OPEN_VM_TOOLS)
-   { "upgrade",   Upgrade_Command,  TRUE,    FALSE,   Upgrade_Help},
+   { "upgrade",   Upgrade_Command,  TRUE,    TRUE,   Upgrade_Help},
 #endif
    { "help",      HelpCommand,      FALSE,   FALSE,   ToolboxCmdHelp},
 };
@@ -399,8 +394,13 @@ ParseCommand(char **argv, // IN: Command line arguments
  */
 
 int
+#if defined(_WIN32)
+wmain(int argc,         // IN: length of command line arguments
+      wchar_t **wargv)  // IN: Command line arguments
+#else
 main(int argc,    // IN: length of command line arguments
      char **argv) // IN: Command line arguments
+#endif
 {
    Bool show_help = FALSE;
    Bool show_version = FALSE;
@@ -408,6 +408,13 @@ main(int argc,    // IN: length of command line arguments
    int c;
    int retval;
 
+#if defined(_WIN32)
+   char **argv;
+   Unicode_InitW(argc, wargv, NULL, &argv, NULL);
+#else
+   Unicode_Init(argc, &argv, NULL);
+#endif
+
    setlocale(LC_ALL, "");
    VMTools_ConfigLogging("toolboxcmd", NULL, FALSE, FALSE);
    VMTools_BindTextDomain(VMW_TEXT_DOMAIN, NULL, NULL);
@@ -427,11 +434,7 @@ main(int argc,    // IN: length of command line arguments
    while (1) {
       int option_index = 0;
 
-#ifdef _WIN32
-      c = getopt(argc, argv, options);
-#else
       c = getopt_long(argc, argv, options, long_options, &option_index);
-#endif
 
       /* Detect the end of the options. */
       if (c == -1) {
@@ -463,7 +466,7 @@ main(int argc,    // IN: length of command line arguments
    }
 
    if (show_version) {
-      printf("%s (%s)\n", TOOLBOXCMD_VERSION_STRING, BUILD_NUMBER);
+      g_print("%s (%s)\n", TOOLBOXCMD_VERSION_STRING, BUILD_NUMBER);
    } else if (show_help) {
       ToolboxCmdHelp(argv[0], "help");
    } else {
